function rhoch4sw = density_ch4sw(press,temp,sal,mch4) 

% DENSITY_CH4SW -  calculates the density of a solution of CH4 in seawater
%
%       Usage: rho = density_ch4sw(p,T,S,m)
%
%       Input parameters (scalars or vectors): 
%           p       - pressure [MPa]     (0.1 < p < 200), 
%           T       - temperature [degC] (0 < T < 250),
%           S       - practical salinity (0 < S < 350)
%           m       - molality of dissolved CH4 [mol/kg]
%                         (default value is saturation concentration) 
%                         (input as scalar or p x T x S matrix)
%
%       Output parameter: 
%           rho     - density of CH4-rich seawater [kg/m^3]
%
%       References:
%       Duan et al. (2006) GCA 70, 3369-3386.
%
%       Comments:
%       The validity range for salinity is not specified and 
%       the limit for NaCl is taken instead.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       September 2011: input format for mch4 added to header
%       November 2012: syntax changed to Matlab2012 style
%       February 2013: function call for seasalt updated
%       April 2013: simplified + more accurate mass calculation for seawater
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('DENSITY_CH4SW:Input','DENSITY_CH4SW: Missing input parameter! Please provide p,T and S!')
else


if nargin == 3 % if no value for mch4 is provided, use saturation concentration
    
% call external function to calculate molality of dissolved CH4 at saturation  
[mch4_sw_sat] = solu_ch4(press,temp,sal); % dissolved CH4 at saturation [mol/kg]
mch4_sw = mch4_sw_sat;

else
    mch4_sw = mch4;
end


%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>200) && ~any(temp<0 | temp>250) ...
        && ~any( sal<0 | sal>350)
else
    bad = find(press<0.1 | press > 200);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>250);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>350);
    sal(bad)=NaN*ones(size(bad));
    warning('DENSITY_CH4SW:OutOfParameterRange','DENSITY_CH4SW: Input values out of range!')
    warning('DENSITY_CH4SW:OutOfParameterRange','Equations valid for p=0.1-200 MPa, T=0-250 degC, S=0-350.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------
[P,T,S] = ndgrid(press.*10,temp+273.15,sal); % pressure in bar, temperature in K

R = 83.14472; % gas constant [bar cm^3/mol/K]

Mch4 = 16.042;  % Molar mass of CH4[g/mol];

%------------------------------------------------------------------------
% Calculation of seasalt composition and properties
%------------------------------------------------------------------------

% call external funtion to calculate density and chlorinity of seawater for
% calculation of salt component molalities
[rsw] = density_sw(press,temp,sal); % rsw [kg/m^3], chl [g/kg]

% call external function to calculate molarities and molalities of salt
% components
[mNa,mMg,mCa,mK] = seasalt(S);

% call exteral function EOS_seawater for a more accurate calculation of the
% seawater density

rho_sw = rsw./1000; % [g/cm^3]

%------------------------------------------------------------------------
% Calculation of mass of solution per 1000 g water [g/(1000g water)]
%------------------------------------------------------------------------
Sa = S*1.005; % absolute salinity
m_sol = 1000 + Mch4 .* mch4_sw + Sa.*(1000./(1000-Sa)); % [g per 1000g water content]

%------------------------------------------------------------------------
% Calculation of the solution volume V_sol per 1000 g water content 
%------------------------------------------------------------------------

% call routine 'molvolch4' to calculate the partial molar volume of CH4
% dissolved in seawater [cm^3/mol]:
Vch4 = molvolch4(P,T,R,mNa, mMg, mCa, mK);

V_sol = (1000 + Sa.*(1000./(1000-Sa)))./rho_sw + mch4_sw.* Vch4; 
%[cm^3 per 1000 g water content]

%------------------------------------------------------------------------
% Calculation of the density of the methane-seawater-solution
%------------------------------------------------------------------------

rhoch4sw = m_sol./V_sol; %[g/cm^3]
rhoch4sw = rhoch4sw .* 1000; % [kg/m^3]

end

% ------------------------------------------------------------

function Vch4 = molvolch4(P,T,R,mNa, mMg, mCa, mK)
% This function calculates the partial molar volume of CH4 dissolved in 
% seawater 


% calculation of the derivative of mu/RT with respect to pressure from a
% multiparametric equation
c6 = 0.0076985890;
c7 = -0.0000050253331;
c8 = -3.0092013;
c9 = 484.68502;

duch4dp = c6 + c7.*T + c8./T + c9./T.^2;

% calculation of derivative of the Pitzer coefficient with respect to 
% pressure from amultiparametric equation

c6_l = 0.000044105635;
c10_l = -0.000000000046797718;

dlNadp = c6_l + 2*c10_l.*P.*T;

% calculation of molar volume of dissolved CH4: 
% V/RT = d(mu/RT)/dP + sum(2*m_cation*dlambda_cation/dP),
% the Pitzer coefficient for anions is defined as zero

Vch4 = R.*T.*(duch4dp + 2.* dlNadp.*(mNa + mK + 2.*mCa + 2.* mMg));%[cm^3/mol]

% ------------------------------------------------------------
