function RSW = density_sw(press,temp,sal,choice)

% DENSITY_SW -  calculates the density of seawater or saline brines
%      The user can choose 5 formalisms for different pTS ranges.
%      If no choice is specified, the formalism is chosen depending on the
%      pTS input.
%
%      Usage: rho = density_sw(p,T,S,c)
%
%      Input parameters (scalars or vectors): 
%          p     - pressure [MPa], 
%          T     - temperature [degC] 
%          S     - practical salinity 
%          c     - user choice of algorithm (default is 'EOS')
%                  'EOS':      IAPSW seawater EOS (eos_sw)
%                  'UNESCO':   Unesco formulae (density_swUnesco)
%                  'TEMP':     for high T (density_swSun)
%                  'BRINE':    for high p,T,S (density_swSpivey)
%                  'DRIESNER': for large p,T,S range (eos_swDriesner)
%
%      Output parameters: 
%          rho   - density of seawater [kg/m^3]
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('DENSITY_SW:Input','DENSITY_SW: Missing input parameter! Please provide p, T and S!')
else

%------------------------------------------------------------------------
% Choose algorithm based on parameter range
%------------------------------------------------------------------------
if nargin == 3
    if ~any(press<0 | press>100) && ~any(temp< -12 | temp>80) ...
        && ~any(sal < 0 | sal>120)

        choice = 'EOS'; % p=0-100 MPa, T=-12-80 degC, S=0-120

    elseif ~any(press<0.1 | press>100) && ~any( temp<0 | temp>374) ...
        && ~any( sal<0 | sal>80)

        choice = 'TEMP'; % p=0.1-100 MPa, T=0-374 degC, S=0-80

    elseif ~any(press<0.1 | press>200) && ~any( temp<0 | temp>275) ...
        && ~any( sal<0 | sal>275)
    
        choice = 'BRINE'; % p=0.1-200 MPa, T=0-275 degC, S=0-275 M
        
     elseif ~any(press<0.1 | press>167.5) && ~any( temp<0 | temp>600) 
    
        choice = 'DRIESNER'; % p=0.1-167.5 MPa, T=0-600 degC, S = 0 ~ saturation

    else
        warning('DENSITY_SW:OutOfParameterRange','DENSITY_SW: Input values out of range!')
        warning('DENSITY_SW:OutOfParameterRange','EOS_SW valid for p=0-100 MPa, T=-12-30 degC, S=0-120.')
        warning('DENSITY_SW:OutOfParameterRange','Extended Equations valid for p=0.1-167.5 (200) MPa, T=0-600 degC, S=0-saturation.')
    end
end

%------------------------------------------------------------------------
% Algorithm choices
%------------------------------------------------------------------------
switch choice
    case 'EOS'
        RSW = eos_sw(press,temp,sal);
    case 'UNESCO' % only used if specified in input
        RSW = density_swUnesco(press,temp,sal);
    case 'BRINE'
        RSW = density_swSpivey(press,temp,sal);
    case 'TEMP'
        RSW = density_swSun(press,temp,sal);
    case 'DRIESNER'
        RSW = eos_swDriesner(press,temp,sal);
    otherwise % use EOS_SW as default
        RSW = eos_sw(press,temp,sal);
end

end
