function [rho] = density_swSpivey(press,temp,sal)

% DENSITY_SWSPIVEY -  calculates the density of seawater
%       at elevated temperatures and NaCl concentrations
%
%       Usage: rho = density_swSpivey(p,T,S)
%
%       Input parameters (scalars or vectors): 
%           p       -  pressure [MPa]     (0 < p < 200) 
%           T       -  temperature [degC] (0 < T < 275) 
%           S       -  practical salinity (0 < S < 216) 
%   
%       Output parameter:
%          rho      - density of seawater [kg/m^3]
%
%       References:
%       Spivey & McCain (2004) J.Can.Petrol.Tech 43, 52-61.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
%   Modification history: 
%       December 2011: Name changed from density_swhighpts to density_swSpivey
%            input changed from mol/kg NaCl to salinity S
%------------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('DENSITY_SWSPIVEY:Input','DENSITY_SWSPIVEY: Missing input parameter! Please provide p,T and S!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>200) && ~any(temp<(0) | temp>275) && ~any(sal<(0) | sal>216)
        
else
    bad = find(press<0.1 | press > 200);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<(0) | temp>275);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<(0) | sal>216);
    sal(bad)=NaN*ones(size(bad));

 warning('DENSITY_SWSPIVEY:OutOfParameterRange','DENSITY_SWSPIVEY: Input values out of range!')
 warning('DENSITY_SWSPIVEY:OutOfParameterRange','Equations valid for p = 0.1-200 MPa, T=0-275 degC, S=0-216 .')
end

%------------------------------------------------------------------------
% Definition of constants, thermodynamic variables and parameters
%------------------------------------------------------------------------

% calculate weight equivalent of NaCl
sal2 = sal./35.*35.16504; % practical salinity to absolute salinity (g/kg solution)
sal2 = sal2.*(1+ 0.001005.*sal);% g/kg(solution) to g/kg(H2O)
m_NaCl = sal2./58.443; % mol/kg(H2O)

[p, T, m] = ndgrid(press, temp, m_NaCl);


%------------------------------------------------------------------------
% Calculation of fresh water density at 70 MPa
%------------------------------------------------------------------------

Dw1 = -0.127213;
Dw2 = 0.645486;
Dw3 = 1.03265;
Dw4 = -0.070291;
Dw5 = 0.639589;

rho_w0=coeffunc(T,Dw1,Dw2,Dw3,Dw4,Dw5);

%------------------------------------------------------------------------
% Calculation of coefficient Ew
%------------------------------------------------------------------------

Ew1 = 4.221;
Ew2 = -3.478;
Ew3 = 6.221;
Ew4 = 0.5182;
Ew5 = -0.4405;

Ew = coeffunc(T,Ew1,Ew2,Ew3,Ew4,Ew5);

%------------------------------------------------------------------------
% Calculation of coefficient Fw
%------------------------------------------------------------------------

Fw1 = -11.403;
Fw2 = 29.932;
Fw3 = 27.952;
Fw4 = 0.20684;
Fw5 = 0.3768;

Fw = coeffunc(T,Fw1,Fw2,Fw3,Fw4,Fw5);

%------------------------------------------------------------------------
% Calculation of brine water density at 70 MPa
%------------------------------------------------------------------------

Dcm2_1 = -7.925e-5;
Dcm2_2 = -1.93e-6;
Dcm2_3 = -3.4254e-4;
Dcm2_4 = 0;
Dcm2_5 = 0;

Dcm32_1 = 1.0998e-3;
Dcm32_2 = -2.8755e-3;
Dcm32_3 = -3.5819e-3;
Dcm32_4 = -0.72877;
Dcm32_5 = 1.92016;

Dcm1_1 = -7.6402e-3;
Dcm1_2 = 3.6963e-2;
Dcm1_3 = 4.36083e-2;
Dcm1_4 = -0.333661;
Dcm1_5 = 1.185685;

Dcm12_1 = 3.746e-4;
Dcm12_2 = -3.328e-4;
Dcm12_3 = -3.346e-4;
Dcm12_4 = 0;
Dcm12_5 = 0;

Dcm2 = coeffunc(T,Dcm2_1,Dcm2_2,Dcm2_3,Dcm2_4,Dcm2_5);
Dcm32 = coeffunc(T,Dcm32_1,Dcm32_2,Dcm32_3,Dcm32_4,Dcm32_5);
Dcm1 = coeffunc(T,Dcm1_1,Dcm1_2,Dcm1_3,Dcm1_4,Dcm1_5);
Dcm12 = coeffunc(T,Dcm12_1,Dcm12_2,Dcm12_3,Dcm12_4,Dcm12_5);

rho_b0 = rho_w0 + Dcm2.*m.^2 + Dcm32.*m.^(3/2) + Dcm1.*m + Dcm12 .*m.^(1/2);

%------------------------------------------------------------------------
% Calculation of coefficient Eb
%------------------------------------------------------------------------

Ec1 = 0;
Ec2 = 0;
Ec3 = 0.1353;
Ec4 = 0;
Ec5 = 0;

Ec = coeffunc(T,Ec1,Ec2,Ec3,Ec4,Ec5);

Eb = Ew + Ec.*m;

%------------------------------------------------------------------------
% Calculation of coefficient Fb
%------------------------------------------------------------------------

Fc32_1 = -1.409;
Fc32_2 = -0.361;
Fc32_3 = -0.2532;
Fc32_4 = 0;
Fc32_5 = 9.216;

Fc1_1 = 0;
Fc1_2 = 5.614;
Fc1_3 = 4.6782;
Fc1_4 = -0.307;
Fc1_5 = 2.6069;

Fc12_1 = -0.1127;
Fc12_2 = 0.2047;
Fc12_3 = -0.0452;
Fc12_4 = 0;
Fc12_5 = 0;

Fc32 = coeffunc(T,Fc32_1,Fc32_2,Fc32_3,Fc32_4,Fc32_5);
Fc1 = coeffunc(T,Fc1_1,Fc1_2,Fc1_3,Fc1_4,Fc1_5);
Fc12 = coeffunc(T,Fc12_1,Fc12_2,Fc12_3,Fc12_4,Fc12_5);

Fb = Fw + Fc32.*m.^(3/2) + Fc1.*m + Fc12.*m.^(1/2);

%------------------------------------------------------------------------
% Calculation of isothermal compressibility
%------------------------------------------------------------------------
p0 = 70; % reference pressure [MPa]

Ib0 = 1./Eb .* log(Eb + Fb); % compressibility at reference pressure 

Ib = 1./Eb .* log(Eb.*p./p0 + Fb);

%------------------------------------------------------------------------
% Calculation of brine density
%------------------------------------------------------------------------

rho = rho_b0.*exp(Ib - Ib0)*1000;

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function r = coeffunc(T,a1,a2,a3,a4,a5)
% Calculation of coefficients according to Spivey's standard equation

T2 = T./100;
r = (a1.*T2.^2 + a2.*T2 + a3)./(a4.*T2.^2 + a5.*T2 +1);

