function [rho] = density_swSun(press,temp,sal)

% DENSITY_SWSUN - calculates the density of seawater at higher temperatures
%
%      Usage: rho = density_swSun(p,T,S)
%
%      Input parameters (scalars or vectors): 
%          p       - pressure [MPa]      (0.1 < p < 100) 
%          T       - temperature [degC]  (0 < T < 374)
%          S       - practical salinity  (0 < S < 80)
%   
%      Output parameter:
%          rho     - density of seawater [kg/m^3]
%
%      References:
%      Sun et al. (2008) DSR I 55, 1304-1310.
%
%      Comment: 
%      Equations are valid in the liquid phase only.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (March 2011)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
%   Modification history: 
%       Equations corrected to equal the those in Sun's FORTRAN code
%       December 2011: Name changed from density_swhtemp to density_swSun
%       November 2012: lower limit for pressure corrected
%------------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('DENSITY_SWSUN:Input','DENSITY_SWSUN: Missing input parameter! Please provide p,T and S!')
else


%------------------------------------------------------------------------
% Check data range validity!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>100) && ~any(temp<(0) | temp>374) && ~any(sal<(0) | sal>80) 
        
else
    bad = find(press<0.1 | press > 100);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<(0) | temp>374);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<(0) | sal>80);
    sal(bad)=NaN*ones(size(bad));

 warning('DENSITY_SWSUN:OutOfParameterRange','DENSITY_SWSUN: Input values out of range!')
 warning('DENSITY_SWSUN:OutOfParameterRange','Equations valid for p = 0.1-100 MPa, T=0-374 degC, S = 0-80.')
end


%------------------------------------------------------------------------
% Definition of constants, thermodynamic variables and parameters
%------------------------------------------------------------------------

[P, T, S] = ndgrid(press,temp, sal*1.005); % absolute salinity


%------------------------------------------------------------------------
% Calculation of density
%------------------------------------------------------------------------

A = 9.9920571E+02...
    + T.*9.5390097E-02 + T.^2.*-7.6186636E-03...
    + T.^3.*3.1305828E-05 + T.^4.*-6.1737704E-08 ...
    + P.*(4.3368858E-01 + T.^2.*2.5495667E-05 ...
    + T.^3.*-2.8988021E-07 + T.^4.*9.5784313E-10) ...
    + P.^2.*(1.7627497E-03 + T.*-1.2312703E-04 + T.^2.*1.3659381E-06...
    + T.^3.*-4.0454583E-09)...
    + P.^3.*(-1.4673241E-05 + T.*8.8391585E-07 + T.^2.*-1.1021321E-08...
    + T.^3.*4.2472611E-11 + T.^4.*-3.9591772E-14);


B = S.*-0.7999922297101534...
    + S.*T.*0.002409364998382315 +S.*T.^2.*-0.000025805277543352034...
    + S.*T.^3.*6.856084054148796e-8...
    + P.*S.*0.0006297611058924313 + P.^2.*S.*-9.362637132680633e-7;


rho = A - B;

end
