function D = diffcoeff_sw(press,temp,sal,species,mco2)

% DIFFCOEFF_SW - calculates the diffusion coefficients of dissolved species
%      (ions, gases, etc.) in seawater or seawater with dissolved CO2
%
%      The correction for CO2-rich seawater is implemented via the
%      respective viscosity. No validation by experimental data exists.
%
%      Usage: D = diffcoeff_sw(p,T,S,'species',mco2)
%
%      Input parameters (p,T,S: scalars or vectors, species: string,
%                        mco2: scalar or matrix of size p x T x S): 
%          p        - pressure [MPa]
%          T        - temperature [degC] 
%          S        - practical salinity
%          mco2     - molality of dissolved CO2 [mol/kg] (default: mco2=0)
%          'species'- choice of chemical substance for which the
%                     diffusion coefficient is calculated:
%   1           O2      'o2'
%   2           NO3-    'no3'
%   3           Mn2+    'mn'
%   4           Fe2+    'fe'
%   5           SO42-   'so4'
%   6           CH4     'ch4'
%   7           HS-     'hs'
%   8           H2S     'h2s'
%   9           NH4+    'nh4'
%   10          NH3     'nh3'
%   11          PO43-   'po4'
%   12          HPO42-  'hpo4'
%   13          H2PO4-  'h2po4'
%   14          H3PO4   'h3po4'
%   15          H2O     'h2o'
%   16          H+      'hp'
%   17          OH-     'oh'
%   18          CO2     'co2'
%   19          HCO3-   'hco3'
%   20          CO32-   'co3'
%   21          B(OH)3  'boh3'
%   22          B(OH)4- 'boh4'
%   23          Mg2+    'mg'
%   24          Ca2+    'ca'
%   25          Ba2+    'ba'
%   26          Sr2+    'sr'
%   27          Cl-     'cl'
%   28          Br-     'br'
%   29          Li+     'li'
%   30          SiO44-  'sio4'
%   31          He      'he'
%   32          Ne      'ne'
%   33          Ar      'ar'
%   34          Kr      'kr'
%   35          Xe      'xe'
%   36          Rn      'rn'
%   37          N2      'n2'
%
%      Output parameter:
%          D        - diffusion coefficient [m^2/s]
%
%      References: 
%      Li & Gregory (1974) GCA 38, 703-714.
%      Hayduk & Laudie (1974) Amer.Inst.Chem.Eng.J. 20(3), 611-615.
%      Boudreau (1997) Diagenetic Models and Their Implementation, Springer Verlag.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%  Modification history: 
%       November 2012: He, Ne, Ar, Kr, Xe and Rn added
%       April 2013: N2 added
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('DIFFCOEFF_SW:Input','DIFFCOEFF_SW: Missing input parameter! Please provide p, T and S!')
else

if nargin == 3
   species = '';
   mco2 = 0;
end

if nargin == 4
   mco2 = 0;
end

[P,T] = ndgrid(press.*10,temp,sal); % pressure in bar

if mco2 == 0
    % viscosity in [mPa s]
    % dynamic viscosity of pure water at T, P=1 bar
    muo = visco_sw(0.1.*ones(size(press)),temp,zeros(size(sal))).*1000;
    % dynamic viscosity of seawater at P, T, S 
    mu = visco_sw(press,temp,sal).*1000;
    % dynamic viscosity at T=25degC and S=36.1
    mu25 = visco_sw(0.1.*ones(size(press)),25*ones(size(temp)),36.1*ones(size(sal))).*1000;
    % dynamic viscosity of seawater at T and S=0 
    mutk = visco_sw(0.1.*ones(size(press)),temp,zeros(size(sal))).*1000;
    % P and S correction using Stokes-Einstein relation
    pscorr = muo./mu;

else
    % dynamic viscosity of pure water at T, P=1 bar
    muo = visco_co2sw(0.1.*ones(size(press)),temp,zeros(size(sal)),0).*1000;
    % dynamic viscosity of seawater at P, T, S 
    mu = visco_co2sw(press,temp,sal,mco2).*1000;
    % dynamic viscosity at T=25degC and S=36.1
    mu25 = visco_co2sw(0.1.*ones(size(press)),25*ones(size(temp)),36.1*ones(size(sal)),mco2).*1000;
    % dynamic viscosity of seawater at T and S=0 
    mutk = visco_co2sw(0.1.*ones(size(press)),temp,zeros(size(sal)),mco2).*1000;
    % P and S correction using Stokes-Einstein relation
    pscorr = muo./mu;    
end

switch species
    case 'o2'
        % Boudreau (1997)
        D = (0.2604 + 0.006383*(T+273.15)./muo)* 1.D-5 .*pscorr;
        % Hayduk & Laudie (1974)
%        D = 4.72D-9*(T+273.15)./(muo/100*27.9^.6) .*pscorr;
    case 'no3'
        D = (9.50 + .388*T)*1.D-6 .*pscorr;
    case 'mn'
        D = (3.18 + .155*T)*1.D-6 .*pscorr;
    case 'fe'
        D = (3.31 + .150*T)*1.D-6 .*pscorr;
    case 'so4'
        D = (4.88 + .232*T)*1.D-6 .*pscorr;
    case 'ch4'
        D = 4.72D-9*(T+273.15)./(muo/100*(37.7^.6)) .*pscorr;
          % Hayduk & Laudie (1974)
    case 'hs'
        D = (10.4 + .273*T)*1.D-6 .*pscorr;
    case 'h2s'
        D = 4.72D-9*(T+273.15)./(muo/100*35.2^.6) .*pscorr;
          % Hayduk & Laudie (1974)
    case 'nh4'
        D = (9.50 + .413*T)*1.D-6 .*pscorr;
    case 'nh3'
        D = 4.72D-9*(T+273.15)./(muo/100*24.5^.6) .*pscorr;
          % Hayduk & Laudie (1974)
    case 'po4'
        D = (2.62 + 0.143*T)*1.D-6 .*pscorr;
    case 'hpo4'
        D = (3.26 + 0.177*T)*1.D-6 .*pscorr;
    case 'h2po4'
        D = (4.02 + 0.223*T)*1.D-6 .*pscorr;
    case 'h3po4'
        % Least (1984): T = 25 degC and S = 0
        Dm = 0.87D-5;
        % dynamic viscosity at T=25degC and S=0 
        D = Dm * mu25/298.15 .* (T+273.15)./mutk .*pscorr;
    case 'h2o'
        % Cohen & Turnbull (1959); Krynicki et al. (1978)
        a = 12.5D-9 * exp(-5.22D-4.*P/1.013);
        b = 925.0 * exp(-2.6D-4.*P/1.013);
        c = 95.0 + 2.61D-02.*P/1.013;
        D = a.*sqrt(T+273.15).*exp(-b./((T+273.15)-c))*1.D4 .*pscorr;
    case 'hp'
        D = (54.4 + 1.555*T)*1.D-6 .*pscorr;
    case 'oh'
        D = (25.9 + 1.094*T)*1.D-6 .*pscorr;
    case 'co2'
        % Boudreau (1997)
        D = (0.1954 + 0.005089*(T+273.15)./muo)* 1.D-5 .*pscorr;
        % Hayduk & Laudie (1974)
%        D = 4.72D-9*(T+273.15)./(muo/100*37.3^.6) .*pscorr;
    case 'hco3'
        D = (5.06 + .275*T)*1.D-6 .*pscorr;
    case 'co3'
        D = (4.33 + .199*T)*1.D-6 .*pscorr;
    case 'boh3'
        % Mackin (1986): T = 25 degC and S = 29.2
        Dm = 1.12D-5;
        % dynamic viscosity at T=25degC and S=29.2 
        D = Dm * mu25/298.15 .* (T+273.15)./mutk .*pscorr;
    case 'boh4'
        % Boudreau & Canfield (1988) assume: 12.5% smaller than B(OH)3
        Dm = 0.875*1.12D-5;
        % dynamic viscosity at T=25degC and S=29.2 
        D = Dm * mu25/298.15 .* (T+273.15)./mutk .*pscorr;
    case 'ca'
        D = (3.60 + .179*T)*1.D-6 .*pscorr;
    case 'mg'
        D = (3.43 + 0.144*T)*1.D-6 .*pscorr;
    case 'ba'
        D = (4.06 + 0.176*T)*1.D-6 .*pscorr;
    case 'sr'
        D = (3.69 + 0.169*T)*1.D-6 .*pscorr;
    case 'cl'
        D = (9.60 + .438*T)*1.D-6 .*pscorr;
    case 'br'
        D = (10.0 + 0.441*T)*1.D-6 .*pscorr;
    case 'li'
        D = (4.43 + .241*T)*1.D-6 .*pscorr;
    case 'sio4'
        % Wollast and Garrels (1971): T = 25 deg C and S = 36.1
        Dm = 1.0D-5;
        D = Dm * mu25/298.15 .* (T+273.15)./mutk .*pscorr;
    case 'he'
        % Hayduk & Laudie (1974) -> does not give expected D0
%        D = 4.72D-9*(T+273.15)./(muo/100*31.9^.6) .*pscorr;
        % Jaehne et al. (1978)
        D = 818.D-5*exp(-11.7D3/8.31441./(T+273.15)) .*pscorr;
    case 'ne'
        % Hayduk & Laudie (1974)
        D = 4.72D-9*(T+273.15)./(muo/100*16.8^.6) .*pscorr;
        % Jaehne et al. (1978)
%        D = 1608.D-5*exp(-14.84D3/8.31441./(T+273.15)) .*pscorr;
    case 'ar'
        % Hayduk & Laudie (1974)
        D = 4.72D-9*(T+273.15)./(muo/100*29.2^.6) .*pscorr;
        % Ohsumi & Horibe (1984)
%        D = 7238.D-5*exp(-19.81D3/8.31441./(T+273.15)) .*pscorr;
    case 'kr'
        % Hayduk & Laudie (1974)
        D = 4.72D-9*(T+273.15)./(muo/100*34.9^.6) .*pscorr;
        % Jaehne et al. (1978)
%        D = 6393.D-5*exp(-20.20D3/8.31441./(T+273.15)) .*pscorr;
    case 'xe'
        % Jaehne et al. (1978)
        D = 9007.D-5*exp(-21.61D3/8.31441./(T+273.15)) .*pscorr;
    case 'rn'
        % Jaehne et al. (1978)
        D = 15877.D-5*exp(-23.26D3/8.31441./(T+273.15)) .*pscorr;
    case 'n2'
        % Hayduk & Laudie (1974)
        D = 4.72D-9*(T+273.15)./(muo/100*34.7^.6) .*pscorr;
    otherwise
        D = 1.D-5 * ones(size(T));% [cm^2/s]
end
D = D.*1e-4; % [m^2/s]
end
