function [rho, cv, cp, s, u, h, f, JT, pv, cs] = eos_ch4(press,temp)

% EOS_CH4 -  calculates thermodynamic properties of CH4 
%     based on an equation of state
%
%     Usage: [rho cv cp s u h f JT pv cs] = eos_ch4(p,T)
% 
%     Input parameters (scalars or vectors): 
%         p       - pressure [MPa]     (0.012 < p < 1000) 
%         T       - temperature [degC] (-183< T < 347)
%
%     Output parameters:
%         rho     - density [kg/m^3]
%         cv      - specific isochoric heat capacity [J/(kg K)]
%         cp      - specific isobaric heat capacity [J/(kg K)]
%         s       - specific entropy [J/(kg K)] 
%         u       - specific internal energy [J/kg]
%         h       - specific enthalpy [J/kg] 
%         f       - fugacity coefficient 
%         JT      - Joule-Thomson coefficient [K/MPa]
%         pv      - vapor pressure [MPa] (only for T < T_critical)
%         cs      - saturated liquid heat capacity [J/kg/K]
%                             (only for T < T_critical)
%      
%     All thermodynamic properties (except for pv) are calculated from the 
%     dimensionless Helmholtz energy Phi = A/(RT) 
%     ( A(rho,T) = h_0(T)-R*T-T*s_0(rho,T) ) and its derivatives with 
%     respect to p and T.
%
%     References: 
%     Setzmann & Wagner (1991) J. Phys. Chem. Ref. Data 20, 1061-1155.
% 
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       November 2009: Constants nk corrected (lines 328-331)
%       December 2009: Parameter names in header corrected (lines 15-19)
%       November 2012: syntax changed to Matlab2012 style
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('EOS_CH4:Input','EOS_CH4: Missing input parameter! Please provide p and T!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.012 | press>1000) && ~any(temp< -183 | temp>347)
else
    bad = find(press<0.012 | press > 1000);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<-183 | temp>347);
    temp(bad)=NaN*ones(size(bad));
    warning('EOS_CH4:OutOfParameterRange','EOS_CH4: Input values out of range!')
    warning('EOS_CH4:OutOfParameterRange','Equations valid for p = 0-1000 MPa, T=(-183) - 347 degC.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[P,T] = ndgrid(press,temp+273.15); % pressure in MPa, temperature in K

rhoc = 162.66; % critical density of CH4 [kg/m^3]
Tc = 190.564; % critical temperature of CH4 [K]
pc = 4.5922;   % critical pressure in [MPa]
R = 0.5182705; % specific gas constant [kJ/(kg K)]
tau = Tc./T; % inverse of reduced temperature

% define empty data structure for density 
rho= zeros(size(P,1),size(P,2),size(P,3)); 

% Check if arrays for P and T contain NaN. If yes, also set corresponding 
% density values to NaN. Calculate density for allowed P and T only.
% If the fit function returns EXITFLAG <= 0, the iteration process has
% terminated with a non-satisfying result. In this case, the initial value
% is changed and the fit procedure is repeated.

options=optimset('Display','off','TolFun',1e-16); % set options for fitfunction
 
for i=1:size(P,1)
    for j=1:size(P,2)
        if isnan(P(i,:));
            rho(i,:) = NaN;
        elseif isnan(T(:,j));
            rho(:,j) = NaN;
        else  
            EXITFLAG = -1; % Initialize EXITFLAG value
            counter = 0; % set counter to 0
            % use internal function to look up first guess for density 
            rho_init1 = guessfunc(P(i,j),T(i,j),R);
            rho_init = rho_init1; % first initial value for density
            while (EXITFLAG < 1) 
                counter = counter + 1;
                [rho(i,j),~,EXITFLAG] = fsolve(@(rho_ch4) minfunc(rho_ch4,P(i,j),T(i,j),rhoc,R,tau(i,j)), rho_init,options);
                % change initial value for fit
                rho_init = rho_init1 + counter*1*(-1)^(counter); 
                 if rho(i,j) <= 0
                    EXITFLAG = -1; % reject negative densities  
                end
                 if rho_init <= 0
                    EXITFLAG = 10; % make sure, that while loop terminates  
                end
            end
        end
    end
end

delta = rho./rhoc; % reduced density

%------------------------------------------------------------------------
% calculation of ideal gas part of Helmholtz energy and derivatives
%------------------------------------------------------------------------
[phi0, dphi0dtau, dphi0dtautau] = helmholtz_idealgas(delta,tau);

%------------------------------------------------------------------------
% calculation of residual part of Helmholtz energy and derivatives
%------------------------------------------------------------------------
[phir, dphirdtau, dphirdtautau, dphirddeltatau, dphirddelta, dphirddeltadelta] = helmholtz_residual(delta,tau);

%------------------------------------------------------------------------
% calculation of derivative of vapor pressure with respect to Temperature
%------------------------------------------------------------------------
[pv, dpvdT] = vappress(T,Tc,pc);
crit = T > Tc;
pv(crit) = NaN;
dpvdT(crit) = NaN;

%------------------------------------------------------------------------
% calculation of thermodynamic properties
%------------------------------------------------------------------------

% Entropy [J/kgK]:
s = R.*(tau.*(dphi0dtau + dphirdtau) - phi0 - phir).*1000 ;

% Internal Energy [J/kg]:
u = R.*T.*tau.*(dphi0dtau + dphirdtau).*1000;

% Isochoric heat capacity [J/kgK]:
cv = -R.*tau.^2.*(dphi0dtautau + dphirdtautau).*1000;

% Enthalpy [J/kg]: 
h = R.*T.*(1 + tau.*(dphi0dtau + dphirdtau) + delta.*dphirddelta).*1000;

% Isobaric heat capacity [J/kgK]:
cp = R.*(-tau.^2.*(dphi0dtautau + dphirdtautau) + ...
    (1 + delta.*dphirddelta - delta.*tau.*dphirddeltatau).^2./...
    (1 + 2.*delta.*dphirddelta + delta.^2.*dphirddeltadelta)).*1000;

% Fugacity coefficient:
f = exp(phir + delta.*dphirddelta - log(1 + delta .*dphirddelta));

% Saturated liquid heat capacity [J/kgK] 
cs =  R.*(-tau.^2.*(dphi0dtautau + dphirdtautau) + ...
    (1 + delta.*dphirddelta - delta.*tau.*dphirddeltatau) ./...
   (1 + 2.*delta.*dphirddelta + delta.^2.*dphirddeltadelta) .* ...
   ((1 + delta.* dphirddelta - delta.*tau.*dphirddeltatau) - ...
   1./rhoc./R./delta.*dpvdT)).*1000;

% Joule-Thomson coefficient [K/kPa]
JT = 1./R./rho .*(-(delta.*dphirddelta + delta.^2.*dphirddeltadelta + ...
    delta.*tau.*dphirddeltatau)./((1 + delta.*dphirddelta - ...
    delta.*tau.*dphirddeltatau).^2 - tau.^2.*(dphi0dtautau + ...
    dphirdtautau).*(1 + 2.*delta.*dphirddelta + delta.^2.*dphirddeltadelta)));

JT = JT .* 10 ; % convert to [K/MPa]

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [phi0, dphi0dtau, dphi0dtautau] = helmholtz_idealgas(delta,tau)

%  calculation of the ideal gas part (phi0) of the Helmholtz energy and its
%  first and second derivatives with respect to the reduced temperature
%  (dphi0dtau, dphi0dtautau)

% a, theta: coefficients of the correlation equation
a1 =   [9.91243972
        -6.33270087
        3.0016];
    
 a2 =   [0.008449
        4.6942
        3.4865
        1.6572
        1.4115];
    
theta =[3.40043240
        10.26951575
        20.43932747
        29.93744884
        79.13351945];

% calculate sum terms
sum1 = zeros(size(tau,1),size(tau,2));
sum2 = zeros(size(tau,1),size(tau,2));
sum3 = zeros(size(tau,1),size(tau,2));

for i = 1:5
    sum1 = sum1 + a2(i).*log(1 -exp(-tau.*theta(i)));
    sum2 = sum2 + a2(i).*theta(i).*((1 -exp(-tau.*theta(i))).^(-1)-1);
    sum3 = sum3 + a2(i).*theta(i).^2.*exp(-tau.*theta(i))...
        .*(1 -exp(-tau.*theta(i))).^(-2);
end

% calculate phi and derivatives
phi0 = log(delta) + a1(1) + a1(2).*tau + a1(3).* log(tau) + sum1;  
dphi0dtau = a1(2) + a1(3)./tau + sum2;
dphi0dtautau = - a1(3)./tau.^2 - sum3;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [phir, dphirdtau, dphirdtautau, dphirddeltatau, dphirddelta, dphirddeltadelta] = helmholtz_residual(delta,tau)

%  calculation of the residual part (phir) of the Helmholtz energy and its
%  first and second derivatives with respect to the inverse reduced temperature
%  (dphirdtau and dphirdtautau) and first and second derivatives with respect to
%  the reduced density (dphirddelta, dphirddeltadelta) as well as 
%  the mixed derivative (dphirddeltadtau)

%----------------------------------------------------------------------
% calculation of sum terms with parameters ni,di,ti
%----------------------------------------------------------------------

ni = [0.4367901028D-1
      0.6709236199D+0
     -0.1765577859D+1
      0.8582330241D+0
     -0.1206513052D+1
      0.5106513052D+0
     -0.4000010791D-3
     -0.1247842423D-1
      0.3100269701D-1
      0.1754748522D-2
     -0.3171921605D-5
     -0.2240346840D-5
      0.2947056156D-6];
  
di = [1 1 1 2 2 2 2 3 4 4 8 9 10];
ti = [-0.5 0.5 1 0.5 1 1.5 4.5 0 1 3 1 3 3];

sumr1 =  zeros(size(tau,1),size(tau,2));
sumrd1 =  zeros(size(tau,1),size(tau,2));
sumrdd1 =  zeros(size(tau,1),size(tau,2));
sumrt1 =  zeros(size(tau,1),size(tau,2));
sumrtt1 =  zeros(size(tau,1),size(tau,2));
sumrdt1 =  zeros(size(tau,1),size(tau,2));

for i = 1:13
    sumr1 = sumr1 + ni(i).*delta.^di(i).*tau.^ti(i);
    sumrd1 = sumrd1 + ni(i).*di(i).*delta.^(di(i)-1).*tau.^ti(i);
    sumrdd1 = sumrdd1 + ni(i).*di(i).*(di(i)-1).*delta.^(di(i)-2).*tau.^ti(i);
    sumrt1 = sumrt1 + ni(i).*ti(i).*delta.^di(i).*tau.^(ti(i)-1);
    sumrtt1 = sumrtt1 + ni(i).*ti(i).*(ti(i)-1).*delta.^di(i).*tau.^(ti(i)-2);
    sumrdt1 = sumrdt1 + ni(i).*ti(i).*di(i).*delta.^(di(i)-1).*tau.^(ti(i)-1);
end


%----------------------------------------------------------------------
% calculation of sum terms with parameters nj,dj,tj,cj
%----------------------------------------------------------------------

nj = [0.1830487909D+0
      0.1511883679D+0
     -0.4289363877D+0
      0.6894002446D-1
     -0.1408313996D-1
     -0.3063054830D-1
     -0.2969906708D-1
     -0.1932040831D-1
     -0.1105739959D+0
      0.9952548995D-1
      0.8548437825D-2
     -0.6150555662D-1
     -0.4291792423D-1
     -0.1813207290D-1
      0.3445904760D-1
     -0.2385919450D-2
     -0.1159094939D-1
      0.6641693602D-1
     -0.2371549590D-1
     -0.3961624905D-1
     -0.1387292044D-1
      0.3389489599D-1
     -0.2927378753D-2];
 
dj = [1 1 1 2 4 5 6 1 2 3 4 4 3 5 5 8 2 3 4 4 4 5 6];
tj = [0 1 2 0 0 2 2 5 5 5 2 4 12 8 10 10 10 14 12 18 22 18 14];
cj = [1 1 1 1 1 1 1 2 2 2 2 2 3 3 3 3 4 4 4 4 4 4 4];

sumr2 =  zeros(size(tau,1),size(tau,2));
sumrd2 =  zeros(size(tau,1),size(tau,2));
sumrdd2 =  zeros(size(tau,1),size(tau,2));
sumrt2 =  zeros(size(tau,1),size(tau,2));
sumrtt2 =  zeros(size(tau,1),size(tau,2));
sumrdt2 =  zeros(size(tau,1),size(tau,2));

for i = 1:23
    sumr2 = sumr2 + nj(i).*delta.^dj(i).*tau.^tj(i).*exp(-delta.^cj(i));
    sumrd2 = sumrd2 + nj(i).*delta.^(dj(i)-1).*tau.^tj(i)....
        .*exp(-delta.^cj(i)).*(dj(i)-cj(i).*delta.^cj(i));
    sumrdd2 = sumrdd2 + nj(i).*delta.^(dj(i)-2).*tau.^tj(i)...
        .*exp(-delta.^cj(i)).*((dj(i)-cj(i).*delta.^cj(i))...
        .*(dj(i)-1-cj(i).*delta.^cj(i))-cj(i)^2.*delta.^cj(i));
    sumrt2 = sumrt2 + nj(i).*delta.^dj(i).*tau.^(tj(i)-1).*tj(i)...
        .*exp(-delta.^cj(i));
    sumrtt2 = sumrtt2 + nj(i).*delta.^dj(i).*tau.^(tj(i)-2).*tj(i)... 
        .*(tj(i)-1).*exp(-delta.^cj(i));
    sumrdt2 = sumrdt2 + nj(i).*delta.^(dj(i)-1).*tau.^(tj(i)-1).*tj(i)...
        .*exp(-delta.^cj(i)).*(dj(i)-cj(i).*delta.^cj(i));
end

%----------------------------------------------------------------------
% calculation of sum terms with parameters nk,dk,tk,ak,bk,ck,ek
%----------------------------------------------------------------------

nk = [  0.9324799946D-4
       -0.6287171518D+1
        0.1271069467D+2
       -0.6423953466D+1];

dk = [2 0 0 0];
tk = [2 0 1 2];
ak = [20 40 40 40];
bk = [200 250 250 250];
ck = [1.07 1.11 1.11 1.11];
ek = [1. 1. 1. 1. 1.];


sumr3 =  zeros(size(tau,1),size(tau,2));
sumrd3 =  zeros(size(tau,1),size(tau,2));
sumrdd3 =  zeros(size(tau,1),size(tau,2));
sumrt3 =  zeros(size(tau,1),size(tau,2));
sumrtt3 =  zeros(size(tau,1),size(tau,2));
sumrdt3 =  zeros(size(tau,1),size(tau,2));

for i = 1:4
    sumr3 = sumr3 + nk(i).*delta.^dk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2);
    sumrd3 = sumrd3 + nk(i).*delta.^dk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2)...
        .*(dk(i)./delta - 2.*ak(i).*(delta -ek(i)));
     sumrdd3 = sumrdd3 + nk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2)...
        .*(- 2.*ak(i).*delta.^dk(i) + 4.*ak(i)^2.*delta.^dk(i)...
        .*(delta - ek(i)).^2 - 4.*dk(i).*ak(i).*delta.^(dk(i)-1)...
        .*(delta - ek(i)) + dk(i).*(dk(i)-1).*delta.^(dk(i)-2));
     sumrt3 = sumrt3 + nk(i).*delta.^dk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2)...
        .*(tk(i)./tau - 2.*bk(i).*(tau -ck(i)));
     sumrtt3 = sumrtt3 + nk(i).*delta.^dk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2)...
        .*((tk(i)./tau - 2.*bk(i).*(tau -ck(i))).^2 - tk(i)./tau.^2 - 2.*bk(i));
     sumrdt3 = sumrdt3 + nk(i).*delta.^dk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2)...
        .*(tk(i)./tau - 2.*bk(i).*(tau -ck(i)))...
        .*(dk(i)./delta - 2.*ak(i).*(delta -ek(i)));
end


%----------------------------------------------------------------------
% calculation of function output values
%----------------------------------------------------------------------

phir = sumr1 + sumr2 + sumr3; % phi_r: residual of Helmholtz energy
dphirddelta = sumrd1 + sumrd2 + sumrd3;        % d(phi_r)/d(delta)
dphirddeltadelta = sumrdd1 + sumrdd2 + sumrdd3; % d^2(phi_r)/d(delta^2)
dphirdtau = sumrt1 + sumrt2 + sumrt3;          % d(phi_r)/d(tau)
dphirdtautau = sumrtt1 + sumrtt2 + sumrtt3;   % d^2(phi_r)/d(tau^2)
dphirddeltatau = sumrdt1 + sumrdt2 + sumrdt3; % d^2(phi_r)/d(delta)d(tau)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [ps, dpsdT] = vappress(T,Tc,pc)

%  calculation of the vapor pressure p_s and its first derivative d(p_s)/d(T)

% coefficients a and t
a = [ -6.036219 1.409353 -0.4945199 -1.443048];
t = [1 1.5 2 4.5];

%calculation of sum terms
sum1 = zeros(size(T,1),size(T,2));
sum2 = zeros(size(T,1),size(T,2));

for i = 1:4
    sum1 = sum1 + a(i).*(1-T./Tc).^t(i);
    sum2 = sum2 + a(i).*t(i).*(1-T./Tc).^(t(i)-1);
end

% calculation of p_s and d(p_s)/d(T)
ps = pc.*exp((Tc./T.*sum1));
dpsdT = pc.*(-Tc./T.^2.*sum1 - 1./T.*sum2) .* exp(Tc./T.*sum1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function f = minfunc(rho,P,T,rhoc,R,tau)
% minimization function to calculate density at given pressure
% based on the equation p = rho*R*T*(1 + rho/rohc * d(phir)/d(rho/rhoc)) 

delta = rho/rhoc;

% calculate d(phir)/d(rho/rhoc)
[~, ~, ~, ~, dphirddelta] = helmholtz_residual(delta,tau);

% calculate density
RHOeos = P * 1000 / (R*T*(1+delta*dphirddelta));

% calculate difference between initial rho and "new" calculated rho
f = rho - RHOeos;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function guess = guessfunc(P,T,R)
% The ideal gas equation is used to calculate a first guess for the density
R = R*1000; % [J/kg/K];
P = P * 1000000; % [Pa]

guess = P./T./R;
