function [rho, cp, cv, s, u, h, beta, f, JT] = eos_h2o(press,temp)

% EOS_H2O -  calculates thermodynamic properties of pure water based on
%     an equation of state
%
%     Usage: [rho cp cv s u h beta f JT] = eos_h2o(p,T)
% 
%     Input parameters (scalars or vectors): 
%         p       - pressure [MPa]     (0 < p < 167.5) 
%         T       - temperature [degC] (0 < T < 1000)
%
%     Output parameters:
%         rho     - density [kg/m^3]
%         cp      - specific isobaric heat capacity [J/(kg K)]
%         cv      - specific isobaric heat capacity [J/(kg K)]
%         s       - specific entropy [J/(kg K)] 
%         u       - specific internal energy [J/kg]
%         h       - specific enthalpy [J/kg] 
%         beta    - isentropic temperature-pressure coefficient [K/Pa]
%         f       - fugacity coefficient 
%         JT      - Joule-Thomson coefficient [K/MPa]
%
%     All thermodynamic properties are calculated from the Gibbs energy
%     (g = u + pV - Ts) and its derivatives with respect to p and T. 
%
%     References: 
%     IAPSW (1996) The IAPWS formlation 1995 for the thermodynamic 
%       properties of ordinary water substance for general and scientific
%       use. www.iapws.org
%     Wagner & Pruss (2002) J.Phys.Chem.Ref.Data 31, 387-535.
%     IAPSW (2007) Revised release for the IAPWS industrial formlation 1997
%       for the thermodynamic properties of water and steam. www.iapws.org
%     Marcus (2000) Phys Chem Chem Phys 2, 1465-1472.
%
%     Comments: 
%     Not valid for ice phase! (No check for ice phase implemented)
%
%     The deviation from the tabulated density values of Wagner and Pruss
%     is less than 1%.
%
%     The formulation of IAPWS (2007) and equations from Marcus (2000) are
%     used to calculate a start value for the iteration of the density. 
%     The IAPWS (2007) formalism is valid for T=0-800 degC and p=0-100 MPa.
%     Since the calculated density is only considered as a start value and
%     only needs to be approximated but not exact, the formulation might be
%     used for T=0-1000 degC and p<167.5 MPa or T=0-227 degC and p<225 MPa.
%
%     The formalism can be extended to the full range of IAPWS
%     (0 < T < 1000 degC and p < 1000 MPa), if good starting values
%     for the density calculations can be provided.
%     A lookup-table could be implemented from table 13.2 in Wagner & 
%     Pruss (2002).
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (March 2011)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%  Modification history: 
%       November 2012: guessfunc takes now care of NaN 
%       November 2012: syntax changed to Matlab2012 style
%       January 2012:  name changed from eos_H2O to eos_h2o
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('EOS_H2O:Input','EOS_H2O: Missing input parameter! Please provide p and T!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------
if ~any(press<=0 | press>167.5) && ~any(temp< 0 | temp>1000)
else
    bad = find(press<=0 | press > 167.5);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>1000);
    temp(bad)=NaN*ones(size(bad));
    warning('EOS_H2O:OutOfParameterRange','EOS_H2O: Input values out of range!')
    warning('EOS_H2O:OutOfParameterRange','Equations valid for 0 < p >= 167.5 MPa, T=0-1000 degC.')
end


%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------
[P,T] = ndgrid(press*1e6,temp+273.15); % pressure in Pa, temperature in K

R = 0.46151805; % specific gas constant [kJ/(kg K)]


%------------------------------------------------------------------------
%------------------------------------------------------------------------
% CALCULATION OF PURE WATER CONTRIBUTION
%------------------------------------------------------------------------
%------------------------------------------------------------------------

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------
rhoc = double(322); % critical density of pure water [kg/m^3]
Tc = double(647.096); % critical temperature of pure water [K]
tau = Tc./T; % inverse of reduced temperature

%------------------------------------------------------------------------
% calculation of density of pure water by minimizing the thermodynamic
% equation p = rho * R * T * (1 + d(phi_r)/d(delta))
%------------------------------------------------------------------------

% create empty data array for density of pure water
rho = zeros(size(P,1),size(P,2)); 

% Check if arrays for P and T contain NaN. If yes, also set corresponding 
% density values to NaN. Calculate density for allowed P and T only.
% If the fit function returns EXITFLAG <= 0, the iteration process has
% terminated with a non-satisfying result. In this case, the initial value
% is changed and the fit procedure is repeated.

options=optimset('Display','off','TolFun',1e-16); % set options for fitfunction

% use internal function to look up first guess for density

rho_init1 = guessfunc(press,temp);


for i=1:size(P,1)
    for j=1:size(P,2)
        if isnan(P(i,:));
            rho(i,:) = NaN;
        elseif isnan(T(:,j));
            rho(:,j) = NaN;
        else  
            EXITFLAG = -1; % Initialize EXITFLAG value
            counter = 0; % set counter to 0
            rho_init = rho_init1(i,j); % first initial value for density
            while (EXITFLAG < 1) 
                counter = counter + 1;
                [rho(i,j),~,EXITFLAG] = fsolve(@(rho_pwat) minfunc(rho_pwat,P(i,j),T(i,j),rhoc,R,tau(i,j)), rho_init,options);
                % change initial value for fit
                rho_init = rho_init1(i,j) + counter*1*(-1)^(counter); 
                 if rho(i,j) <= 0
                    EXITFLAG = -1; % reject negative densities  
                end
                if rho_init <= 0
                    EXITFLAG = 10; % make sure, that while loop terminates  
                end
            end
        end
    end
end

delta_pw = rho./rhoc; % reduced density of pure water

%------------------------------------------------------------------------
% calculation of ideal gas part of dimensionless Helmholtz energy 
% and its derivatives
%------------------------------------------------------------------------
[phi0, dphi0dtau, dphi0dtautau] = helmholtz_idealgas(delta_pw,tau);

%------------------------------------------------------------------------
% calculation of residual part of Helmholtz energy and its derivatives
%------------------------------------------------------------------------
[phir, dphirdtau, dphirdtautau, dphirddeltatau, dphirddelta, dphirddeltadelta] = helmholtz_residual(delta_pw,tau);

%------------------------------------------------------------------------
% calculation of thermodynamic properties of pure water
%------------------------------------------------------------------------

% Entropy [kJ/kgK]:
s = R.*(tau.*(dphi0dtau + dphirdtau) - phi0 - phir); 
s = s*1000; % [J/kgK]
        
% Internal Energy [kJ/kg]:
 u = R.*T.*tau.*(dphi0dtau + dphirdtau);
 u = u*1000; % [J/kgK]

% Isochoric heat capacity [kJ/kgK]:
cv = -R.*tau.^2.*(dphi0dtautau + dphirdtautau);
cv = cv*1000; % [J/kgK]

% Enthalpy [kJ/kg]: 
h = R.*T.*(1 + tau.*(dphi0dtau + dphirdtau) + delta_pw.*dphirddelta);
h = h*1000; % [J/kgK]

% Isobaric heat capacity [kJ/kgK]:
cp = R.*(-tau.^2.*(dphi0dtautau + dphirdtautau) + ...
    (1 + delta_pw.*dphirddelta - delta_pw.*tau.*dphirddeltatau).^2./...
    (1 + 2.*delta_pw.*dphirddelta + delta_pw.^2.*dphirddeltadelta));
cp = cp*1000; % [J/kgK]

% Speed of sound [m/s]
% w = sqrt(R.*T.*(1 + 2 .*delta_pw.*dphirddelta + delta_pw.^2.*dphirddeltadelta...
%    -  (1 + delta_pw.*dphirddelta - delta_pw.*tau.*dphirddeltatau).^2./...
%    (tau.^2.*(dphi0dtautau + dphirdtautau))));
% w = ww.*sqrt(1000); % convert units to [m/s]

% Isentropic temperature-pressure coefficient [m^3*K/kJ]
beta = 1./rho./R.*(...
    (1 + delta_pw.*dphirddelta - delta_pw.*tau.*dphirddeltatau)./ ...
    ((1 + delta_pw.*dphirddelta - delta_pw.*tau.*dphirddeltatau).^2 - ...
    tau.^2.*(dphi0dtautau + dphirdtautau).* ...
    (1 + 2.*delta_pw.*dphirddelta + delta_pw.^2.*dphirddeltadelta)));
beta = beta/1000; % [m^3*K/J]

% Fugacity coefficient:
f = exp(phir + delta_pw.*dphirddelta - log(1 + delta_pw .*dphirddelta));

% Joule-Thomson coefficient [K/kPa]
JT = 1./R./rho .*(-(delta_pw.*dphirddelta + delta_pw.^2.*dphirddeltadelta + ...
   delta_pw.*tau.*dphirddeltatau)./((1 + delta_pw.*dphirddelta - ...
   delta_pw.*tau.*dphirddeltatau).^2 - tau.^2.*(dphi0dtautau + ...
   dphirdtautau).*(1 + 2.*delta_pw.*dphirddelta + delta_pw.^2.*dphirddeltadelta)));

JT = JT/1000 ; % convert to [K/Pa]

% pressure can be used as check value for density calculation:
%PP = rho_pw.*(1000.*R.*T.*(1+delta_pw.*dphirddelta))

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [phi0, dphi0dtau, dphi0dtautau] = helmholtz_idealgas(delta,tau)

%  calculation of the ideal gas part (phi0) of the Helmholtz energy and its
%  first and second derivatives with respect to the reduced temperature
%  (dphi0dtau, dphi0dtautau)

% a, theta: coefficients of the correlation equation
a1 =   [-8.32044648201
        6.6832105268
        3.00632];
    
a2 =   [0.012436
        0.97315
        1.2795
        0.96956
        0.24873];
    
theta =[1.28728967
        3.53734222
        7.74073708
        9.24437796
        27.5075105];

% calculate sum terms
sum1 = zeros(size(tau,1),size(tau,2),size(tau,3));
sum2 = zeros(size(tau,1),size(tau,2),size(tau,3));
sum3 = zeros(size(tau,1),size(tau,2),size(tau,3));
for i=1:5
    sum1 = sum1 + a2(i).*log(1 -exp(-tau.*theta(i)));
    sum2 = sum2 + a2(i).*theta(i).*((1 -exp(-tau.*theta(i))).^(-1)-1);
    sum3 = sum3 + a2(i).*theta(i).^2.*exp(-tau.*theta(i))...
        .*(1 -exp(-tau.*theta(i))).^(-2);
end

% calculate phi and derivatives
phi0 = log(delta) + a1(1) + a1(2).*tau + a1(3).* log(tau) + sum1;  
dphi0dtau = a1(2) + a1(3)./tau + sum2;
dphi0dtautau = - a1(3)./tau.^2 - sum3;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [phir, dphirdtau, dphirdtautau, dphirddeltatau, dphirddelta, dphirddeltadelta] = helmholtz_residual(delta,tau)

%  calculation of the residual part (phir) of the Helmholtz energy and its
%  first and second derivatives with respect to the inverse reduced temperature
%  (dphirdtau and dphirdtautau) and first and second derivatives with respect to
%  the reduced density (dphirddelta, dphirddeltadelta) as well as 
%  the mixed derivative (dphirddeltadtau)

%----------------------------------------------------------------------
% calculation of sum terms with parameters ni,di,ti
%----------------------------------------------------------------------

ni = [  0.12533547935523e-1
         0.78957634722828e1
        -0.87803203303561e1
         0.31802509345418
        -0.26145533859358
        -0.78199751687981e-2
         0.88089493102134e-2];
  
di = [1 1 1 2 2 3 4];
ti = [-0.5 0.875 1 0.5 0.75 0.375 1];

sumr1   =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrd1  =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrdd1 =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrt1  =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrtt1 =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrdt1 =  zeros(size(tau,1),size(tau,2),size(tau,3));

for i=1:7
    sumr1   = sumr1 + ni(i).*delta.^di(i).*tau.^ti(i);
    sumrd1  = sumrd1 + ni(i).*di(i).*delta.^(di(i)-1).*tau.^ti(i);
    sumrdd1 = sumrdd1 + ni(i).*di(i).*(di(i)-1).*delta.^(di(i)-2).*tau.^ti(i);
    sumrt1  = sumrt1 + ni(i).*ti(i).*delta.^di(i).*tau.^(ti(i)-1);
    sumrtt1 = sumrtt1 + ni(i).*ti(i).*(ti(i)-1).*delta.^di(i).*tau.^(ti(i)-2);
    sumrdt1 = sumrdt1 + ni(i).*ti(i).*di(i).*delta.^(di(i)-1).*tau.^(ti(i)-1);
end


%----------------------------------------------------------------------
% calculation of sum terms with parameters nj,dj,tj,cj
%----------------------------------------------------------------------

nj = [  -0.66856572307965
         0.20433810950965
        -0.66212605039687e-4
        -0.19232721156002
        -0.25709043003438
         0.16074868486251
        -0.40092828925807e-1
         0.39343422603254e-6
        -0.75941377088144e-5
         0.56250979351888e-3
        -0.15608652257135e-4
         0.11537996422951e-8
         0.36582165144204e-6
        -0.13251180074668e-11
        -0.62639586912454e-9
        -0.10793600908932
         0.17611491008752e-1
         0.22132295167546
        -0.40247669763528
         0.58083399985759
         0.49969146990806e-2
        -0.31358700712549e-1
        -0.74315929710341
         0.47807329915480
         0.20527940895948e-1
        -0.13636435110343
         0.14180634400617e-1
         0.83326504880713e-2
        -0.29052336009585e-1
         0.38615085574206e-1
        -0.20393486513704e-1
        -0.16554050063734e-2
         0.19955571979541e-2
         0.15870308324157e-3
        -0.16388568342530e-4
         0.43613615723811e-1
         0.34994005463765e-1
        -0.76788197844621e-1
         0.22446277332006e-1
        -0.62689710414685e-4
        -0.55711118565645e-9
        -0.19905718354408
         0.31777497330738
        -0.11841182425981];
 
dj = [1 1 1 2 2 3 4 4 5 7 9 10 11 13 15 1 2 2 2 3 4 4 4 5 6 6 7 ...
    9 9 9 9 9 10 10 12 3 4 4 5 14 3 6 6 6];
tj = [4 6 12 1 5 4 2 13 9 3 4 11 4 13 1 7 1 9 10 10 3 7 10 10 6 10 10 ...
    1 2 3 4 8 6 9 8 16 22 23 23 10 50 44 46 50];
cj = [1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 ...
    2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 3 3 3 3 4 6 6 6 6];

sumr2   =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrd2  =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrdd2 =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrt2  =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrtt2 =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrdt2 =  zeros(size(tau,1),size(tau,2),size(tau,3));

for i=1:44
    sumr2 = sumr2 + nj(i).*delta.^dj(i).*tau.^tj(i).*exp(-delta.^cj(i));
    sumrd2 = sumrd2 + nj(i).*delta.^(dj(i)-1).*tau.^tj(i)....
        .*exp(-delta.^cj(i)).*(dj(i)-cj(i).*delta.^cj(i));
    sumrdd2 = sumrdd2 + nj(i).*delta.^(dj(i)-2).*tau.^tj(i)...
        .*exp(-delta.^cj(i)).*((dj(i)-cj(i).*delta.^cj(i))...
        .*(dj(i)-1-cj(i).*delta.^cj(i))-cj(i)^2.*delta.^cj(i));
    sumrt2 = sumrt2 + nj(i).*delta.^dj(i).*tau.^(tj(i)-1).*tj(i)...
        .*exp(-delta.^cj(i));
    sumrtt2 = sumrtt2 + nj(i).*delta.^dj(i).*tau.^(tj(i)-2).*tj(i).*(tj(i)-1)...
        .*exp(-delta.^cj(i));
    sumrdt2 = sumrdt2 + nj(i).*delta.^(dj(i)-1).*tau.^(tj(i)-1).*tj(i)...
        .*exp(-delta.^cj(i)).*(dj(i)-cj(i).*delta.^cj(i));
end

%----------------------------------------------------------------------
% calculation of sum terms with parameters nk,dk,tk,ak,bk,ck,ek
%----------------------------------------------------------------------

nk = [-0.31306260323435e2
       0.31546140237781e2
      -0.25213154341695e4];

dk = [3 3 3];
tk = [0 1 4];
ak = [20 20 20];
bk = [150 150 250];
ck = [1.21 1.21 1.25];
ek = [1. 1. 1.];

sumr3   =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrd3  =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrdd3 =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrt3  =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrtt3 =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrdt3 =  zeros(size(tau,1),size(tau,2),size(tau,3));

for i=1:3
    sumr3 = sumr3 + nk(i).*delta.^dk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2);
    sumrd3 = sumrd3 + nk(i).*delta.^dk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2)...
        .*(dk(i)./delta - 2.*ak(i).*(delta -ek(i)));
     sumrdd3 = sumrdd3 + nk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2)...
        .*(- 2.*ak(i).*delta.^dk(i) + 4.*ak(i)^2.*delta.^dk(i)...
        .*(delta - ek(i)).^2 - 4.*dk(i).*ak(i).*delta.^(dk(i)-1)...
        .*(delta - ek(i)) + dk(i).*(dk(i)-1).*delta.^(dk(i)-2));
     sumrt3 = sumrt3 + nk(i).*delta.^dk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2)...
        .*(tk(i)./tau - 2.*bk(i).*(tau -ck(i)));
     sumrtt3 = sumrtt3 + nk(i).*delta.^dk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2)...
        .*((tk(i)./tau - 2.*bk(i).*(tau -ck(i))).^2 - tk(i)./tau.^2 - 2.*bk(i));
     sumrdt3 = sumrdt3 + nk(i).*delta.^dk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2)...
        .*(tk(i)./tau - 2.*bk(i).*(tau -ck(i)))...
        .*(dk(i)./delta - 2.*ak(i).*(delta -ek(i)));
end

%----------------------------------------------------------------------
% calculation of sum terms with parameters nl,al,bl,betl,Al,Bl,Cl,Dl
%----------------------------------------------------------------------

nl = [-0.14874640856724
       0.31806110878444];
al = [3.5 3.5];
bl = [0.85 0.95];
betl = [0.3 0.3];
Al = [0.32 0.32];
Bl = [0.2 0.2];
Cl = [28 32];
Dl = [700 800];

sumr4 =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrd4 =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrdd4 =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrt4 =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrtt4 =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrdt4 =  zeros(size(tau,1),size(tau,2),size(tau,3));

for i=1:2
    % calculation of terms and their derivatives which are frequently used in the sums 
    psi = exp(-Cl(i).*(delta-1).^2 - Dl(i).*(tau -1).^2);
    dpsidd = -2.*Cl(i).*(delta -1).*psi;                 % d(psi)/d(delta)
    dpsiddd = (2.*Cl(i).*(delta -1).^2-1).*2.*Cl(i).*psi;% d^2(psi)/d(delta^2)
    dpsidt = -2.*Dl(i).*(tau -1).*psi;                   % d(psi)/d(tau)
    dpsidtt = (2.*Dl(i).*(tau -1).^2-1).*2.*Dl(i).*psi;  % d^2(psi)/d(tau^2)
    dpsiddt = 4.*Cl(i).*Dl(i).*(delta -1).*(tau - 1).*psi;% d^2(psi)/d(delta)d(tau)
    
    theta = (1 - tau) + Al(i).*((delta -1).^2).^(1./(2.*betl(i)));
    
    Delta = theta.^2 + Bl(i).*((delta - 1).^2).^al(i);
    dDeltadd = (delta -1)...
        .*(Al(i).*theta.*2./betl(i).*((delta-1).^2).^(1./(2.*betl(i))-1)...
        + 2.*Bl(i).*al(i).*((delta -1).^2).^(al(i)-1)); % d(Delta)/d(delta)
    dDeltaddd = 1./(delta -1).*dDeltadd + (delta - 1).^2.*...
        (4.*Bl(i).*al(i).*(al(i)-1).*((delta - 1).^2).^(al(i)-2)...
        + 2.*Al(i).^2 .* (1./betl(i).^2) .* ...
        (((delta - 1).^2).^(1./2./betl(i)-1)).^2 + ...
        Al(i).*theta.*4./betl(i).*(1./2./betl(i) -1) .*...
        ((delta - 1).^2).^(1./2./betl(i) - 2)); % d^2(Delta)/d(delta^2)
    
    Deltab = Delta.^bl(i);
    dDeltabdd = bl(i).*Delta.^(bl(i)-1).*dDeltadd; % d(Deltab)/d(delta)
    dDeltabddd = bl(i).*(Delta.^(bl(i)-1).*dDeltaddd + ...
        (bl(i)-1).*Delta.^(bl(i)-2).*dDeltadd.^2); % d^2(Delatb)/d(delta^2)
    dDeltabdt = -2.*theta.*bl(i).*Delta.^(bl(i)-1); % d(Deltab)/d(tau)
    dDeltabdtt = 2.*bl(i).*Delta.^(bl(i)-1) + ...
        4 .* theta.^2 .* bl(i).*(bl(i)-1).*Delta.^(bl(i)-2); % d^2(Deltab)/d(tau^2)
    dDeltabddt = - Al(i).*bl(i).*2./betl(i).*Delta.^(bl(i)-1).*(delta-1)...
        .*((delta-1).^2).^(1./2./betl(i)-1) - ...
        2.*theta.*bl(i).*(bl(i)-1).*Delta.^(bl(i)-2).*dDeltadd;% d^2(Deltab)/d(delta)d(tau)
    
    sumr4 = sumr4 + nl(i).*Deltab.*delta.*psi;
    sumrd4 = sumrd4 + nl(i).*(Deltab .*(psi + delta .* dpsidd) ...
        + dDeltabdd.*delta.*psi);
    sumrdd4 = sumrdd4 + nl(i).*(Deltab.*(2.*dpsidd + delta.*dpsiddd)...
        + 2.*dDeltabdd.*(psi + delta.*dpsidd) + dDeltabddd.*delta.*psi);
    sumrt4 = sumrt4 + nl(i).*delta.*(dDeltabdt.*psi + Deltab.*dpsidt);
    sumrtt4 = sumrtt4 + nl(i).* delta .*(dDeltabdtt .* psi ...
        + 2.*dDeltabdt .* dpsidt + Deltab .* dpsidtt);
    sumrdt4 = sumrdt4 + nl(i).*(Deltab .*(dpsidt + delta.*dpsiddt) + ...
        delta.*dDeltabdd .* dpsidt + dDeltabdt.*(psi + delta .* dpsidd) ...
        + dDeltabddt .* delta .* psi);
    
end

%----------------------------------------------------------------------
% calculation of function output values
%----------------------------------------------------------------------
phir = sumr1 + sumr2 + sumr3 + sumr4; % phi_r: residual of Helmholtz energy
dphirddelta = sumrd1 + sumrd2 + sumrd3 + sumrd4;        % d(phi_r)/d(delta)
dphirddeltadelta = sumrdd1 + sumrdd2 + sumrdd3 + sumrdd4; % d^2(phi_r)/d(delta^2)
dphirdtau = sumrt1 + sumrt2 + sumrt3 + sumrt4;          % d(phi_r)/d(tau)
dphirdtautau = sumrtt1 + sumrtt2 + sumrtt3 + sumrtt4;   % d^2(phi_r)/d(tau^2)
dphirddeltatau = sumrdt1 + sumrdt2 + sumrdt3 + sumrdt4; % d^2(phi_r)/d(delta)d(tau)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function f = minfunc(rho_pw,P,T,rhoc,R,tau)
% minimization function to calculate density of pure water
% based on the equation p = rho*R*T*(1 + rho/rohc * d(phir)/d(rho/rhoc)) 

delta = rho_pw/rhoc;

% calculate d(phir)/d(rho/rhoc)
[~, ~ , ~, ~, dphirddelta] = helmholtz_residual(delta,tau);

% calculate density
RHOeos = P ./ (1000.*R.*T.*(1+delta.*dphirddelta));

% calculate difference between initial rho and "new" calculated rho
f = rho_pw - RHOeos;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function rho = guessfunc(press,temp) 
%   The formulation of the International Association for the Properties of Water and Steam (IAPSW)
%       (2007) ("Revised release for the IAPWS industrial formlation 1997 for
%       the thermodynamic properties of water and steam", IAPWS-IF) is used to
%       calculate a  start value for the iteration of the density. The
%       formalism is valid for 0 < T < 800C and 0 < P < 100 MPa. It
%       includes equations for water densities in the liquid region (region
%       1) and the gas and supercritical-gaslike region (region 2) of the phase diagram. 
%       Nevertheless, there is no direct equation for the water density in
%       teh supercritical liquid-like region (region 3). Therefore, starting values
%       for region 3 are calculated from the equation of Y. Marcus (PCCP2,
%       2002).
%       Since the calculated density is only considers as a start value and does only
%       need to be approximate but not exact, the formulation might be used
%       for 0 < T < 1400C and P < 167.5 MPa or 0 < T < 227C and p < 225 MPa

[P,T] = ndgrid(press,temp+273.15); %  temperature in K
R = 0.461526; % specific gas constant [kJ/(kg K)]

% rhoc = double(322); % critical density of pure water [kg/m^3]
% Tc = double(647.096); % critical temperature of pure water [K]
% tau = Tc./T; % inverse of reduced temperature

%------------------------------------------------------------------------
% Calculation of phase boundary pressure p_b1 
%------------------------------------------------------------------------
n = zeros(8,1);

n(1) = 0.11670521452767d4;     n(6) = 0.14915108613530d2;
n(2) = -0.72421316703206d6;    n(7) = -0.48232657361591d4;
n(3) = -0.17073846940092d2;    n(8) = 0.40511340542057d6;
n(4) = 0.12020824702470d5;     n(9) = -0.23855557567849;
n(5) = -0.32325550322333d7;    n(10) = 0.65017534844798d3;

theta = T + n(9)./(T - n(10));
A = theta.^2 + n(1).*theta + n(2);
B = n(3).*theta.^2 + n(4).*theta + n(5);
C = n(6).*theta.^2 + n(7).*theta + n(8);

p_b1 = (2.*C./(-B + (B.^2 - 4.*A.*C).^(0.5))).^4;

%------------------------------------------------------------------------
% Calculation of "phase boundary pressure" p_b2 above the citical point
%------------------------------------------------------------------------

p_b2 = 0.34805185628969d3 - 0.11671859879975d1.*T + 0.10192970039326d-2.*T.^2;

%------------------------------------------------------------------------
% Calculation of "phase boundary" for the given input temperatures
%------------------------------------------------------------------------

% depending on the temperature values, decide, which equation to use 

decide = T > 623.15;

p_b = p_b2.*decide - p_b1.*(decide-1);

%------------------------------------------------------------------------
% calculation of specific volume in region 1 (liquid, T< 623.15 K)
%------------------------------------------------------------------------

ps = 16.53; % reference pressure [MPa]
Ts = 1386; % reference temerature [K]

p_i = P./ps; % dimensionless pressure
tau = Ts./T; % dimensionless temperature

% first exponent 
I = [0 0 0 0 0 0 0 0 1 1 1 1 1 1 2 2 2 2 2 3 3 3 4 4 4 5 8 8 21 23 29 30 31 32];

% second exponent 
J = [-2 -1 0 1 2 3 4 5 -9 -7 -1 0 1 3 -3 0 1 3 17 -4 0 6 -5 -2 10 -8 -11 ...
    -6 -29 -31 -38 -39 -40 -41];

n = zeros(34,1);

n(1)  =  0.14632971213167;
n(18) = -0.44141845330846d-5;
n(2) = -0.84548187169114 ;
n(19)= -0.72694996297594d-15;
n(3) = -0.3756303672040d1 ;
n(20)= -0.31679644845054d-4;
n(4) = 0.33855169168385d1 ;
n(21)= -0.28270797985312d-5;
n(5) = -0.95791963387872;
n(22)= -0.85205128120103d-9;
n(6) = 0.15772038513228;
n(23)= -0.22425281908000d-5;
n(7) = -0.16616417199501d-1 ;
n(24)= -0.65171222895601d-6;
n(8) = 0.81214629983568d-3;
n(25)= -0.14341729937924d-12;
n(9) = 0.28319080123804d-3;
n(26)= -0.40516996860117d-6;
n(10)= -0.60706301565874d-3 ;
n(27)= -0.12734301741641d-8;
n(11)= -0.18990068218419d-1 ;
n(28)= -0.17424871230634d-9;
n(12)= -0.32529748770505d-1 ;
n(29)= -0.68762131295531d-18;
n(13)= -0.21841717175414d-1 ;
n(30)= 0.14478307828521d-19;
n(14)= -0.52838357969930d-4 ;
n(31)= 0.26335781662795d-22;
n(15)= -0.47184321073267d-3 ;
n(32)= -0.11947622640071d-22;
n(16)= -0.30001780793026d-3 ;
n(33)= 0.18228094581404d-23;
n(17)= 0.47661393906987d-4 ;
n(34)= -0.93537087292458d-25;

% derivative of the dimensionless Gibbs energy with respect to pressure

g_p = zeros(size(P)); % empty matrix

for i=1:34
    g_p = g_p - n(i).*I(i).*(7.1-p_i).^(I(i)-1).*(tau-1.222).^J(i);
end

% specific volume in m^3/kg
v1 = R.*1000.*T./(P.*1d6).*p_i.*g_p;


%------------------------------------------------------------------------
% calculation of specific volume in region 2 (gas, T< 1073.15K)
%------------------------------------------------------------------------

ps = 1; % reference pressure [MPa]
Ts = 540; % reference temerature [K]

p_i = P./ps; % dimensionless pressure
tau = Ts./T; % dimensionless temperature

g_p_0 = 1./p_i; % ideal gas part of dimensionless Gibbs free energy

I = zeros(43,1);
J = zeros(43,1);
n = zeros(43,1);

I(1) = 1;    J(1) = 0;  n(1) = -0.17731742473213d-2;
I(2) = 1;    J(2) = 1;  n(2) = -0.17834862292358d-1;
I(3) = 1;    J(3) = 2;  n(3) = -0.45996013696365d-1;
I(4) = 1;    J(4) = 3;  n(4) = -0.57581259083432d-1;
I(5) = 1;    J(5) = 6;  n(5) = -0.50325278727930d-1;
I(6) = 2;    J(6) = 1;  n(6) = -0.3303264167020d-4;
I(7)= 2;     J(7) =2;   n(7) =-0.18948987516315d-3;
I(8)= 2;     J(8) =4;   n(8) =-0.39392777243355d-2;
I(9)= 2;     J(9) =7;   n(9) =-0.43797295650573d-1;
I(10)= 2;    J(10) =36; n(10) =-0.26674547914087d-4;
I(11)= 3;    J(11) =0;  n(11) =0.20481737692309d-7;
I(12)= 3;    J(12) =1;  n(12) =0.43870667284435d-6;
I(13)= 3;    J(13) = 3; n(13) =-0.32277677238570d-4;
I(14)= 3;    J(14) =6;  n(14) =-0.15033924542148d-2;
I(15)= 3;    J(15) =35; n(15) =-0.40668253562649d-1;
I(16)= 4;    J(16) =1;   n(16) =-0.78847309559367d-9;
I(17)= 4;    J(17) =2;   n(17) =0.12790717852285d-7;
I(18)= 4;    J(18) =3;   n(18) =0.48225372718507d-6;
I(19)= 5;    J(19) =7;   n(19) =0.22922076337661d-5;
I(20)= 6;    J(20) =3;   n(20) =-0.16714766451061d-10;
I(21)= 6;    J(21) =16;  n(21) =-0.21171472321355d-2;
I(22)= 6;    J(22) =35;  n(22) =-0.23895741934104d2;
I(23)= 7;    J(23) =0;   n(23) =-0.59059564324270d-17;
I(24)= 7;    J(24) =11;  n(24) =-0.12621808899101d-5;
I(25)= 7;    J(25) =25;  n(25) =-0.38946842435739d-1;
I(26)= 8;    J(26) =8;   n(62) =0.11256211360459d-10;
I(27)= 8;    J(27) =36;  n(27) =-0.82311340897998d1;
I(28)= 9;    J(28) =13;  n(28) =0.19809712802088d-7;
I(29)= 10;   J(29) =4;   n(29) =0.10406965210174d-18;
I(30)= 10;   J(30) =10;  n(30) =-0.10234747095929d-12;
I(31)= 10;   J(31) =14;  n(31) =-0.10018179379511d-8;
I(32)= 16;   J(32) =29;  n(32) =-0.80882908646985d-10;
I(33)= 16;   J(33) =50;  n(33) =0.10693031879409;
I(34)= 18;   J(34) =57;  n(34) =-0.33662250574171;
I(35)= 20;   J(35) =20;  n(35) = 0.89185845355421d-24;
I(36)= 20;   J(36) =35;  n(36) =0.30629316876232d-12;
I(37)= 20;   J(37) =48;  n(37) =-0.42002467698208d-5;
I(38)= 21;   J(38) =21;  n(38) =-0.59056029685639d-25;
I(39)= 22;   J(39) =53;  n(39) =0.37826947613457d-5;
I(40)= 23;   J(40) =39;  n(40) =-0.12768608934681d-14;
I(41)= 24;   J(41) =26;  n(41) =0.73087610595061d-28;
I(42)= 24;   J(42) =40;  n(42) =0.55414715350778d-16;
I(43)= 24;   J(43) =58;  n(43) =-0.94369707241210d-6;

g_p_r = zeros(size(P)); % empty matrix for residual Gibbs free energy

for i=1:43
    g_p_r = g_p_r + n(i).*I(i).*(p_i).^(I(i)-1).*(tau-0.5).^J(i);
end

% specific volume in m^3/kg
v2 = R.*1000.*T./(P.*1d6).*p_i.*(g_p_0 + g_p_r);

%------------------------------------------------------------------------
% calculation of specific volume in region 3 using the equation of Marcus.
% Marcus' equations are valid for 720K < T < 880K and 40MPa < p < 100 MPa
% and are assumed by Marcus to be reasonable for 670K < T < 880K and 
% 35 MPa < p < 100 MPa. A visual check reveals that the density at lower
% temperatures (< 700K) looks good at low pressure (< 50 MPa).
%------------------------------------------------------------------------

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

RM = 8.31451;% Gas constant [J/mol/K]
T0 = 637; % reference temperature [K]
PM = P; % pressure in MPa

alpha0 = -1.7e-5 + 0.44.*PM.^(-2);
alpha1 = -0.2e-5 + 0.55.*PM.^(-2);

E = 18 + 0.028.*PM.^2;
p_M = 1 - exp(-E./RM./(T-T0)); % probability of bond formation

%------------------------------------------------------------------------
% molar volumes for water with different number of hydrogen bonds
%------------------------------------------------------------------------

V00 = 27.1 - 0.27.*PM;
V01 = 29.6 - 0.072.*PM;
V02 = 3/2*V01;

V0 = V00.*(1 + alpha0.*(T - T0).^2);
V1 = V01.*(1 + alpha1.*(T - T0).^2);
V2 = V02.*(1 + alpha1.*(T - T0).^2);

%------------------------------------------------------------------------
% fraction of water with different number of hydrogen bonds
%------------------------------------------------------------------------

f0 = 1./(1 + 4.*p_M./(1 - p_M) + 6.*p_M.^2./(1 - p_M).^2);
f1 = 4.*p_M.*f0./(1 - p_M);
f2 = 6.*p_M.^2.*f0./(1 - p_M).^2;

%------------------------------------------------------------------------
% specific volume calculated from molar volume fractions for different
% numbers of hydrogen bonds
%------------------------------------------------------------------------

Vmol = f0.*V0 + f1.*V1 + f2.*V2; % molar volume [cm^3/mol]

v_Marcus = Vmol./18.015./1000; % specific volume [m^3/kg]

v_Marcus(isnan(v_Marcus))=0;

%------------------------------------------------------------------------
% determine, which region applies
%------------------------------------------------------------------------

% first: distinguish between region 1 and region 3 (Marcus' formalism)

% use Marcus for temperatures > 720K
decide = T > 720;

v1 = -v1.*(decide-1) + v_Marcus.*(decide);

% at pressures below 50 Mpa, use Marcus for temperatures > 660 K

decide_p = P < 500;
decide_T = T > 660;

decide = decide_p.*decide_T;

v1 = -v1.*(decide-1) + v_Marcus.*(decide);

% second: distinguish between liquid like (2 and 3) and gas like (1)
% regions


% decide, if given p/T combinations are in liquid or vapor regime
decide = P >= p_b;


v = v1.*decide - v2.*(decide-1);

rho = 1./v; % density in kg/m^3