function [rho, phi] = eos_h2s(press,temp)

% EOS_H2S -  calculates the density and fugacity coefficient of H2S 
%     from a cubic equation of state
%   
%     Usage:  [rho phi] = eos_h2s(p,T)
%
%     Input parameters (scalars or vectors): 
%           p        -  pressure [MPa]     (0.1 < p < 35)
%           T        -  temperature [degC] (0 < T < 137.7)
%     
%     Output parameters: 
%           rho      - density [kg/m3]
%           phi      - fugacity coefficient
%
%     References: 
%     Duan et al. (2007) Energy & Fuels 21, 2056-2065.
%     Reamer et al. (1950) Ind. Eng. Chem. 1, 140-143. (lookup table)
%        
% -----------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (January 2013)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% -----------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('EOS_H2S:Input','EOS_H2S: Missing input parameter! Please provide p,T,S and pp(optional)!')
else
 
%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>35) && ~any(temp<0 | temp>137.7)

else
    bad = find(press<0.1 | press > 35);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>137.7);
    temp(bad)=NaN*ones(size(bad));
    warning('EOS_H2S:OutOfParameterRange','EOS_H2S: Input values out of range!')
    warning('EOS_H2S:OutOfParameterRange','Equations valid for p = 0.1-35 MPa, T= 0-137.7 degC.')
end


%------------------------------------------------------------------------
% Definition of constants and thermodynamic parameters
%------------------------------------------------------------------------


[P, T] = ndgrid(press,temp+273.15); % temperature in [K]

MM = 34.08d-3; % molar mass of H2S [kg/mol]

%------------------------------------------------------------------------
% solve equation of state for volume of H2S
%------------------------------------------------------------------------

V = zeros(size(P)); % empty matrix for Vm
options=optimset('Display','off','TolFun',1e-16); % set options for fitfunction

for i=1:size(P,1)
    for j=1:size(P,2)
        if isnan(press(i)) || isnan(temp(j))
            V(i,j) = NaN;
        else  
            V_init = guessfunc(press(i),temp(j)); % starting value for solver
            % calculate volume by numerical iteration
            V(i,j)= fsolve(@(V) minfunc(V,P(i,j)*10,T(i,j)), V_init,options);
        end
    end
end

rho = MM./V*1d5; % denisty in kg/m^3
[~, phi] = eqofstate_h2s(T,V); % fugacity coefficient of gaseous H2S
end

% ------------------------------------------------------------
function [Pvh2s] = pvaph2s(T)

%------------------------------------------------------------------------
% Definition of constants and thermodynamic parameters
%------------------------------------------------------------------------

Tc = 373.1; % critical temperature of H2S [K]
Pc = 8.9987; % critical pressure of H2S [MPa]
Tt = 187.6991; % triple point temperature of H2S [K]
Pt = 23.2d-3; % triple point pressure of H2S [MPa]
Ptr = Pt/Pc; % reduced pressure at triple point
Ttr = Tt/Tc; % reduced temperature at triple point
Tbr = 0.57049;
Tr = T./Tc; % reduced temperature
t = (T - Tt)./(Tc -Tt);
tb = (Tbr - Ttr)./(1 - Ttr);

a1 = 0.090204;
b0 = -0.154;

phi = (1 - t)./(1 + a1.*t)./(1 + b0.*t.*(t - tb));

lnpr = phi.*Ttr.*log(Ptr)./Tr;
Pvh2s = exp(lnpr)*Pc;

% ------------------------------------------------------------
function [Vinit] = guessfunc(press,temp)

% this function determines a starting value for the molar volume

R = 8.314472; % gas constant in [Pa*m^3/mol/K] 
Pv = pvaph2s(temp + 273.15); % vapor pressure of H2S at T = temp

tguess = [40 100 160 220 280]; % temperature values for lookup table [F]
tguess = (tguess-32).*5./9; % temperature values for lookup table [C]

Vguess = [0.65 0.71 0.8 0.95 1.5];

if (press > Pv)
    if temp <= tguess(1)
        Vinit = Vguess(1);
        Vinit = Vinit.*28.316846592./453.59237*1d2; % molar volume in Duan's strange unit
    else
        f = find(tguess < temp);
        f = f(end);
        fraca = (temp - tguess(f))/(tguess(f+1)-tguess(f));
        fracb = (tguess(f+1) - temp)/(tguess(f+1)-tguess(f));
        Vinit = fraca*Vguess(f) + fracb*Vguess(f+1); % linear interpolation
        Vinit = Vinit.*28.316846592./453.59237*1d2; % molar volume in Duan's strange unit
    end
else
    Vinit = R*(temp + 273.15)/(press*10); % ideal gas law , T in [K], P in [bar]
end

%------------------------------------------------------------------------
function [Z, phi] = eqofstate_h2s(T,V)
%calculates the gas phase fugacity of H2S

%------------------------------------------------------------------------
% Definition of constants and thermodynamic parameters
%------------------------------------------------------------------------

R = 8.314467; % gas constant [Pa m^3/K/mol]
Tc = 373.6; % critical temperature [K]
Pc = 90.08; % critical pressure [bar]
Vc = R*Tc./Pc;  % pseudo-critical volume

Tr = T./Tc;     % reduced temperature
Vr = V./Vc;  % reduced volume

a1 =5.2386075d-2;
a2 = -2.7463906d-1;
a3 = -9.6760173d-2;
a4 = 1.3618104d-2;
a5 = -8.8681753d-2;
a6 = 4.1176908d-2;
a7 =3.6354018d-4;
a8 = 2.2719194d-3;
a9 = -7.6962514d-4;
a10 = -2.1948579d-5;
a11 = -1.1707631d-4;
a12 = 4.0756926d-5;
a13 = 5.7582260d-2;
a14 = 1.00;
a15 =0.06;

T1 = (a1 + a2./Tr.^2 + a3./Tr.^3)./Vr ;
T2 = (a4 + a5./Tr.^2 + a6./Tr.^3)./Vr.^2;
T3 = (a7 + a8./Tr.^2 + a9./Tr.^3)./Vr.^4;
T4 = (a10 + a11./Tr.^2 + a12./Tr.^3)./Vr.^5;

%------------------------------------------------------------------------
% Calculation of Z and fugacity coefficient phi
%------------------------------------------------------------------------

Z = 1 + T1 + T2 + T3 + T4 + ...
    a13./Tr.^3./Vr.^2.*(a14 + a15./Vr.^2).*exp(-a15./Vr.^2);

phi = exp(Z - 1 - log(Z) + T1 + T2./2 + T3./4 + T4./5 +...
     a13./2./Tr.^3./a15.*(a14 + 1 - (a14 + 1 + a15./Vr.^2).*exp(-a15./Vr.^2)));

%------------------------------------------------------------------------
function f = minfunc(V,P,T)
%minimization function for calculation of Vm

R = 8.314467; % gas constant [Pa m^3/K/mol]
Tc = 373.6; % critical temperature [K]
Pc = 90.08; % critical pressure [bar]
Vc = R*Tc./Pc;  % pseudo-critical volume

Tr = T./Tc;  % reduced temperature
Vr = V./Vc;  % reduced volume
Pr = P./Pc;  % reduced pressure

% call external function to calculate Z
Z = eqofstate_h2s(T,V);
f = Z - Pr.*Vr./Tr;
