function [rho_sw, cp_sw, s_sw, u_sw, h_sw, phi_sw, mu_sw, mu_ss, alpha_sw, beta_sw, kappaT_sw, kappas_sw, beta_hal, fw] = eos_sw(press,temp,sal)

% EOS_SW - calculates thermodynamic properties of seawater based on
%     an equation of state
%
%     Usage: [rho cp s u h phi mu_sw mu_ss alpha beta kappaT kappaS beta_hal fw] 
%                = eos_sw(p,T,S)
% 
%     Input parameters (scalars or vectors): 
%         p         - pressure [MPa]     (0 < p < 100), 
%         T         - temperature [degC] (-12 (0)< T < 80),
%         S         - practical salinity (0 < S < 120)
%
%     Output parameters:
%         rho       - Density [kg/m^3]
%         cp        - Specific isobaric heat capacity [J/(kg K)]
%         s         - Specific entropy [J/(kg K)] 
%         u         - Specific internal energy [J/kg]
%         h         - Specific enthalpy [J/kg] 
%         phi       - Specific Helmholtz energy [J/kg]
%         mu_sw     - Relative chemical potential of seawater [J/kg]
%         mu_ss     - Chemical potential of sea salt [J/kg]
%         alpha     - Thermal expansion coefficient [1/K]
%         beta      - Isentropic temperature-pressure coefficient [K/Pa]
%         kappaT    - Isothermal compressibility [1/Pa]
%         kappaS    - Isentropic compressibility [1/Pa]
%         beta_hal  - Haline contraction coefficient [kg/kg]
%         fw        - fugacity coefficient of pure(!) water 
%
%     All thermodynamic properties are calculated from the Gibbs energy
%     (g = u + pV - Ts) and its derivatives with respect to p, T and S. 
%     The Gibbs energy of seawater is taken as the sum of the Gibbs energy 
%     of pure water and the Gibbs energy of the saline part. 
%     The pure water properties are first calculated from a formalism based
%     on the Helmholtz energy (a = u - Ts) as given in IAPWS (1995).
%     The Gibbs energy of pure water and its derivatives are then derived 
%     from those calculated thermodynamic potentials. 
%     The saline contribution to the Gibbs energy is directly calculated
%     from the formalism given in IAPWS (2008).
%
%     References: 
%     IAPSW (1996) The IAPWS formulation 1995 for the thermodynamic 
%       properties of ordinary water substance for general and scientific
%       use. www.iapws.org
%     Wagner & Pruss (2002) J.Phys.Chem.Ref.Data 31, 387-535.
%     IAPSW (2008) Release on the IAPWS formulation 2008 for the 
%       thermodynamic properties of seawater. www.iapws.org 
%     Feistel (2008) DSR I 55, 16391671. 
%
%     Comments: 
%     Not valid for ice phase!
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       November 2009: array size of g corrected (line 656)
%       December 2009: %  Parameter names in header corrected (lines 17-18)
%       September 2011: Name changed from eos_seawater to eos_sw
%       November 2012: Name in function call changed (unesco_sw ->
%       density_swUnesco
%       November 2012: adapted to Matlab2012 syntax
%       January 2013: use density_swSun for generation of solver starting
%       value (instead of density_swUNESCO
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('EOS_SW:Input','EOS_SW: Missing input parameter! Please provide p, T and S!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------
if ~any(press<0 | press>100) && ~any(temp< -12 | temp>80) ...
        && ~any(sal < 0 | sal>120)
else
    bad = find(press<0 | press > 100);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<-12 | temp>80);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>120);
    sal(bad)=NaN*ones(size(bad));
    warning('EOS_SW:OutOfParameterRange','EOS_SW: Input values out of range!')
    warning('EOS_SW:OutOfParameterRange','Equations valid for p=0-100 MPa, T=(-12)-80 degC, S=0-120.')
end


%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------
[P,T,S] = ndgrid(press*1e6,temp+273.15,sal*1.005); % pressure in Pa, temperature in K, absolute salinity

R = 0.46151805; % specific gas constant [kJ/(kg K)]


%------------------------------------------------------------------------
%------------------------------------------------------------------------
% CALCULATION OF PURE WATER CONTRIBUTION
%------------------------------------------------------------------------
%------------------------------------------------------------------------

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------
rhoc = double(322); % critical density of pure water [kg/m^3]
Tc = double(647.096); % critical temperature of pure water [K]
tau = Tc./T; % inverse of reduced temperature

%------------------------------------------------------------------------
% calculation of density of pure water by minimizing the thermodynamic
% equation p = rho * R * T * (1 + d(phi_r)/d(delta))
%------------------------------------------------------------------------

% define empty data structure for density of pure water
rho_pw = zeros(size(P,1),size(P,2),size(P,3)); 

% Check if arrays for P and T contain NaN. If yes, also set corresponding 
% density values to NaN. Calculate density for allowed P and T only.
% If the fit function returns EXITFLAG <= 0, the iteration process has
% terminated with a non-satisfying result. In this case, the initial value
% is changed and the fit procedure is repeated.

warning off 'UNESCO_SW:OutOfParameterRange'
options=optimset('Display','off','TolFun',1e-16); % set options for fitfunction

for i=1:size(P,1)
    for j=1:size(P,2)
        if isnan(P(i,:,:));
            rho_pw(i,:,:) = NaN;
        elseif isnan(T(:,j,:));
            rho_pw(:,j,:) = NaN;
        else  
            EXITFLAG = -1; % Initialize EXITFLAG value
            counter = 0; % set counter to 0
            % use internal function to look up first guess for density
            if P(i,j,1) < 100*1e6 && T(i,j,1) < 310
                rho_init1 = density_swSun(press(i),temp(j),0);
            else
            rho_init1 = guessfunc(T(1,j,1));
            end
            rho_init = rho_init1; % first initial value for density
            while (EXITFLAG < 1) 
                counter = counter + 1;
                [rho_pw(i,j,:) , ~, EXITFLAG] = fsolve(@(rho_pwat) minfunc(rho_pwat,P(i,j,1),T(i,j,1),rhoc,R,tau(i,j,1)), rho_init,options);
                % change initial value for fit
                rho_init = rho_init1 + counter*1*(-1)^(counter); 
                 if rho_pw(i,j,1) <= 0
                    EXITFLAG = -1; % reject negative densities  
                end
                if rho_init <= 0
                    EXITFLAG = 10; % make sure, that while loop terminates  
                end
            end
        end
    end
end

delta_pw = rho_pw./rhoc; % reduced density of pure water

%------------------------------------------------------------------------
% calculation of ideal gas part of dimensionless Helmholtz energy 
% and its derivatives
%------------------------------------------------------------------------
[phi0, dphi0dtau, dphi0dtautau] = helmholtz_idealgas(delta_pw,tau);

%------------------------------------------------------------------------
% calculation of residual part of Helmholtz energy and its derivatives
%------------------------------------------------------------------------
[phir, dphirdtau, dphirdtautau, dphirddeltatau, dphirddelta, dphirddeltadelta] = helmholtz_residual(delta_pw,tau);

%------------------------------------------------------------------------
% calculation of thermodynamic properties of pure water
%------------------------------------------------------------------------

% Entropy [kJ/kgK]:
sw = R.*(tau.*(dphi0dtau + dphirdtau) - phi0 - phir); 
        
% Internal Energy [kJ/kg]:
% uw = R.*T.*tau.*(dphi0dtau + dphirdtau);

% Isochoric heat capacity [kJ/kgK]:
%cvw = -R.*tau.^2.*(dphi0dtautau + dphirdtautau);

% Enthalpy [kJ/kg]: 
hw = R.*T.*(1 + tau.*(dphi0dtau + dphirdtau) + delta_pw.*dphirddelta);

% Isobaric heat capacity [kJ/kgK]:
cpw = R.*(-tau.^2.*(dphi0dtautau + dphirdtautau) + ...
    (1 + delta_pw.*dphirddelta - delta_pw.*tau.*dphirddeltatau).^2./...
    (1 + 2.*delta_pw.*dphirddelta + delta_pw.^2.*dphirddeltadelta));

% Speed of sound [m/s]
ww = sqrt(R.*T.*(1 + 2 .*delta_pw.*dphirddelta + delta_pw.^2.*dphirddeltadelta...
   -  (1 + delta_pw.*dphirddelta - delta_pw.*tau.*dphirddeltatau).^2./...
   (tau.^2.*(dphi0dtautau + dphirdtautau))));
ww = ww.*sqrt(1000); % convert units to [m/s]

% Isentropic temperature-pressure coefficient [m^3*K/kJ]
betaw = 1./rho_pw./R.*(...
    (1 + delta_pw.*dphirddelta - delta_pw.*tau.*dphirddeltatau)./ ...
    ((1 + delta_pw.*dphirddelta - delta_pw.*tau.*dphirddeltatau).^2 - ...
    tau.^2.*(dphi0dtautau + dphirdtautau).* ...
    (1 + 2.*delta_pw.*dphirddelta + delta_pw.^2.*dphirddeltadelta)));

% Fugacity coefficient:
fw = exp(phir + delta_pw.*dphirddelta - log(1 + delta_pw .*dphirddelta));

% Joule-Thomson coefficient [K/kPa]
%JTw = 1./R./rho .*(-(delta.*dphirddelta + delta.^2.*dphirddeltadelta + ...
%    delta.*tau.*dphirddeltatau)./((1 + delta.*dphirddelta - ...
%    delta.*tau.*dphirddeltatau).^2 - tau.^2.*(dphi0dtautau + ...
%    dphirdtautau).*(1 + 2.*delta.*dphirddelta + delta.^2.*dphirddeltadelta)));

%JTw = JTw .* 100 ; % convert to [K/bar]
% pressure can be used as check value for density calculation
%PP = rho_pw.*(1000*R*T*(1+delta_pw*dphirddelta))

%------------------------------------------------------------------------
% Calculation of Gibbs energy (gw) and its derivatives with respect
% to p,T,S
%------------------------------------------------------------------------
[gw, dgwdT, dgwdp, dgwdTT, dgwdTp, dgwdpp] = Helmholtz_to_Gibbs (T,rho_pw,hw,sw,cpw,betaw,ww);


%------------------------------------------------------------------------
%------------------------------------------------------------------------
% CALCULATION OF SALINE CONTRIBUTION
%------------------------------------------------------------------------
%------------------------------------------------------------------------

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------
T0 = double(273.15); % Celsius zero point in Kelvin
p0 = double(101325); % Normal pressure [Pa]
Tstar = double(40) ;% reducing temperature [K]
pstar = double(1e8); % reducing pressure [Pa]
Sstar = double(0.03516504*40/35); % reducing salinity [g/kg]
tau_sw = (T - T0)./ Tstar; % reduced temperature for seawater
pi_sw = (P - p0)./pstar; % reduced pressure
xi_sw = sqrt(S./1000./Sstar); % square root of reduced salinity

%------------------------------------------------------------------------
% Calculation of Gibbs energy (gs) and its derivatives with respect
% to p,T,S
%------------------------------------------------------------------------
[gs, dgsdT, dgsdp, dgsdTT, dgsdTp, dgsdpp, dgsdS, dgsdSp] = gibbs_saline(pi_sw,tau_sw,xi_sw,Tstar,pstar,Sstar);

% for zero salinity: set saline contributions to zero 
%(some of them become NaN instead of 0 for S=0)
gs(S == 0) = 0;
dgsdT(S == 0) = 0;
dgsdTT(S == 0) = 0;
dgsdTp(S == 0) = 0;
dgsdp(S == 0) = 0;
dgsdpp(S == 0) = 0;
dgsdS(S == 0) = 0;
dgsdSp(S == 0) = 0;

%------------------------------------------------------------------------
%------------------------------------------------------------------------
% CALCULATION OF SEAWATER THERMODYNAMIC PROPERTIES
%   (COMBINED PURE WATER AND SALINE CONTRIBUTIONS)
%------------------------------------------------------------------------
%------------------------------------------------------------------------

%------------------------------------------------------------------------
% Calculation of total Gibbs energy (gt) and its derivatives with respect
% to p,T,S
%------------------------------------------------------------------------
gt = gw + gs; % total Gibbs energy [J/kg]

% derivative of Gibbs energy with respect to temperature [J/(kg K)]
dgtdT = dgwdT + dgsdT; 

% second derivative with respect to temperature [J/(kg K^2)]
dgtdTT = dgwdTT + dgsdTT; 

% derivative with respect to pressure [m^3/kg]
dgtdp = dgwdp + dgsdp; 

 % second derivative with respect to pressure [m^3/(kg Pa)]
dgtdpp = dgwdpp + dgsdpp;

% derivative with respect to pressure and temperature [m^3/(kg K)]
dgtdTp = dgwdTp + dgsdTp; 

% derivative with respect to salinity [J/kg]
dgtdS = dgsdS; 

% derivative with respect to pressure and salinity [m^3/kg]
dgtdSp = dgsdSp; 

%------------------------------------------------------------------------
% Calculation of thermodynamic properties of seawater
%------------------------------------------------------------------------

% Density [kg(m^3]
rho_sw = 1./dgtdp;

% Specific entropy [J/(kg K)]
s_sw = -dgtdT;

% Specific isobaric heat capacity [J/(kg K)]
cp_sw = - T.*dgtdTT;

% Specific enthalpy [J/kg]
h_sw = gt - T.*dgtdT;

% Specific internal energy [J/kg]
u_sw = gt - T.*dgtdT - P.* dgtdp;

% Specific helmholtz energy [J/kg]
phi_sw = gt - P.*dgtdp;

% Thermal expansion coefficient [1/K]
alpha_sw = dgtdTp./dgtdp;

% Isentropic temperature-pressure coefficient [K/Pa]
beta_sw = -dgtdTp./dgtdTT;

% Isothermal compressibility [1/Pa]
kappaT_sw = -dgtdpp./dgtdp;

% Isentropic compressibility [1/Pa]
kappas_sw = (dgtdTp.^2 - dgtdTT.*dgtdpp)./(dgtdp.*dgtdTT);

% Chemical Potential of H2O [J/kg]
mu_sw = gt - S./1000.*dgtdS;

% Chemical potential of sea salt [J/kg]
mu_ss = gt + (1-S./1000).*dgtdS;

% Osmotic coefficient [1] can be calculated for known molality of seawater
% Rm = 8.314472 % [J/(mol K)]
% phi_os = - (gs - S./1000.*dgtdS)./(m_sw .*Rm.*T);

% Haline contraction coefficient [kg/kg]
beta_hal = -dgtdSp ./dgtdp;

end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [phi0, dphi0dtau, dphi0dtautau] = helmholtz_idealgas(delta,tau)

%  calculation of the ideal gas part (phi0) of the Helmholtz energy and its
%  first and second derivatives with respect to the reduced temperature
%  (dphi0dtau, dphi0dtautau)

% a, theta: coefficients of the correlation equation
a1 =   [-8.32044648201
        6.6832105268
        3.00632];
    
a2 =   [0.012436
        0.97315
        1.2795
        0.96956
        0.24873];
    
theta =[1.28728967
        3.53734222
        7.74073708
        9.24437796
        27.5075105];

% calculate sum terms
sum1 = zeros(size(tau,1),size(tau,2),size(tau,3));
sum2 = zeros(size(tau,1),size(tau,2),size(tau,3));
sum3 = zeros(size(tau,1),size(tau,2),size(tau,3));
for i=1:5
    sum1 = sum1 + a2(i).*log(1 -exp(-tau.*theta(i)));
    sum2 = sum2 + a2(i).*theta(i).*((1 -exp(-tau.*theta(i))).^(-1)-1);
    sum3 = sum3 + a2(i).*theta(i).^2.*exp(-tau.*theta(i))...
        .*(1 -exp(-tau.*theta(i))).^(-2);
end

% calculate phi and derivatives
phi0 = log(delta) + a1(1) + a1(2).*tau + a1(3).* log(tau) + sum1;  
dphi0dtau = a1(2) + a1(3)./tau + sum2;
dphi0dtautau = - a1(3)./tau.^2 - sum3;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [phir, dphirdtau, dphirdtautau, dphirddeltatau, dphirddelta, dphirddeltadelta] = helmholtz_residual(delta,tau)

%  calculation of the residual part (phir) of the Helmholtz energy and its
%  first and second derivatives with respect to the inverse reduced temperature
%  (dphirdtau and dphirdtautau) and first and second derivatives with respect to
%  the reduced density (dphirddelta, dphirddeltadelta) as well as 
%  the mixed derivative (dphirddeltadtau)

%----------------------------------------------------------------------
% calculation of sum terms with parameters ni,di,ti
%----------------------------------------------------------------------

ni = [  0.12533547935523e-1
         0.78957634722828e1
        -0.87803203303561e1
         0.31802509345418
        -0.26145533859358
        -0.78199751687981e-2
         0.88089493102134e-2];
  
di = [1 1 1 2 2 3 4];
ti = [-0.5 0.875 1 0.5 0.75 0.375 1];

sumr1   =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrd1  =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrdd1 =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrt1  =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrtt1 =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrdt1 =  zeros(size(tau,1),size(tau,2),size(tau,3));

for i=1:7
    sumr1   = sumr1 + ni(i).*delta.^di(i).*tau.^ti(i);
    sumrd1  = sumrd1 + ni(i).*di(i).*delta.^(di(i)-1).*tau.^ti(i);
    sumrdd1 = sumrdd1 + ni(i).*di(i).*(di(i)-1).*delta.^(di(i)-2).*tau.^ti(i);
    sumrt1  = sumrt1 + ni(i).*ti(i).*delta.^di(i).*tau.^(ti(i)-1);
    sumrtt1 = sumrtt1 + ni(i).*ti(i).*(ti(i)-1).*delta.^di(i).*tau.^(ti(i)-2);
    sumrdt1 = sumrdt1 + ni(i).*ti(i).*di(i).*delta.^(di(i)-1).*tau.^(ti(i)-1);
end


%----------------------------------------------------------------------
% calculation of sum terms with parameters nj,dj,tj,cj
%----------------------------------------------------------------------

nj = [  -0.66856572307965
         0.20433810950965
        -0.66212605039687e-4
        -0.19232721156002
        -0.25709043003438
         0.16074868486251
        -0.40092828925807e-1
         0.39343422603254e-6
        -0.75941377088144e-5
         0.56250979351888e-3
        -0.15608652257135e-4
         0.11537996422951e-8
         0.36582165144204e-6
        -0.13251180074668e-11
        -0.62639586912454e-9
        -0.10793600908932
         0.17611491008752e-1
         0.22132295167546
        -0.40247669763528
         0.58083399985759
         0.49969146990806e-2
        -0.31358700712549e-1
        -0.74315929710341
         0.47807329915480
         0.20527940895948e-1
        -0.13636435110343
         0.14180634400617e-1
         0.83326504880713e-2
        -0.29052336009585e-1
         0.38615085574206e-1
        -0.20393486513704e-1
        -0.16554050063734e-2
         0.19955571979541e-2
         0.15870308324157e-3
        -0.16388568342530e-4
         0.43613615723811e-1
         0.34994005463765e-1
        -0.76788197844621e-1
         0.22446277332006e-1
        -0.62689710414685e-4
        -0.55711118565645e-9
        -0.19905718354408
         0.31777497330738
        -0.11841182425981];
 
dj = [1 1 1 2 2 3 4 4 5 7 9 10 11 13 15 1 2 2 2 3 4 4 4 5 6 6 7 ...
    9 9 9 9 9 10 10 12 3 4 4 5 14 3 6 6 6];
tj = [4 6 12 1 5 4 2 13 9 3 4 11 4 13 1 7 1 9 10 10 3 7 10 10 6 10 10 ...
    1 2 3 4 8 6 9 8 16 22 23 23 10 50 44 46 50];
cj = [1 1 1 1 1 1 1 1 1 1 1 1 1 1 1 ...
    2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 2 3 3 3 3 4 6 6 6 6];

sumr2   =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrd2  =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrdd2 =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrt2  =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrtt2 =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrdt2 =  zeros(size(tau,1),size(tau,2),size(tau,3));

for i=1:44
    sumr2 = sumr2 + nj(i).*delta.^dj(i).*tau.^tj(i).*exp(-delta.^cj(i));
    sumrd2 = sumrd2 + nj(i).*delta.^(dj(i)-1).*tau.^tj(i)....
        .*exp(-delta.^cj(i)).*(dj(i)-cj(i).*delta.^cj(i));
    sumrdd2 = sumrdd2 + nj(i).*delta.^(dj(i)-2).*tau.^tj(i)...
        .*exp(-delta.^cj(i)).*((dj(i)-cj(i).*delta.^cj(i))...
        .*(dj(i)-1-cj(i).*delta.^cj(i))-cj(i)^2.*delta.^cj(i));
    sumrt2 = sumrt2 + nj(i).*delta.^dj(i).*tau.^(tj(i)-1).*tj(i)...
        .*exp(-delta.^cj(i));
    sumrtt2 = sumrtt2 + nj(i).*delta.^dj(i).*tau.^(tj(i)-2).*tj(i).*(tj(i)-1)...
        .*exp(-delta.^cj(i));
    sumrdt2 = sumrdt2 + nj(i).*delta.^(dj(i)-1).*tau.^(tj(i)-1).*tj(i)...
        .*exp(-delta.^cj(i)).*(dj(i)-cj(i).*delta.^cj(i));
end

%----------------------------------------------------------------------
% calculation of sum terms with parameters nk,dk,tk,ak,bk,ck,ek
%----------------------------------------------------------------------

nk = [-0.31306260323435e2
       0.31546140237781e2
      -0.25213154341695e4];

dk = [3 3 3];
tk = [0 1 4];
ak = [20 20 20];
bk = [150 150 250];
ck = [1.21 1.21 1.25];
ek = [1. 1. 1.];

sumr3   =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrd3  =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrdd3 =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrt3  =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrtt3 =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrdt3 =  zeros(size(tau,1),size(tau,2),size(tau,3));

for i=1:3
    sumr3 = sumr3 + nk(i).*delta.^dk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2);
    sumrd3 = sumrd3 + nk(i).*delta.^dk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2)...
        .*(dk(i)./delta - 2.*ak(i).*(delta -ek(i)));
     sumrdd3 = sumrdd3 + nk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2)...
        .*(- 2.*ak(i).*delta.^dk(i) + 4.*ak(i)^2.*delta.^dk(i)...
        .*(delta - ek(i)).^2 - 4.*dk(i).*ak(i).*delta.^(dk(i)-1)...
        .*(delta - ek(i)) + dk(i).*(dk(i)-1).*delta.^(dk(i)-2));
     sumrt3 = sumrt3 + nk(i).*delta.^dk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2)...
        .*(tk(i)./tau - 2.*bk(i).*(tau -ck(i)));
     sumrtt3 = sumrtt3 + nk(i).*delta.^dk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2)...
        .*((tk(i)./tau - 2.*bk(i).*(tau -ck(i))).^2 - tk(i)./tau.^2 - 2.*bk(i));
     sumrdt3 = sumrdt3 + nk(i).*delta.^dk(i).*tau.^tk(i)...
        .*exp(-ak(i).*(delta-ek(i)).^2 - bk(i).*(tau - ck(i)).^2)...
        .*(tk(i)./tau - 2.*bk(i).*(tau -ck(i)))...
        .*(dk(i)./delta - 2.*ak(i).*(delta -ek(i)));
end

%----------------------------------------------------------------------
% calculation of sum terms with parameters nl,al,bl,betl,Al,Bl,Cl,Dl
%----------------------------------------------------------------------

nl = [-0.14874640856724
       0.31806110878444];
al = [3.5 3.5];
bl = [0.85 0.95];
betl = [0.3 0.3];
Al = [0.32 0.32];
Bl = [0.2 0.2];
Cl = [28 32];
Dl = [700 800];

sumr4 =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrd4 =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrdd4 =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrt4 =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrtt4 =  zeros(size(tau,1),size(tau,2),size(tau,3));
sumrdt4 =  zeros(size(tau,1),size(tau,2),size(tau,3));

for i=1:2
    % calculation of terms and their derivatives which are frequently used in the sums 
    psi = exp(-Cl(i).*(delta-1).^2 - Dl(i).*(tau -1).^2);
    dpsidd = -2.*Cl(i).*(delta -1).*psi;                 % d(psi)/d(delta)
    dpsiddd = (2.*Cl(i).*(delta -1).^2-1).*2.*Cl(i).*psi;% d^2(psi)/d(delta^2)
    dpsidt = -2.*Dl(i).*(tau -1).*psi;                   % d(psi)/d(tau)
    dpsidtt = (2.*Dl(i).*(tau -1).^2-1).*2.*Dl(i).*psi;  % d^2(psi)/d(tau^2)
    dpsiddt = 4.*Cl(i).*Dl(i).*(delta -1).*(tau - 1).*psi;% d^2(psi)/d(delta)d(tau)
    
    theta = (1 - tau) + Al(i).*((delta -1).^2).^(1./(2.*betl(i)));
    
    Delta = theta.^2 + Bl(i).*((delta - 1).^2).^al(i);
    dDeltadd = (delta -1)...
        .*(Al(i).*theta.*2./betl(i).*((delta-1).^2).^(1./(2.*betl(i))-1)...
        + 2.*Bl(i).*al(i).*((delta -1).^2).^(al(i)-1)); % d(Delta)/d(delta)
    dDeltaddd = 1./(delta -1).*dDeltadd + (delta - 1).^2.*...
        (4.*Bl(i).*al(i).*(al(i)-1).*((delta - 1).^2).^(al(i)-2)...
        + 2.*Al(i).^2 .* (1./betl(i).^2) .* ...
        (((delta - 1).^2).^(1./2./betl(i)-1)).^2 + ...
        Al(i).*theta.*4./betl(i).*(1./2./betl(i) -1) .*...
        ((delta - 1).^2).^(1./2./betl(i) - 2)); % d^2(Delta)/d(delta^2)
    
    Deltab = Delta.^bl(i);
    dDeltabdd = bl(i).*Delta.^(bl(i)-1).*dDeltadd; % d(Deltab)/d(delta)
    dDeltabddd = bl(i).*(Delta.^(bl(i)-1).*dDeltaddd + ...
        (bl(i)-1).*Delta.^(bl(i)-2).*dDeltadd.^2); % d^2(Delatb)/d(delta^2)
    dDeltabdt = -2.*theta.*bl(i).*Delta.^(bl(i)-1); % d(Deltab)/d(tau)
    dDeltabdtt = 2.*bl(i).*Delta.^(bl(i)-1) + ...
        4 .* theta.^2 .* bl(i).*(bl(i)-1).*Delta.^(bl(i)-2); % d^2(Deltab)/d(tau^2)
    dDeltabddt = - Al(i).*bl(i).*2./betl(i).*Delta.^(bl(i)-1).*(delta-1)...
        .*((delta-1).^2).^(1./2./betl(i)-1) - ...
        2.*theta.*bl(i).*(bl(i)-1).*Delta.^(bl(i)-2).*dDeltadd;% d^2(Deltab)/d(delta)d(tau)
    
    sumr4 = sumr4 + nl(i).*Deltab.*delta.*psi;
    sumrd4 = sumrd4 + nl(i).*(Deltab .*(psi + delta .* dpsidd) ...
        + dDeltabdd.*delta.*psi);
    sumrdd4 = sumrdd4 + nl(i).*(Deltab.*(2.*dpsidd + delta.*dpsiddd)...
        + 2.*dDeltabdd.*(psi + delta.*dpsidd) + dDeltabddd.*delta.*psi);
    sumrt4 = sumrt4 + nl(i).*delta.*(dDeltabdt.*psi + Deltab.*dpsidt);
    sumrtt4 = sumrtt4 + nl(i).* delta .*(dDeltabdtt .* psi ...
        + 2.*dDeltabdt .* dpsidt + Deltab .* dpsidtt);
    sumrdt4 = sumrdt4 + nl(i).*(Deltab .*(dpsidt + delta.*dpsiddt) + ...
        delta.*dDeltabdd .* dpsidt + dDeltabdt.*(psi + delta .* dpsidd) ...
        + dDeltabddt .* delta .* psi);
    
end

%----------------------------------------------------------------------
% calculation of function output values
%----------------------------------------------------------------------
phir = sumr1 + sumr2 + sumr3 + sumr4; % phi_r: residual of Helmholtz energy
dphirddelta = sumrd1 + sumrd2 + sumrd3 + sumrd4;        % d(phi_r)/d(delta)
dphirddeltadelta = sumrdd1 + sumrdd2 + sumrdd3 + sumrdd4; % d^2(phi_r)/d(delta^2)
dphirdtau = sumrt1 + sumrt2 + sumrt3 + sumrt4;          % d(phi_r)/d(tau)
dphirdtautau = sumrtt1 + sumrtt2 + sumrtt3 + sumrtt4;   % d^2(phi_r)/d(tau^2)
dphirddeltatau = sumrdt1 + sumrdt2 + sumrdt3 + sumrdt4; % d^2(phi_r)/d(delta)d(tau)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [gw, gwt, gwp, gwtt, gwtp, gwpp] = Helmholtz_to_Gibbs(T,rho_pw,hw,sw,cpw,betaw,ww)

%------------------------------------------------------------------------
% Transformation from Helmholtz energy based formalism to Gibbs energy 
% based formalism
%------------------------------------------------------------------------

gw = (hw -T.*sw).* 1000; % Gibbs Energy [J/kg]
% derivatives of Gibbs energy
gwt = -sw.*1000; % d(gw)/d(T) [J/(kg*K)]
gwp = 1./rho_pw; % d(gw)/d(p) [m^3/kg]
gwtt = - cpw./T.*1000; % d^2(gw)/d(t^2) [J/(kg*K^2)]
gwtp = betaw.*cpw./T; % d^2(gw)/d(p)d(t) [m^3/(kg*K)]
gwpp = -1./(rho_pw.*ww).^2 - betaw.^2.*cpw./T./1000; % d^2(gw)/d(p^2) [m^3/(kg*Pa)]

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [gs, dgsdT, dgsdp, dgsdTT, dgsdTp, dgsdpp, dgsdS, dgsdSp] = gibbs_saline(pi_sw,tau,xi,Tstar,pstar,Sstar)

% gibbs_saline calculates the saline part of the Gibbs energy (gs) and its 
% derivatives with respect to p,T,S

%----------------------------------------------------------------------
% definition of parameters gijk
%----------------------------------------------------------------------
g = zeros(7,7,6);

g(1,1,1) =  0.581281456626732e4;
g(2,1,1) =  0.141627648484197e4;
g(3,1,1) = -0.243214662381794e4;
g(4,1,1) =  0.202580115603697e4;
g(5,1,1) = -0.109166841042967e4;
g(6,1,1) =  0.374601237877840e3;
g(7,1,1) = -0.485891069025409e2;

g(1,2,1) =  0.851226734946706e3;
g(2,2,1) =  0.168072408311545e3;
g(3,2,1) = -0.493407510141682e3;
g(4,2,1) =  0.543835333000098e3;
g(5,2,1) = -0.196028306689776e3;
g(6,2,1) =  0.367571622995805e2;

g(2,3,1) =  0.880031352997204e3;
g(3,3,1) = -0.430664675978042e2;
g(4,3,1) = -0.685572509204491e2;

g(2,4,1) = -0.225267649263401e3;
g(3,4,1) = -0.100227370861875e2;
g(4,4,1) =  0.493667694856254e2;

g(2,5,1) =  0.914260447751259e2;
g(3,5,1) =  0.875600661808945;
g(4,5,1) = -0.171397577419788e2;

g(2,6,1) = -0.216603240875311e2;
g(4,6,1) =  0.249697009569508e1;

g(2,7,1) =  0.213016970847183e1;

g(2,1,2) = -0.331049154044839e4;
g(3,1,2) =  0.199459603073901e3;
g(4,1,2) = -0.547919133532887e2;
g(5,1,2) =  0.360284195611086e2;

g(2,2,2) =  0.729116529735046e3;
g(3,2,2) = -0.175292041186547e3;
g(4,2,2) = -0.226683558512829e2;

g(2,3,2) = -0.860764303783977e3;
g(3,3,2) =  0.383058066002476e3;

g(2,4,2) =  0.694244814133268e3;
g(3,4,2) = -0.460319931801257e3;

g(2,5,2) = -0.297728741987187e3;
g(3,5,2) =  0.234565187611355e3;

g(2,1,3) =  0.384794152978599e3;
g(3,1,3) = -0.522940909281335e2;
g(4,1,3) = -0.408193978912261e1;

g(2,2,3) = -0.343956902961561e3;
g(3,2,3) =  0.831923927801819e2;

g(2,3,3) =  0.337409530269367e3;
g(3,3,3) = -0.541917262517112e2;

g(2,4,3) = -0.204889641964903e3;

g(2,5,3) =  0.747261411387560e2;

g(2,1,4) = -0.965324320107458e2;
g(3,1,4) =  0.680444942726459e2;
g(4,1,4) = -0.301755111971161e2;

g(2,2,4) =  0.124687671116248e3;
g(3,2,4) = -0.294830643494290e2;

g(2,3,4) = -0.178314556207638e3;
g(3,3,4) =  0.256398487389914e2;

g(2,4,4) =  0.113561697840594e3;

g(2,5,4) = -0.364872919001588e2;

g(2,1,5) =  0.158408172766824e2;
g(3,1,5) = -0.341251932441282e1;

g(2,2,5) = -0.316569643860730e2;

g(2,3,5) =  0.442040358308000e2;

g(2,4,5) = -0.111282734326413e2;

g(2,1,6) = -0.262480156590992e1;

g(2,2,6) =  0.704658803315449e1;

g(2,3,6) = -0.792001547211682e1;

%----------------------------------------------------------------------
% calculation of Gibbs energy gs [J/kg]
%----------------------------------------------------------------------
gs = zeros(size(tau,1),size(tau,2),size(tau,3)); %[J/kg]
for k=1:6 
    for j=1:7
        sum1 = zeros(size(tau,1),size(tau,2),size(tau,3));
        for i=2:7
            sum1 = sum1 + g(i,j,k).*xi.^i;
        end
        gs =gs + (g(1,j,k).*xi.^2.*log(xi) + sum1).*tau.^(j-1).*pi_sw.^(k-1);
    end
end

%----------------------------------------------------------------------
% calculation of derivatives of Gibbs energy
%----------------------------------------------------------------------    
dgsdT = zeros(size(tau,1),size(tau,2),size(tau,3)); % d(gs)/d(T) [J/(kgK)]
for k=1:6 
    for j=2:7
        sum1 = zeros(size(tau,1),size(tau,2),size(tau,3));
        for i=2:7
            sum1 = sum1 + g(i,j,k).*xi.^i;
        end
        dgsdT =dgsdT + ...
            1./Tstar.*(j-1).*(g(1,j,k).*xi.^2.*log(xi) + sum1)...
            .*tau.^(j-2).*pi_sw.^(k-1);
    end
end  
   
dgsdp = zeros(size(tau,1),size(tau,2),size(tau,3)); % d(gs)/d(p) [m^3/kg]
for k=2:6 
    for j=1:7
        for i=2:7
        dgsdp =dgsdp + ...
            1./pstar.*(k-1).*g(i,j,k).*xi.^i.*tau.^(j-1).*pi_sw.^(k-2);
        end
    end
end    
   
% d^2(gs)/d(T^2) [J/(kg K^2)]
dgsdTT = zeros(size(tau,1),size(tau,2),size(tau,3)); 
for k=1:6 
    for j=3:7
        for i=2:7
        dgsdTT =dgsdTT + 1./Tstar.^2 .*(j-1).*(j-2).*g(i,j,k) ...
            .*xi.^i.*tau.^(j-3).*pi_sw.^(k-1);
        end
    end
end    

% d^2(gs)/d(T)d(p) [m^3/(kg K)]
dgsdTp = zeros(size(tau,1),size(tau,2),size(tau,3));
for k=2:6 
    for j=2:7
        for i=2:7
        dgsdTp =dgsdTp + ...
            1./pstar./Tstar.*(k-1).*(j-1).*g(i,j,k)...
            .*xi.^i.*tau.^(j-2).*pi_sw.^(k-2);
        end
    end
end   

% d^2(gs)/d(p^2) [m^3/(kg Pa)]
dgsdpp = zeros(size(tau,1),size(tau,2),size(tau,3));
for k=3:6 
    for j=1:7
        for i=2:7
        dgsdpp =dgsdpp + ...
            1./pstar.^2.*(k-1).*(k-2)...
            .*g(i,j,k).*xi.^i.*tau.^(j-1).*pi_sw.^(k-3);
        end
    end
end   

dgsdS = zeros(size(tau,1),size(tau,2),size(tau,3)); % d(gs)/d(S) [J/kg]
for k=1:6 
    for j=1:7
        sum1 = zeros(size(tau,1),size(tau,2),size(tau,3));
        for i=2:7
            sum1 = sum1 + i.*g(i,j,k).*xi.^(i-2);
        end
        dgsdS = dgsdS + 1./2./Sstar.*...
            (g(1,j,k).*(2.*log(xi)+1) + sum1).*tau.^(j-1).*pi_sw.^(k-1);
    end
end

% d^2(gs)/d(S)d(p) [m^3/kg]
dgsdSp = zeros(size(tau,1),size(tau,2),size(tau,3)); 
for k=2:6 
    for j=1:7
        for i=2:7
        dgsdSp =dgsdSp + ...
            1./(2.*Sstar.*pstar)...
            .*(k-1).*i.*g(i,j,k).*xi.^(i-2).*tau.^(j-1).*pi_sw.^(k-2);
        end
    end
end     
    
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function f = minfunc(rho_pw,P,T,rhoc,R,tau)
% minimization function to calculate density of pure water
% based on the equation p = rho*R*T*(1 + rho/rohc * d(phir)/d(rho/rhoc)) 

delta = rho_pw/rhoc;

% calculate d(phir)/d(rho/rhoc)
[~, ~, ~, ~, dphirddelta] = helmholtz_residual(delta,tau);

% calculate density
RHOeos = P / (1000*R*T*(1+delta*dphirddelta));

% calculate difference between initial rho and "new" calculated rho
f = rho_pw - RHOeos;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function guess = guessfunc(T)
% this function defines the starting value for the numerical iteration of
% the density for pure water. The listed values are the density values at
% the phase boundary as a function of temperature, as listed in the Wagner
% paper. The output of this routine is the density that belongs to the
% temperature closest to the input temperature.

g = zeros(2,39);
for i = 1:39
    g(1,i)= 260 + i * 10;
end

g(2,1) = 999.79;
g(2,2) = 999.86;
g(2,3) = 998.758;
g(2,4) = 996.51;
g(2,5) = 993.34;
g(2,6) = 989.387;
g(2,7) = 984.75;
g(2,8) = 979.5;
g(2,9) = 973.7;
g(2,10) = 967.386;
g(2,11) = 960.587;
g(2,12) = 953.327;
g(2,13) = 945.624;
g(2,14) = 937.486;
g(2,15) = 928.921;
g(2,16) = 919.929;
g(2,17) = 910.507;
g(2,18) = 900.649;
g(2,19) = 890.341;
g(2,20) = 879.569;
g(2,21) = 868.310;
g(2,22) = 856.537;
g(2,23) = 844.219;
g(2,24) = 831.313;
g(2,25) = 817.772;
g(2,26) = 803.535;
g(2,27) = 788.527;
g(2,28) = 772.657;
g(2,29) = 755.808;
g(2,30) = 737.831;
g(2,31) = 718.530;
g(2,32) = 697.638;
g(2,33) = 674.781;
g(2,34) = 649.411;
g(2,35) = 620.65;
g(2,36) = 586.88;
g(2,37) = 544.25;
g(2,38) = 481.53;
g(2,39) = 322;

t_i = floor(T/10) - 26;
t_i(t_i < 1) = 1;
t_i(t_i > 39) = 39;

guess = g(2,t_i);
