function [rho, h, cp] = eos_swDriesner(press,temp,sal)

% EOS_SWDRIESNER -  calculates the density, enthalpy and isobaric heat
%      capacity of the system H2O-NaCl based on multiparametric equations
%
%      Usage: [rho h cp] = eos_swDriesner(p,T,S)
%
%      Input parameters (vectors or scalars, S: scalar): 
%           p       - pressure [MPa]        (0.1 < P < 167.5)
%           T       - temperature [degC]    (0 < T < 600)
%           S       - salinity (see comments)
%
%      Output parameters:
%          rho      - density [kg/m^3]
%          h        - specific enthalpy [J/kg]
%          cp       - specific isobaric heat capacity [J/kg/K]
%
%      The densities, enthalpies and heat capacities are calculated from 
%      equations for pure water at a modified temperature that balances 
%      out the deviations caused by salinities.
%
%      References:
%      Driesner & Heinrich (2007) GCA 71, 4902-4919.
%
%      Comments: 
%      Currently, the EOS for pure water based on the IAPWS formulation
%      (eos_h2o), which is called by this routine, is not implemented for
%      its full pT range. It holds for T < 1000 degC and p < 167.5 MPa and
%      some more extended ranges. For details, see comments in eos_h2o.
%
%      The second extrapolation function of Driesner's formalism for
%      T >= 600 degC, p <= 350 bar and high salinities (close to saturation)
%      is not implemented. As a comsequence, the temperature range of
%      this script is reduced to 0 < T < 600 degC.
%
%      This implemetation of Driesner's formalism calls the IAPS95 
%      formulation for the calculation of pure water densities. 
%      Driesner developed his formalism based on the IAPS84 formulation.
%      The two formulations differ mainly in the vicinity of the critical
%      point of water. For calculations close to this point, it is 
%      recommended to edit this script to call the IAPS84 formulation.
%
%      Although Driesner's paper states that the formulation is valid for
%      mole fractions of up to 100% NaCl, the calculated densities for 
%      100% NaCl do not approach the pure NaCl densities properly, except
%      for conditions with high saturation concentrations. It seems that
%      the formulation is only meant for NaCl concentrations up to 
%      saturation. Since the curve for the density as a function of NaCl
%      concentration progresses smoothly after exceeding the saturation
%      concentration by some 10%, this script will allow for the 
%      calculation of oversaturated solutions up to mole fraction values 
%      of 0.2 above the saturation mole fraction. If this value is exceeded,
%      a warning will be issued.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (March 2011)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       November 2012: syntax changed to Matlab2012 style
%       January  2013: check for NaN before calling fsolve
%                      names in internal function calls changed: 
%                      eos_H2O -> eos_h2o, solu_NaCl -> solu_nacl 
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('EOS_SWDRIESNER:Input','EOS_SWDRIESNER: Missing input parameter! Please provide p, T and S!')
    rho = NaN;
    h = NaN;
    cp = NaN;
elseif length(sal) > 1
     warning('EOS_SWDRIESNER:Input','EOS_SWDRIESNER: Only scalar input for Salinity!')
    rho = NaN;
    h = NaN;
    cp = NaN;
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if  ~any(press<(0.1) | press>167.5) && ~any(temp<(0) | temp>600)

else
    bad = find(press<(0.1) | press>167.5);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<(0) | temp>600);
    temp(bad)=NaN*ones(size(bad));

 warning('EOS_SWDRIESNER:OutOfParameterRange','EOS_SWDRIESNER: Input values out of range!')
 warning('EOS_SWDRIESNER:OutOfParameterRange','Equations valid for p = 0.1-167.5 MPa, T=0-600 degC.')
end

%------------------------------------------------------------------------
% Transforming salinity to mole fraction of NaCl
%------------------------------------------------------------------------


mmNaCl = 58.443; % molar mass of NaCl
mmH2O = 18.015;  % molar mass of H2O

% calculate weight equivalent of NaCl
sal2 = sal./35.*35.16504; % practical salinity to absolute salinity (g/kg solution)
sal2 = sal2.*(1+ 0.001005.*sal);% g/kg(solution) to g/kg(H2O)
nNaCl = sal2./mmNaCl; % mol/kg(H2O)

nH2O = (1000 - sal)./mmH2O; % moles of H2O in 1 kg solution
xNaCl = nNaCl./(nNaCl + nH2O); % mole fraction of NaCl

mxsat = solu_nacl(press,temp); % saturation concentration at given pressure and temperature
xsat = m2x(mxsat);

for i = 1:length(xNaCl)
    findsat = xNaCl(i) > (xsat + 0.2) | (xsat > 1); % find NaCl mole fractions that are 0.2 above saturation
    xNaCl(findsat) = NaN; % set NaCl mole fractions that are 0.2 above saturation to NaN
end

if any(isnan(xNaCl))
    warning('EOS_SWDRIESNER:OutOfParameterRange','EOS_SWDRIESNER: Salinity too high!')
end
%------------------------------------------------------------------------
% Definition of constants, thermodynamic variables and parameters
%------------------------------------------------------------------------
[P,T,X] = ndgrid(press.*10,temp,xNaCl); % pressure in bar

rho = zeros(size(P)); % empty matrix for density
h   = zeros(size(P)); % empty matrix for specific enthalpy
cp  = zeros(size(P)); % empty matrix for heat capacity

PcH2O  = 2.2054915e2; % critical pressure of pure H2O [bar]


s1 = size(P,1);
s2 = size(P,2);
s3 = size(P,3);

%------------------------------------------------------------------------
% calculation of density
%------------------------------------------------------------------------

% parameters that are needed for the calculation of the density

n11 = -54.2958 - 45.7623.*exp(-9.44785e-4.*P);
n21 = -2.6142 - 0.000239092.*P;
n22 = 0.0356828 + 4.37235e-6.*P + 2.0566e-9.*P.^2;

% parameters n1 and n2 for xNaCl = 0:

n1_XNaCl0 = 0;
n2_XNaCl0 = 1;

% parameters n1 and n2 for xNaCl = 1:
n1_XNaCl1 =  330.47 + 0.942876.*sqrt(P) + 0.0817193.*P ...
            - 2.47556e-8.*P.^2 + 3.45052e-10.*P.^3;
n2_XNaCl1 = -0.0370751 + 0.00237723.*sqrt(P) + 5.42049e-5.*P ...
            + 5.84709e-9.*P.^2 - 5.99373e-13.*P.^3;
        
% use solutions for XNaCl = 0 and XNaCl = 1 to calculate missing parameters       

n10 = n1_XNaCl1;
n12 = n1_XNaCl0 - n10 - n11;

n20 = n2_XNaCl0 - n21.*sqrt(n22);
n23 = n2_XNaCl1 - n20 -n21.*sqrt(1+n22);

% calculat parameters n1 and n2

n1 = n10 + n11.*(1 - X) + n12.*(1 - X).^2;
n2 = n20 + n21.*sqrt(X + n22) + n23.*X;

% calculation of the deviation function Dv

n300  = 7.60664e6./(P + 472.051).^2;
n301  =-50 - 86.1446.*exp(-6.21128e-4.*P);
n302  = 294.318.*exp(-5.66735e-3.*P);
n310  =-0.0732761.*exp(-2.3772e-3.*P) - 5.2948e-5.*P;
n311  =-47.2747 + 24.3653.*exp(-1.25533e-3.*P);
n312  =-0.278529 - 0.00081381.*P;

n30 = n300.*(exp(n301.*X) -1) + n302.*X;
n31 = n310.*exp(n311.*X) + n312.*X;

o2  = 2.0125e-7 + 3.29977e-9 .*exp(-4.31279.*log10(P))...
     -1.17748e-7.*log10(P) + 7.58009e-8.*(log10(P)).^2;

Dv = n30 .*exp(n31.*T);

% calculation of modified temperature Tv

Tv = n1 + n2.*T + Dv;

% check, if the modified temperature would lead to the calculation of a
% vapor phase density instead of a liquid phase density

pb_pure = phaseboundpw(Tv); % phase boundary pressur for pure water in bar
pb_brine = zeros(size(P));  % empty matrix for phase boundary pressure

warning off PHASE_SW:OutOfParameterRange
[~, pb_brine1] = phase_sw(temp,xNaCl); % phase boundary for NaCl solution

for i = 1:s1
    pb_brine(i,:) = pb_brine1;
end

pb_brine = pb_brine.*10; % [bar]
options=optimset('Display','off','TolFun',1e-16); % set options for fitfunction


pb_brinesat = zeros(size(P)); % empty vector for phase boundary pressure at saturation

for i = 1:s1
    for j=1:s2
        [~, pb_brinesattemp] = phase_sw(temp(j),xsat(i,j)-0.001); % phase boundary for NaCl saturation
        pb_brinesat(i,j) = pb_brinesattemp;
    end
end
warning on PHASE_SW:OutOfParameterRange

pb_brinesat=pb_brinesat*10; % pressure in bar

% replace NaN values in phase boundary wih phase boundary at saturation
ina = isnan(pb_brine); % find NaN values in matrix

pb_brine = pb_brinesat.*ina - pb_brine.*(ina-1);

% calculation of molar volume by calling external function for density of
% pure water with modified temperatures

for i = 1:s1
    for j = 1:s2
        for k = 1:s3
             % if NaCl solution is liquid and the modified temperature would
             % lead to a pure water solution in the vapor phase
            if P(i,j,k) <= PcH2O && P(i,j,k) >= pb_brine(i,j) && P(i,j,k) < pb_pure(j) 
                
                % check for NaN before calling fsolve: if any of the input
                % parameters is NaN, set corresponding density matrix
                % element to NaN
                if isnan(P(i,j,k)) || isnan(T(i,j,k)) || isnan(X(i,j,k))
                    rho(i,j,k) = NaN;
                else
                    % call solver to calculate temperature at phase boundary
                    % setting starting value for solver:
                    T_init = T(i,j,k); % should be close enough, else this loop would not be invoked
                    T_bound = fsolve(@(Tb)minfunc_Tbound(Tb,P(i,j,k)),T_init,options); % call solver
                    % calculate density at phase boundary and close to phase
                    % boundary
                    rho_temp = eos_h2o(P(i,j,k)./10,[(T_bound-0.0011) (T_bound - 0.001)]);
             
                    mV_temp = 1./rho_temp*1000.*mmH2O; % molar volume
                    mV_pb = mV_temp(2); % molar volume at phase boundary
                    dmV_pb = (-mV_temp(1)+ mV_temp(2))/0.0001; % approximate temperature derivative at phase boundary
                    o1 = dmV_pb - 3.*o2(i,j,k).*T_bound.^2;
                    o0 = mV_pb - o1*T_bound - o2(i,j,k).*T_bound.^3;
                    mV = o0 + o1.*T(i,j,k) + o2(i,j,k).*T(i,j,k).^3;
                    rho(i,j,k) = 1/mV*1000*(X(i,j,k)*mmNaCl + (1-X(i,j,k))*mmH2O);
                end
            else
                rhoTv = eos_h2o(P(i,j,k)./10,Tv(i,j,k));
                % approach is valid for molar volumes -> transform density into
                % molar volume mV
                mV = 1/rhoTv*1000.*mmH2O;
                rho(i,j,k) = 1/mV*1000*(X(i,j,k)*mmNaCl + (1-X(i,j,k))*mmH2O);
            end
            % transform molar volume back into density for NaCl solution
       
        end
    end
end

%------------------------------------------------------------------------
% calculation of the specific enthalpy and specific heat capacity
%------------------------------------------------------------------------

% parameters that are needed for the calculation of the specific enthalpy

q11 = -32.1724 + 0.0621255.*P;
q21 = -1.69513 - 4.52781e-4.*P - 6.04279e-8.*P.^2;
q22 = 0.0612567 + 1.88082e-5.*P;

% parameters q1 and q2 for xNaCl = 1:

q1_XNaCl0 = 0;
q2_XNaCl0 = 1;

% parameters q1 and q2 for xNaCl = 1:
q1_XNaCl1 =  47.9048 - 9.36994e-3.*P + 6.51059e-6.*P.^2;
q2_XNaCl1 =  0.241022 + 3.45087e-5.*P - 4.28356e-9.*P.^2;
        
% use solutions for XNaCl = 0 and XNaCl = 1 to calculate missing parameters       

q10 = q1_XNaCl1;
q12 = q1_XNaCl0 - q10 - q11;

q20 = q2_XNaCl0 - q21.*sqrt(q22);
q23 = q2_XNaCl1 - q20 -q21.*sqrt(1+q22);

% calculat parameters n1 and n2

q1 = q10 + q11.*(1 - X) + q12.*(1 - X).^2;
q2 = q20 + q21.*sqrt(X + q22) + q23.*X;


% calculation of modified temperature Tv

Tv = q1 + q2.*T;

% calculation of molar volume by calling external function for density of
% pure water with modified temperatures

for i = 1:s1
    for j = 1:s2
        for k = 1:s3
            [~, cpw , ~, ~, ~, hTv] = eos_h2o(P(i,j,k)./10,Tv(i,j,k));
            h(i,j,k) = hTv; % specific enthalpy
            cp(i,j,k) = cpw.*q2(i,j,k);
        end
    end
end

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function pp = phaseboundpw(T) 

% calculates the liquid-vapor phase boundary for pure water

T = T+273.15; %temperature in K

n = zeros(8,1);

n(1) = 0.11670521452767d4;     n(6) = 0.14915108613530d2;
n(2) = -0.72421316703206d6;    n(7) = -0.48232657361591d4;
n(3) = -0.17073846940092d2;    n(8) = 0.40511340542057d6;
n(4) = 0.12020824702470d5;     n(9) = -0.23855557567849;
n(5) = -0.32325550322333d7;    n(10) = 0.65017534844798d3;

theta = T + n(9)./(T - n(10));
A = theta.^2 + n(1).*theta + n(2);
B = n(3).*theta.^2 + n(4).*theta + n(5);
C = n(6).*theta.^2 + n(7).*theta + n(8);

p_b1 = (2.*C./(-B + (B.^2 - 4.*A.*C).^(0.5))).^4;

%------------------------------------------------------------------------
% Calculation of "phase boundary pressure" p_b2 above the citical point
%------------------------------------------------------------------------

p_b2 = 0.34805185628969d3 - 0.11671859879975d1.*T + 0.10192970039326d-2.*T.^2;

%------------------------------------------------------------------------
% Calculation of "phase boundary" for the given input temperatures
%------------------------------------------------------------------------

% depending on the temperature values, decide, which equation to use 

decide = T > 623.15;

pp = p_b2.*decide - p_b1.*(decide-1); % in MPa
pp = pp.*10;  % in bar

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function f = minfunc_Tbound(T,P) 
% minimizing function for calculating the phase boundary temperature at
% given pressure

P_bound = phaseboundpw(T) ;
f = P_bound - P;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function x = m2x(m)
% calculates mole fraction from molality

mmH2O = 18.015;  % molar mass of H2O
x = m.*mmH2O./(1000 + m.*mmH2O);