function [fCH4] = fugacity_ch4sw(press,temp,sal,mch4)

% FUGACITY_CH4SW -  calculates the fugacity of dissolved CH4 gas
%     in equilibrium with seawater
%
%     Usage: f = fugacity_ch4sw(p,T,S,m)
%
%     Input parameters (scalars or vectors): 
%           p        -  pressure [MPa]      (0.1 < p < 200)
%           T        -  temperature [degC]  (0 < T < 250)
%           S        -  practical salinity  (0 < S < 350)
%           m        -  molality of dissolved CH4 in seawater [mol/kg],
%                       default is saturation concentration
%
%     Output parameters: 
%           f        - fugacity of CH4 in seawater [MPa]
% 
%     References: 
%     Harvey (1996) AIChE 42, 1491-1494. (Henry constant)
%     Duan & Mao (2006) GCA 70, 3369-3386. (gamma term + partial molar volume of CH4)
%     Duan et al. (2008) Energy & Fuels 22, 1666-1674. (molar volume of CH4)
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history:
%       September 2011: function name changed: 
%                       fugacity_ch4h2o -> fugacity_ch4sw
%       January 2013: internal function call changed from henry_ch4gas to
%                       henry_ch4
%       February 2013: function call for seasalt updated
%                      mch4 added to input; kH added to output; script
%                      changed to match Henry constant calculations in
%                      soluhenry_ch4sw;
%                      mNaCl substituded by ionic strength equivalent
%      June 2013:     Henry constant is calculated by calling an external function
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('FUGACITY_CH4SW:Input','FUGACITY_CH4SW: Missing input parameter! Please provide p, T and S!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press< 0.1 | press>50) && ~any(temp< 0 | temp>360) && ~any(sal< 0 | sal>40)
else
    bad = find(press<0.1 | press > 50);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp > 360);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>40);
    sal(bad)=NaN*ones(size(bad));
    warning('FUGACITY_CH4SW:OutOfParameterRange','FUGACITY_CH4SW: Input values out of range!')
    warning('FUGACITY_CH4SW:OutOfParameterRange','Equations valid for P = 0.1-50 MPa, T = 0-360 degC, S= 0-40.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

   if nargin == 3 % if no CH4 concentration is provided by user, assume saturation
       %call external function to calculate solubility
        mch4 = solu_ch4gas(press,temp,sal);
   end

%------------------------------------------------------------------------
% Calculation of solubility
%------------------------------------------------------------------------
% call external function to calculate pressure and salinity corrected Henry
% constant
[~,~,kH] = soluhenry_ch4sw(press,temp,sal,0.1); % [1/MPa]

x = mch4./(mch4 + 55.5); % mole fraction of CH4

fCH4 = x./kH; % fugacity

end

