function [fCO2] = fugacity_co2sw(press,temp,sal,mco2)

% FUGACITY_CO2SW -  calculates the fugacity of dissolved CO2 gas
%     in equilibrium with seawater
%
%     Usage: f = fugacity_co2sw(p,T,S,m)
%
%     Input parameters (scalars or vectors, m: scalar/matrix of size pxtxS): 
%           p        -  pressure [MPa]     (0.1 < p < 35 or gas/liquid phase boundary)
%           T        -  temperature [degC] (0 < T < 162)
%           S        -  practical salinity (S < 263)
%           m        -  molality of dissolved CO2 in seawater [mol/kg],
%                       default is saturation concentration
%
%     Output parameters: 
%           f       - fugacity of CO2 in seawater [MPa]
%
%     References:
%     Harvey (1996) AIChE 42, 1491-1494.
%     Wagner & Pruss (2002) J.Phys.Chem.Ref.Data 31, 387-535. (vapor pressure of water)
%     Duan et al. (2006) Mar.Chem. 98, 131-139. (term gamma)
%     Plyasunov et al. (2000) GCA 64, 495-512. (partial molar volume of CO2) 
%
%     Comments: 
%     Formalism is not valid for pT conditions where pure CO2 is liquid.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               IFM-GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history:
%       November 2009: Constants k0 - k3 changed, pressure is now
%           consistently used in MPa (lines 148-151)
%       September 2011: function name changed: 
%                       fugacity_co2h2o -> fugacity_co2sw
%       December 2012: Henry constant now has the unit [mol/kg/MPa]
%       January  2013: Internal function call changed from henry_co2gas to
%                      henry_co2; 
%                      mNaCl substituted by ionic strength equivalent
%       February 2013: mco2 added to input; kH added to output; script
%           changed to match Henry constant calculations in soluhenry_co2sw
%      June 2013:     Henry constant is calculated by calling an external function
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('FUGACITY_CO2SW:Input','FUGACITY_CO2SW: Missing input parameter! Please provide p, T and S!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press< 0.1 | press>50) && ~any(temp< 0 | temp>80) && ~any(sal< 0 | sal>40)
else
    bad = find(press<0.1 | press > 50);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp > 80);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>40);
    sal(bad)=NaN*ones(size(bad));
    warning('FUGACITY_CO2SW:OutOfParameterRange','FUGACITY_CO2SW: Input values out of range!')
    warning('FUGACITY_CO2SW:OutOfParameterRange','Equations valid for P = 0.1-50 MPa, T = 0-80 degC, S= 0-40.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

   if nargin == 3 % if no CO2 concentration is provided by user, assume saturation
       %call external function to calculate solubility
        mco2 = solu_co2gl(press,temp,sal);
   end

%------------------------------------------------------------------------
% Calculation of fugacity
%------------------------------------------------------------------------
% call external function to calculate pressure and salinity corrected Henry
% constant
[~,~,kH] = soluhenry_co2sw(press,temp,sal,0.1); % [1/MPa]

x = mco2./(mco2 + 55.5); % mole fraction of CO2

fCO2 = x./kH; % fugacity

end

