function [f_ch4, Theta_CH4_s, Theta_CH4_l] = fugacity_h2omh(press,temp)

% FUGACITY_H2OMH - calculates the fugacity of water in CH4 SI-hydrate
%
%       Usage: [f theta_s theta_l] = fugacity_h2omh(p,T)
%
%       Input parameters (scalars or vectors): 
%           p        - pressure [MPa]  
%           T        - temperature [degC] (-133 < T < 57)
%   
%       Output parameter:
%           f        - fugacity of water in CH4 SI-hydrate [MPa]
%           theta_s  - fractional occupancy of small hydrate cages
%           theta_l  - fractional occupancy of large hydrate cages
%           
%       References:
%       Klauda & Sandler (2003) Chem. Eng. Sci. 58, 27-41.
%       Klauda & Sandler (2000) Ind. Eng. Chem. Res. 39, 3377-3386.
%
%       Comments: 
%       Coefficients A-D for the vapor pressure of the empty hydrate
%       lattice are derived from a fit to match the liquid water fugacity
%       as provided by the toolbox function fugacity_sw to the water
%       fugacity as calculated by the Klauda & Sandler formalism at
%       the hydrate phase boundary (calculated by phase_ch4sw).
%
%       Fugacities of CH4 in the hydrate phase are approximated as
%       fugacities of gas phase CH4. For the calculation of hydrate phase
%       fugacities, the total mount of CH4 in the hydrate phase has to be
%       known.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
%   Modification history: 
%       December 2009: Parameter name in header corrected (line 12)
%       November 2012: syntax changed to Matlab2012 style, 
%                       error corrected: fugacity coefficient is multiplied
%                       with pressure to obtain fugacity
%       March 2013:     Fractional cage occupancies added as output
%                       parameters
%------------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('FUGACITY_H2OMH:Input','FUGACITY_H2OMH: Missing input parameter! Please provide p and T!')
else


%------------------------------------------------------------------------
% Check validity of data range!
%------------------------------------------------------------------------

if ~any(temp<-133 | temp>57) 
else
    bad = find(temp<-133 | temp > 57);
    temp(bad)=NaN*ones(size(bad));
    warning('FUGACITY_H2OMH:OutOfParameterRange','FUGACITY_H2OMH: Input values out of range!')
    warning('FUGACITY_H2OMH:OutOfParameterRange','Equations valid for T = -133-57 degC.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[P, T] = ndgrid(press,temp+273.15); %  pressure in MPa, temperature in K

% call external function to calculate gas phase fugacity coefficient phi_CH4
[~ , ~ , ~, ~, ~, ~, phi_CH4] = eos_ch4(press,temp); 
f_CH4 = phi_CH4.*1e6.*P; % fugacity [Pa]

Na = 6.0222045e23; % Avogadro's number
Nb = 46; % water molecules per unit cell
R = 8.314472; % gas constant [J/mol/K]

%------------------------------------------------------------------------
% Calculation of the molar volume of empty SI hydrate
% Comment: Power of 3 inserted into equation (seems to be missing in paper)
%------------------------------------------------------------------------

V_b = (11.835 + 2.217e-5.*T + 2.242e-6.*T.^2).^3.*1e-30.*Na./Nb - ...
    8.006e-9.*P + 5.448e-12.*P.^2; %  [m3/mol]

%------------------------------------------------------------------------
% Calculation of the Langmuir constants (s = small cage, l = large cage)
%------------------------------------------------------------------------

Ac_CH4_s = -23.6453;
Ac_CH4_l = -22.0651;
Bc_CH4_s = 2714.5643;
Bc_CH4_l = 2760.1604;
Dc_CH4_s = 0;
Dc_CH4_l = 0;

% Langmuir constants:
C_CH4_s = exp(Ac_CH4_s + Bc_CH4_s./T + Dc_CH4_s./T.^2);
C_CH4_l = exp(Ac_CH4_l + Bc_CH4_l./T + Dc_CH4_l./T.^2);

%------------------------------------------------------------------------
% Calculation of the cage occupancies
%------------------------------------------------------------------------

Theta_CH4_s = C_CH4_s .*f_CH4./(1 + C_CH4_s .*f_CH4);
Theta_CH4_l = C_CH4_l .*f_CH4./(1 + C_CH4_l .*f_CH4);

%------------------------------------------------------------------------
% Calculation of chemical potential difference
%------------------------------------------------------------------------

Delta_mu_CH4 = -R.*T.*(2./46.*log(1 - Theta_CH4_s) + ...
    6./46.*log(1 - Theta_CH4_l));

%------------------------------------------------------------------------
% Calculation of the vapor pressure of the empty hydrate lattice
%------------------------------------------------------------------------
Ai_CH4 =    -5.594165083804380e+02;
Bi_CH4 =   -8.537852570452284e+04;
Di_CH4 =    0.985140475507720;
Ci =  3.188647540215796e+03;

P_sat_CH4 = exp(Ai_CH4.*log(T) + Bi_CH4./T + Ci + Di_CH4.*T); % [Pa]

%------------------------------------------------------------------------
% Calculation of the fugacity of the empty hydrate lattice
%------------------------------------------------------------------------

f_wb_CH4 = P_sat_CH4.*exp(V_b.*(P.*1e6 - P_sat_CH4)./R./T); % [Pa]

%------------------------------------------------------------------------
% Calculation of the fugacity of water in CH4 gas hydrate 
%------------------------------------------------------------------------

f_ch4 = f_wb_CH4.*exp(- Delta_mu_CH4./R./T) .*1e-6; % [MPa]

end
