function [fO2] = fugacity_o2sw(press,temp,sal,mo2)

% FUGACITY_O2SW -  calculates the fugacity of dissolved O2 gas
%      in equilibrium with seawater
%
%      Usage: f = fugacity_o2sw(p,T,S,m)
%
%      Input parameters (scalars or vectors, m: scalar/matrix of size pxTxS): 
%           p      -  pressure [MPa]               (0.1 < p < 20)
%           T      -  temperature [degC]           (0 < T < 127)
%           S      -  practical salinity           (S < 350)
%           m      - molality of dissolved O2 in seawater [mol/kg],
%                       default is saturation concentration
%
%      Output parameters: 
%           f      - fugacity of O2 in seawater [MPa]
% 
%     References: 
%     Harvey (1996) AIChE 42, 1491-1494. (Henry constant)
%     Plyasunov et al. (2000) GCA 64, 495-512. (partial molar volume of O2)
%     Geng & Duan (2010) GCA 74, 5631-5640. (for gamma term)
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('FUGACITY_O2SW:Input','FUGACITY_O2SW: Missing input parameter! Please provide p,T,S and pp(optional)!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>81.8) && ~any(temp<0 | temp>80) ...
        && ~any( sal<0 | sal>120)
else
    bad = find(press<0.1 | press > 81.8);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>80);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>120);
    sal(bad)=NaN*ones(size(bad));
    warning('FUGACITY_O2SW:OutOfParameterRange','FUGACITY_O2SW: Input values out of range!')
    warning('FUGACITY_O2SW:OutOfParameterRange','Equations valid for p = 0.1-81.8 MPa, T= 0-80 degC, S = 0-120.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic parameters
%------------------------------------------------------------------------

   if nargin == 3 % if no O2 concentration is provided by user, assume saturation
       %call external function to calculate solubility
        mo2 = solu_o2sw(press,temp,sal);
   end

%------------------------------------------------------------------------
% Calculation of solubility
%------------------------------------------------------------------------
% call external function to calculate pressure and salinity corrected Henry
% constant
[~,~,kH] = soluhenry_o2sw(press,temp,sal,0.1); % [1/MPa]

x = mo2./(mo2 + 55.5); % mole fraction of O2

fO2 = x./kH; % fugacity
end

