function lambda = heatcond_ch4(press,temp)

% HEATCOND_CH4 - calculates the thermal conductivity of pure CH4 
%       The thermal conductivity of methane is calculated from a multi-
%       parametric equation, including real gas effects.
%
%       Usage: lambda = heatcond_ch4(p,T)
%
%       Input parameters (scalars or vectors): 
%           p       - pressure [MPa]     (0 < p < 50)
%           T       - temperature [degC] (-5 < T < 227)
%   
%       Output parameter:
%           lambda  -   thermal conductivity of CH4 [W/m/K]
%           
%       References:
%       Hanley et al. (1977) J.Phys.Chem.Ref.Data 6, 597-609.
%
%       Comments:
%       The critical term delta_lambda_c is not specified in the paper and
%       is therefore missing in the calculation of this script.
%       The error is largest close to the critical point of methane and is
%       about 1-2% at temperatures above -5 degC. This error is not much
%       larger then the error of the equation itself. As a consequence, the
%       valid parameter range, that originally spanned the temperature
%       range from -178 to 227 degC, was restricted.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       November 2012: syntax changed to Matlab2012 style
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 2
     warning('HEATCOND_CH4:Input','HEATCOND_CH4: Missing input parameter! Please provide p and T!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0 | press>50) && ~any(temp< -5 | temp>227)
else
    bad = find(press<0 | press > 50);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<-5 | temp>227);
    temp(bad)=NaN*ones(size(bad));
    warning('HEATCOND_CH4:OutOfParameterRange','HEATCOND_CH4: Input values out of range!')
    warning('HEATCOND_CH4:OutOfParameterRange','Equations valid for p = 0-50 MPa, T=(-5) - 227 degC.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[~, T] = ndgrid(press,temp+273.15); % temperature in K

ek= 168; % (energy parameter)/(Boltzmann constant) [K]
rhoc = 0.1628; % critical density [g/cm^3]

% call external functions to calculate density of CH4
[rho] = density_ch4(press,temp); %[kg/m^3]
rho = rho./1000;  % [g/cm^3]

%------------------------------------------------------------------------
% Calculation of the ideal gas part of the thermal conductivity: lambda_0
%------------------------------------------------------------------------

G1 = -2.147621e5;
G2 =  2.190461e5;
G3 = -8.618097e4;
G4 =  1.496099e4;
G5 = -4.730660e2;
G6 = -2.331178e2;
G7 =  3.778439e1;
G8 = -2.320481;
G9 =  5.311764e-2;

lambda_0 = G1./T + G2.*T.^(-2/3) + G3.*T.^(-1/3) + G4 + G5.*T.^(1/3) ...
    + G6.*T.^(2/3) + G7.*T + G8.*T.^(4/3) + G9.*T.^(5/3);

%------------------------------------------------------------------------
% Calculation of the dilute gas contribution to thermal conductivity: lambda_1
%------------------------------------------------------------------------

A = -0.25276292;
B =  0.33432859;
C =  1.12;

lambda_1 = A + B.*(C - log(T./ek).^2);

%------------------------------------------------------------------------
% Calculation of the viscosity remainder: lambda_r
%------------------------------------------------------------------------

theta = (rho - rhoc)./rhoc; % reduced density

E= 1;
j1 = -7.0403639907;
j2 = 12.319512908;
j3 = -8.8525979933e2;
j4 = 72.835897919;
j5 =  0.74421462902;
j6 = -2.9706914540;
j7 =  2.22097585016e3;

lambda_r = E.*(exp(j1 + j4./T) .* exp(rho.^(0.1) .* (j2 + j3./T.^(3/2)) + ...
    theta .* rho.^(0.5) .*(j5 + j6./T + j7./T.^2)) -1);

%------------------------------------------------------------------------
% Calculation of the total viscosity: eta
%------------------------------------------------------------------------

lambda = (lambda_0 + lambda_1 + lambda_r); % [mW/m/K]
lambda = lambda./1000; % [W/m/K]

end
