function  hydrate_phasediagram(S,t1,t2,p2)

% HYDRATE_PHASEDIAGRAM -  plots the phasediagrams of the binary systems
%       CH4-H2O and CO2-H2O
%
%     Usage: hydrate_phasediagram(S,t1,t2,p2)
%
%     Input parameters (scalar!):
%           S    - practical salinity (default: 35)
%           t1   - start of temperature axis [degC] (default: -10)
%           t2   - end of temperature axis [degC] (default: 35)
%           p2   - end of pressure axis [m water depth] (default: 2500)
%                  (starting pressure is 0)
%                
%     References:
%     Tishchenko et al. (2005) Chem. Geol. 219, 35-52.
%     Tishchenko et al. (2009) 8th International CO2 Conference, Jena.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------
 
% ------------------------------------------------------------------
%   Modification history:
%       September 2011: name changed in function call: 
%                       phase_ch4h2o -> phase_ch4sw
%       September 2011: name changed in function call: 
%                       phase_co2h2o -> phase_co2sw
%       November 2012: syntax changed to Matlab2012 style
% ------------------------------------------------------------------

warning off all

%------------------------------------------------------------------------
% Check number of input parameters, default value: S = 35 g/kg,
%  T = (-10) - 35 C, p = 0 - 2500 m water depth
%------------------------------------------------------------------------


if nargin == 0 
    S = 35;
end

if nargin < 2 
    t1 = -10;
end

if nargin < 3
    t2 = 35;
end

if nargin < 4
    p2 = 2500;
end

%------------------------------------------------------------------------
% Define temperature range
%------------------------------------------------------------------------

       t=(t1:.1:t2);
       
%------------------------------------------------------------------------
% PT curve of model reservoir 
%------------------------------------------------------------------------ 
%tmod = t; % model temperature: 2C- 20C
%tmod(tmod<2) = NaN;
%tmod(tmod>20)= NaN;
%pmod = (tm - 2) .*1000./30 + 1500;% model pressure (m H2O)

%------------------------------------------------------------------------
% Calculation of  CO2 phase boundaries
%------------------------------------------------------------------------

       % call external function
       [pm,ps,pv,pgh]=phase_co2sw(t,S); % [MPa]
       pm = pm.*1000./9.80665;% [m H2O]
       ps = ps.*1000./9.80665;% [m H2O]
       pv = pv.*1000./9.80665;% [m H2O]
       pgh = pgh.*1000./9.80665;% [m H2O]
       
%------------------------------------------------------------------------
% Plot phase diagram of pure CO2
%------------------------------------------------------------------------
%        figure(3)
%        clf
%        axis ij
%       hold on;
%       plot(t,pm,'k-','LineWidth',2);
%       plot(t,ps,'b-','LineWidth',2);
%       plot(t,pv,'r-','LineWidth',2);
%      %plot(t,pgh,'g--','LineWidth',2);
%       set(gca,'YLim',[0 2100]);
%       xlabel('Temperature / C','FontSize',12,'FontWeight','bold');
%       ylabel('Pressure / m H2O','FontSize',12,'FontWeight','bold');
%       title('Phase diagram of CO_2','FontSize',14,'FontWeight','bold');
%      plot(30.9782,73.773*100./9.80665,'r.','MarkerSize',20);
%      text(30,755,'critical point of CO_2 \rightarrow','HorizontalAlignment','right');
%      text(-90,300,'CO_2 solid','FontWeight','bold');
%      text(-30,500,'CO_2 liquid','FontWeight','bold');
%      text(0,200,'CO_2 gas','FontWeight','bold');
%       hold off;


  
%------------------------------------------------------------------------
% Plot phase diagram of CO2 in (sea)water
%------------------------------------------------------------------------      

       pv2 = pv;
       bad = find(pm>pgh);
       pm(bad)=NaN*ones(size(bad));
       bad = find(pv>pgh);
       pv(bad)=NaN*ones(size(bad));
       bad = find(pv<pgh);
       pv2(bad)=NaN*ones(size(bad));
       bad = find(ps>pgh);
       ps(bad)=NaN*ones(size(bad));

       figure(1)
       clf
       axis ij
       hold on;
       plot(t,pm,'k-','LineWidth',2);
       plot(t,ps,'b-','LineWidth',2);
       plot(t,pv,'r-','LineWidth',2);
       plot(t,pv2,'r:','LineWidth',1);
       plot(t,pgh,'g-','LineWidth',2);
       set(gca,'YLim',[0 p2]);
        set(gca,'XLim',[t1 t2]);
       xlabel('Temperature / C','FontSize',12,'FontWeight','bold');
       ylabel('Pressure / m water depth','FontSize',12,'FontWeight','bold');
       title(['Phase diagram of CO_2 - seawater, S =' , num2str(S)],'FontSize',14,'FontWeight','bold');
       plot(30.9782,752.28,'r.','MarkerSize',20);
       text(29,765,'critical point of CO_2 \rightarrow','HorizontalAlignment','right');
       text(-5,700,'CO_2 hydrate','FontWeight','bold');
       text(15,300,'CO_2 gas + sw','FontWeight','bold');
       text(17,1100,'CO_2 liquid + sw','FontWeight','bold');
       
       hold off;
       
%------------------------------------------------------------------------
% Calculation of  CH4 phase boundaries
%------------------------------------------------------------------------
     
       t=(t1:.01:t2);      
      [~,pghsw]=phase_ch4sw(t,S); % call external function, [MPa]
       pghsw = pghsw.*1000./9.80665;% [m H2O]
      
%------------------------------------------------------------------------
% Plot phase diagram of CH4 in (sea)water
%------------------------------------------------------------------------      
     
       figure(2)
       clf
       axis ij
       hold on;
       plot(t,pghsw,'r-','LineWidth',2);
       set(gca,'YLim',[0 p2]);
       set(gca,'XLim',[t1 t2]);
       xlabel('Temperature / C','FontSize',12,'FontWeight','bold');
       ylabel('Pressure / m water depth','FontSize',12,'FontWeight','bold');
       title(['Phase diagram of CH_4 - seawater, S =', num2str(S)],'FontSize',14,'FontWeight','bold');
       text(-5,800,'CH_4 hydrate','FontWeight','bold');
       text(5,300,'CH_4 gas + sw','FontWeight','bold');
       
       hold off;
       
 warning on all

