function pch4 = mol2pp_ch4(press,temp,sal,mch4)

% MOL2PP_CH4 - calculates the partial pressure of CH4 for a given CH4
%      concentration in aqueous solution using Henry's law.
%      The Henry constant for CH4 gas in water is calculated as a function
%      of pressure, temperature and salinity from a multiparametric equation.
%
%      Usage: pp = mol2pp_ch4(p,T,S,m)
%
%      Input parameters (p,T,S: scalars or vectors; m: scalar/matrix of size P x T x S): 
%           p      -  pressure [MPa]      (0.1 < p < 200)
%           T      -  temperature [degC]  (0 < T < 250)
%           S      -  practical salinity  (0 < S < 350)
%           m      -  molal concentration of CH4 [mol/kg]
%
%      Output parameter: 
%           pp     -  partial pressure of CH4 [MPa] 
% 
%      References:
%      Harvey (1996) AIChE 42, 1491-1494.
%      Wagner & Pruss (2002) J.Phys.Chem.Ref.Data 31, 387-535. (water vapor pressure)
%      Duan & Mao (2006) GCA 70, 3369-3386. (gamma term + partial molar volume of CH4)
%      Duan & Sun (2003) Chem. Geol. 193, 257-271. (Pitzer-type salinity correction)    
%
%      Comments: 
%      No interactions with molecules other than CH4 in the gas phase are
%      considered, because no reliable interaction parameters were found. 
%      Change of phase boundary with pressure is not considered in calculation
%      of partial molar volume.
%        
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (May 2012)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       November 2012: syntax changed to Matlab2012 style
%       February 2013: parameters for calculation of seasalt molalities updated
%       June 2013:     external function is called for calculation of Henry
%                       constant
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 4
     warning('MOL2PP_CH4:Input','MOL2PP_CH4: Missing input parameter! Please provide P,T,S and Mch4!')
else
%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>100) && ~any(temp<0 | temp>360) ...
        && ~any( sal<0 | sal> 263) 
else
    bad = find(press<0.1 | press > 100);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>360);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>263);
    sal(bad)=NaN*ones(size(bad));
    warning('MOL2PP_CH4:OutOfParameterRange','MOL2PP_CH4: Input values out of range!')
    warning('MOL2PP_CH4:OutOfParameterRange','Equations valid for P = 0.1 - 100 MPa, T = 0-360 degC, S = 0-263.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[P] = ndgrid(press,temp,sal); 

% calculate mole fraction x of dissolved CH4
x = mch4./(55.5 + mch4);

% call external function to calculate Henry constant KH

[~,~, KH] = soluhenry_ch4sw(press,temp,sal,0.1); % [1/MPa]

F = x./KH; % calculate fugacity

% solve EOS numerically for pch4 at given fugacity
options=optimset('Display','off','TolFun',1e-16); % set options for fitfunction

pch4=zeros(size(P)); % empty matrix for partial pressure

for i = 1:length(press)
    for j = 1:length(temp)
        for k=1:length(sal)
         if isnan(press(i)) || isnan(temp(j)) || isnan(sal(k)) || isnan(F(i,j,k))
            pch4(i,j,:) = NaN;
         else
            % find starting value for P_ch4
            [~,~,~,~,~,~,ff] = eos_ch4(press(i),temp(j)); % f = fugacity coefficient
            p_init = press(i).*(F(i,j,k)./(press(i).*ff));
            [PP_ch4] = fsolve(@(P_ch4) minfunc(P_ch4,press(i),temp(j),F(i,j,k)),p_init,options);
            pch4(i,j,k) = PP_ch4;
         end
         end
    end
end

 
end

% ----------------------------------------------------------

function D = minfunc(pch4,p,temp,F)
% minimization function to calculate the partial pressure of CH4 in the
% gas phase

% calculate fugacity coefficient
[~,~,~,~,~,~,f] = eos_ch4(p,temp); % f = fugacity coefficient

% calculate difference between initial fugacity coefficient and new
% fugacity coefficient
D = f - F./pch4;
