function pco2 = mol2pp_co2(press,temp,sal,mco2)

% MOL2PP_CO2 -  calculates the partial pressure of CO2 for a given CO2
%     concentration in aqueous solution using Henry's law
%     The Henry constant for CO2 gas in water is calculated as a function
%     of pressure, temperature and salinity from a multiparametric equation.
%
%     Usage: pp = mol2pp_co2((p,T,S,m)
%
%     Input parameters (p,T,S: scalars or vectors; m: scalar/matrix of size(p x T x S)): 
%           p        -  pressure [MPa]     (0.1 < p < 35 or gas/liquid phase boundary)
%           T        -  temperature [degC] (0 < T < 162)
%           S        -  practical salinity (S < 263)
%           m        -  molal concentration of CO2 [mol/kg]
%
%     Output parameter: 
%           pp       - partial pressure of CO2 [MPa] 
% 
%     References:
%     Harvey (1996) AIChE 42, 1491-1494.
%     Wagner & Pruss (2002) J.Phys.Chem.Ref.Data 31, 387-535. (water vapor pressure)
%     Plyasunov et al. (2000) GCA 64, 2779-2795. (Poynting-pressure correction)
%     Duan & Sun (2003) Chem. Geol. 193, 257-271. (Pitzer-type salinity correction)
%
%     Comments: 
%     No interactions with molecules other than CO2 in the gas phase are
%     considered, because no reliable interaction parameters were found. 
%     Change of phase boundary with pressure is not considered in calculation
%     of partial molar volume.
%        
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (May 2012)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       November 2012: syntax changed to Matlab2012 style
%       February 2013: parameters for calculation of seasalt molalities updated
%       June 2013:     external function is called for calculation of Henry
%                       constant
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 4
     warning('MOL2PP_CO2:Input','MOL2PP_CO2: Missing input parameter! Please provide P,T,S and Mco2!')
else
%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>100) && ~any(temp<0 | temp>360) ...
        && ~any( sal<0 | sal> 263) 
else
    bad = find(press<0.1 | press > 100);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>360);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>263);
    sal(bad)=NaN*ones(size(bad));
    warning('MOL2PP_CO2:OutOfParameterRange','MOL2PP_CO2: Input values out of range!')
    warning('MOL2PP_CO2:OutOfParameterRange','Equations valid for P = 0.1 - 100 MPa, T = 0-360 degC, S = 0-263.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[P] = ndgrid(press,temp,sal); 

% calculate mole fraction x of dissolved CH4
x = mco2./(55.5 + mco2);

% call external function to calculate Henry constant KH

[~,~, KH] = soluhenry_co2sw(press,temp,sal,0.1); % [1/MPa]

F = x./KH; % calculate fugacity

% solve EOS numerically for pco2 at given fugacity
options=optimset('Display','off','TolFun',1e-16); % set options for fitfunction

pco2=zeros(size(P)); % empty matrix for partial pressure

for i = 1:length(press)
    for j = 1:length(temp)
        for k = 1:length(sal)
         if isnan(press(i)) || isnan(temp(j)) ||isnan(sal(k)) || isnan(F(i,j,k))
            pco2(i,j,:) = NaN;
         else
            % find statring value for P_co2
            [~,~,~,~,~,~,ff] = eos_co2(press(i),temp(j)); % f = fugacity coefficient
            p_init = press(i).*(F(i,j,k)./(press(i).*ff));
            [PP_co2] = fsolve(@(P_co2) minfunc(P_co2,press(i),temp(j),F(i,j,k)),p_init,options);
            pco2(i,j,k) = PP_co2;
         end
         end
    end
end

 
end

% ----------------------------------------------------------

function D = minfunc(pco2,p,temp,F)
% minimization function to calculate the partial pressure of CO2 in the
% gas phase

% calculate fugacity coefficient
[~,~,~,~,~,~,f] = eos_co2(p,temp); % f = fugacity coefficient

% calculate difference between initial fugacity coefficient and new
% fugacity coefficient
D = f - F./pco2;
