function pn2 = mol2pp_n2(press,temp,sal,mn2)

% MOL2PP_N2 - calculates the partial pressure of N2 for a given N2
%      concentration in aqueous solution using Henry's law.
%      The Henry constant for N2 gas in water is calculated as a function
%      of pressure, temperature and salinity from a multiparametric equation.
%
%      Usage: pp = mol2pp_n2(p,T,S,m)
%
%      Input parameters (p,T,S: scalars or vectors; m: scalar/matrix of size P x T x S): 
%           p      -  pressure [MPa]               (0.1 < p < 35)
%           T      -  temperature [degC]           (0 < T < 317)
%           S      -  practical salinity           (S < 350)
%           m      -  molal concentration of N2 [mol/kg]
%
%      Output parameter: 
%           pp     -  partial pressure of N2 [MPa] 
% 
%     References: 
%     Harvey (1996) AIChE 42, 1491-1494. (Henry constant)
%     Plyasunov et al. (2000) GCA 64, 495-512. (partial molar volume of N2)
%     Mao & Duan (2006) Fluid Phase Equilib. 248, 103-114. (gamma term)
%        
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (June 2013)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 4
     warning('MOL2PP_N2:Input','MOL2PP_N2: Missing input parameter! Please provide P,T,S and m!')
else
%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>35) && ~any(temp<0 | temp>80) ...
        && ~any( sal<0 | sal> 120) 
else
    bad = find(press<0.1 | press > 35);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>80);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>120);
    sal(bad)=NaN*ones(size(bad));
    warning('MOL2PP_N2:OutOfParameterRange','MOL2PP_N2: Input values out of range!')
    warning('MOL2PP_N2:OutOfParameterRange','Equations valid for P = 0.1 - 35 MPa, T = 0-80 degC, S = 0-120.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[P] = ndgrid(press,temp,sal); 

% calculate mole fraction x of dissolved N2
x = mn2./(55.5 + mn2);

% call external function to calculate Henry constant KH

[~,~, KH] = soluhenry_n2sw(press,temp,sal,0.1); % [1/MPa]

F = x./KH; % calculate fugacity

% solve EOS numerically for pn2 at given fugacity
options=optimset('Display','off','TolFun',1e-16); % set options for fitfunction

pn2=zeros(size(P)); % empty matrix for partial pressure

for i = 1:length(press)
    for j = 1:length(temp)
        for k=1:length(sal)
         if isnan(press(i)) || isnan(temp(j)) || isnan(sal(k)) || isnan(F(i,j,k))
            pn2(i,j,:) = NaN;
         else
            % find starting value for P_n2
            [~,~,~,~,~,~,ff] = eos_n2(press(i),temp(j)); % f = fugacity coefficient
            p_init = press(i).*(F(i,j,k)./(press(i).*ff));
            [PP_n2] = fsolve(@(P_n2) minfunc(P_n2,press(i),temp(j),F(i,j,k)),p_init,options);
            pn2(i,j,k) = PP_n2;
         end
         end
    end
end

 
end

% ----------------------------------------------------------

function D = minfunc(pn2,p,temp,F)
% minimization function to calculate the partial pressure of N2 in the
% gas phase

% calculate fugacity coefficient
[~,~,~,~,~,~,f] = eos_n2(p,temp); % f = fugacity coefficient

% calculate difference between initial fugacity coefficient and new
% fugacity coefficient
D = f - F./pn2;
