function po2 = mol2pp_o2(press,temp,sal,mo2)

% MOL2PP_O2 - calculates the partial pressure of O2 for a given O2
%      concentration in aqueous solution using Henry's law.
%      The Henry constant for O2 gas in water is calculated as a function
%      of pressure, temperature and salinity from a multiparametric equation.
%
%      Usage: pp = mol2pp_o2(p,T,S,m)
%
%      Input parameters (p,T,S: scalars or vectors; m: scalar/matrix of size P x T x S): 
%           p      -  pressure [MPa]               (0.1 < p < 20)
%           T      -  temperature [degC]           (0 < T < 127)
%           S      -  practical salinity           (S < 350)
%           m      -  molal concentration of O2 [mol/kg]
%
%      Output parameter: 
%           pp     -  partial pressure of O2 [MPa] 
% 
%      References:
%      Harvey (1996) AIChE 42, 1491-1494.
%      Wagner & Pruss (2002) J.Phys.Chem.Ref.Data 31, 387-535. (water vapor pressure)
%      Plyasunov et al. (2000) GCA 64, 2779-2795. (Poynting-pressure correction)
%      Duan & Sun (2003) Chem. Geol. 193, 257-271. (Pitzer-type salinity correction)    
% 
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (June 2013)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 4
     warning('MOL2PP_O2:Input','MOL2PP_O2: Missing input parameter! Please provide P,T,S and m!')
else
%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>81.8) && ~any(temp<0 | temp>80) ...
        && ~any( sal<0 | sal> 120) 
else
    bad = find(press<0.1 | press > 81.8);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>80);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>120);
    sal(bad)=NaN*ones(size(bad));
    warning('MOL2PP_O2:OutOfParameterRange','MOL2PP_O2: Input values out of range!')
    warning('MOL2PP_O2:OutOfParameterRange','Equations valid for P = 0.1 - 81.8 MPa, T = 0-80 degC, S = 0-120.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[P] = ndgrid(press,temp,sal); 

% calculate mole fraction x of dissolved O2
x = mo2./(55.5 + mo2);

% call external function to calculate Henry constant KH

[~,~, KH] = soluhenry_o2sw(press,temp,sal,0.1); % [1/MPa]

F = x./KH; % calculate fugacity

% solve EOS numerically for po2 at given fugacity
options=optimset('Display','off','TolFun',1e-16); % set options for fitfunction

po2=zeros(size(P)); % empty matrix for partial pressure

for i = 1:length(press)
    for j = 1:length(temp)
        for k=1:length(sal)
         if isnan(press(i)) || isnan(temp(j)) || isnan(sal(k)) || isnan(F(i,j,k))
            po2(i,j,:) = NaN;
         else
            % find starting value for P_o2
            [~,~,~,~,~,~,ff] = eos_o2(press(i),temp(j)); % f = fugacity coefficient
            p_init = press(i).*(F(i,j,k)./(press(i).*ff));
            [PP_o2] = fsolve(@(P_o2) minfunc(P_o2,press(i),temp(j),F(i,j,k)),p_init,options);
            po2(i,j,k) = PP_o2;
         end
         end
    end
end

 
end

% ----------------------------------------------------------

function D = minfunc(po2,p,temp,F)
% minimization function to calculate the partial pressure of O2 in the
% gas phase

% calculate fugacity coefficient
[~,~,~,~,~,~,f] = eos_o2(p,temp); % f = fugacity coefficient

% calculate difference between initial fugacity coefficient and new
% fugacity coefficient
D = f - F./po2;
