 function perm = permeab(Poros,Dp,method)

% PERMEAB -  derives the permeability of the sediment from the porosity
%      and the grain size
%
%      Usage: k = permeab(po,dp,'method')
%
%      Input parameters (poros, dp: scalars or vectors, method: string of SAME size): 
%           po       - porosity
%           dp       - mean particle diameter [m]
%           'method' - choice of empirical relationship between porosity
%                      and permeability to be used.
%           The following 'methods' are included:
%               'ck'   Carman-Kozeny:  k= dp^2/180*(po^3/(1-po)^2))
%               'bk'   Blake-Kozeny:   k= dp^2/150*(po^3/(1-po)^2))
%               'rg'   Rumpf-Gupte:    k= dp^2*po^5.5/5.6
%               'hc'   Hsu-Cheng:      k= dp^2*po^2/(18*(1-po))
%               'bhc'  blended H-C:    k= dp^2*po^3/(180*(1-po)^2)*(1-exp(-10*(1-po)/po))
%
%      Output parameter:
%           k        - permeability [m^2] 
%
%      Default values: 
%      A value for porosity has to be provided. The default value for dp 
%      is 1*10^(-6) m (the particle size in mud) and the default method is 
%      the blended Hsu-Cheng formula.
%   
%      References: 
%      Boudreau (1997) Diagenetic Models and Their Implementation,
%         Springer Verlag.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               IFM-GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
%   Modification history: 
%       September 2011: format information in header extended
%       April 2013: Hsu-Cheng equation corrected
%------------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 1
     warning('PERMEAB:Input','PERMEAB: Missing input parameter! Please provide porosity!')
else

if nargin <2
    Dp = 1D-6; % particle size in mud [m]
end

if nargin <3
    method = '';
end
[poros, dp] = ndgrid(Poros,Dp);

switch method
    case 'ck' % good fit for poros->0.4 (sands)
        perm = dp.^2/180.*(poros.^3./(1-poros).^2);
    case 'bk'
        perm = dp.^2/150.*(poros.^3./(1-poros).^2);
    case 'rg'
        perm = dp.^2.*poros.^5.5 /5.6;
    case 'hc' % good fit for poros->1
        perm = dp.^2.*poros.^2./(18*(1-poros));
    case 'bhc' % at the limits approaching Carman-Kozeny and Hsu-Cheng
        perm = dp.^2.*poros.^3./(180*(1-poros).^2) .* (1 - exp(-10*(1-poros)./poros));
    otherwise % default is blended Hsu-Cheng formula
        perm = dp.^2.*poros.^3./(180.*(1-poros).^2) .* (1 - exp(-10.*(1-poros)./poros));
end
end