function [eqsys] = ph_model(press,temp,sal,varargin)

% PH_MODEL -  calculates the pH and equilibrium concentrations
%      of the acid-base species in seawater as well as their total 
%      concentrations and the total alkalinity.
%
%      Usage:  eqsys = ph_model(p,T,S,'species1',conc1,'species2',conc2,...)
%
%       Input parameters (scalars): 
%           p         -  pressure [MPa], 
%           T         -  temperature [degC] 
%           S         -  salinity [g/kg]
%           'species' -  input species name
%           conc      -  species concentrations [mol/kg] 
%
%       Output parameter: 
%           eqsys     -  structure array of pH and species equilibrium 
%                        concentrations [mol/kg]
%                        of the form ('species1',conc1,'species2',conc2,...)
%
%   To allow for good solver performance it is desirable to provide
%   one species out of each of the following groups, 
%   respectively conservation equations:
%       'pH','H','OH','TAlk'
%       'DIC','CO2','HCO3','CO3'
%       'TBOH4','BOH4','BOH3'
%       'TH2S','H2S','HS'
%       'TNH4','NH4','NH3',
%       'TPO4','H3PO4','H2PO4','HPO4','PO4'
%       'TSi','H4SiO4','H3SiO3'
%       'TSO4','HSO4','SO4'
%       'TF','HF','F'
%   Note: The above spelling of species names must be used for the input.
%
%   To exclude a group of species from the pH calculation the respective
%   total concentration, i.e. DIC, TBOH4, TH2S, TNH4, TPO4, TSi, TSO4, TF,
%   must be set explicitely to zero, e.g. 'TF',0.
%   Default values set for unspecified species groups are:
%    TA = 2.3e-3 eq/kg; DIC = 2.0e-3 mol/kg
%    TBOH4, TSO4, TF are calculated from salinity using the function SEASALT
%    TH2S, TNH4, TPO4, TSi are set to zero
%
%   Depending on the supplied input array the 21 equations solved are:
%    a) 12 mass-action laws
%       0 = Kw - H * OH                H2O <--> H+ + OH-
%       0 = Kc1 * CO2 - H * HCO3       CO2 + H2O <--> H+ + HCO3-
%       0 = Kc2 * HCO3 - H * CO3       HCO3- <--> H+ + CO32-
%       0 = Kb * BOH3 - H * BOH4       B(OH)3 + H2O <--> H+ + B(OH)4-
%       0 = Ks * H2S - H * HS          H2S <--> H+ + HS-
%       0 = Kn * NH4 - H * NH3         NH4+ <--> H+ + NH3
%       0 = Kp1 * H3PO4 - H * H2PO4    H3PO4 <--> H+ + H2PO4-
%       0 = Kp2 * H2PO4 - H * HPO4     H2PO4- <--> H+ + HPO42-
%       0 = Kp3 * HPO4 - H * PO4       HPO42- <--> H+ + PO43-
%       0 = Ksi * H4SiO4 - H * H3SiO4  H4SiO4 <--> H+ + H3SiO4-
%       0 = Kso * HSO4 - H * SO4       HSO4- <--> H+ + SO42-
%       0 = Kf * HF - H * F            HF <--> H+ + F-
%
%    b) 8 mass conservation and 1 alkalinity conservation equation
%       0 = TALK - HCO3 - 2*CO3 - BOH4 - HS - NH3 - HPO4 - 2*PO4 - H3SiO4
%                - OH + H + H3PO4 + HSO4 + HF
%       0 = DIC - CO2 - HCO3 - CO3
%       0 = TBOH4 - BOH4 - BOH3
%       0 = TH2S - H2S - HS
%       0 = TNH4 - NH4 - NH3
%       0 = TPO4 - H3PO4 - H2PO4 - HPO4 - PO4
%       0 = TSi - H4SiO4 - H3SiO4
%       0 = TSO4 - HSO4 - SO4
%       0 = TF - HF - F
%
%    The definition of TALK follows that of Dickson (1981) DSR 28A, 609623.
%    See also discussion in Wolf-Gladrow et al. (2007) Mar.Chem.106, 287-300.
%
%    Equilibrium constants are calculated with KEQUILIB_SW and if the CO2
%    concentration exceeds 0.01 mol/kg KCO2_SW is used.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (February 2013)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       February 2013: function call for seasalt changed
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters and input field names
%------------------------------------------------------------------------
if nargin < 4
     error('PHMODEL:Input','PH_MODEL: Missing input parameter! Please provide p,T,S,species!')
end
if nargin > 12
    error('PHMODEL:Input','PH_MODEL: System is overdetermined - reduce number of species!')
end

% define output field names
eqsys_names = {'pH','H','OH','TAlk','DIC','CO2','HCO3','CO3',...
    'TBOH4','BOH4','BOH3','TH2S','H2S','HS','TNH4','NH4','NH3',...
    'TPO4','H3PO4','H2PO4','HPO4','PO4','TSi','H4SiO4','H3SiO4',...
    'TSO4','HSO4','SO4','TF','HF','F'};

% read input species names and concentration values
species = cell2struct(varargin(2:2:end),varargin(1:2:end-1),2);
species_names = fieldnames(species);
if ismember(0,ismember(species_names,eqsys_names)) == 1
    error('PHMODEL:Input','PH_MODEL: Unknown species names used in structure array!')
end

%------------------------------------------------------------------------
% Check consistency of input and set defaults for missing inputs
%------------------------------------------------------------------------
% convert pH to H field if pH is provided
if ismember('pH',species_names) == 1
    species = setfield(species,'H',10^-(getfield(species,'pH')));
    species = rmfield(species,'pH');
end
% If total species = 0 then remove from calculation
if ismember('DIC',species_names) == 1 && species.DIC == 0
    species.CO2 = 0;
    species.HCO3 = 0;
    species.CO3 = 0;
end
if ismember('TBOH4',species_names) == 1 && species.TBOH4 == 0
    species.BOH4 = 0;
    species.BOH3 = 0;
end
if ismember('TH2S',species_names) == 1 && species.TH2S == 0
    species.H2S = 0;
    species.HS = 0;
end
if ismember('TNH4',species_names) == 1 && species.TNH4 == 0
    species.NH4 = 0;
    species.NH3 = 0;
end
if ismember('TPO4',species_names) == 1 && species.TPO4 == 0
    species.H3PO4 = 0;
    species.H2PO4 = 0;
    species.HPO4 = 0;
    species.PO4 = 0;
end
if ismember('TSi',species_names) == 1 && species.TSi == 0
    species.H4SiO4 = 0;
    species.H3SiO4 = 0;
end
if ismember('TSO4',species_names) == 1 && species.TSO4 == 0
    species.HSO4 = 0;
    species.SO4 = 0;
end
if ismember('TF',species_names) == 1 && species.TF == 0
    species.HF = 0;
    species.F = 0;
end
% Set defaults for missing acid-base equations
[~,~,~,~,~,~,~,~,fluoride,sulfate,borate] = seasalt(sal);
if ismember({'TAlk','DIC','CO2','HCO3','CO3'},species_names) == 0
    species.TAlk = 2.3e-3;
    species.DIC = 2.e-3;
end
if ismember({'TBOH4','BOH4','BOH3'},species_names) == 0
    species.TBOH4 = borate;
end
if ismember({'TH2S','H2S','HS'},species_names) == 0
    species.TH2S = 0;
    species.H2S = 0;
    species.HS = 0;
end
if ismember({'TNH4','NH4','NH3'},species_names) == 0
    species.TNH4 = 0;
    species.NH4 = 0;
    species.NH3 = 0;
end
if ismember({'TPO4','H3PO4','H2PO4','HPO4','PO4'},species_names) == 0
    species.TPO4 = 0;
    species.H3PO4 = 0;
    species.H2PO4 = 0;
    species.HPO4 = 0;
    species.PO4 = 0;
end
if ismember({'TSi','H4SiO4','H3SiO4'},species_names) == 0
    species.TSi = 0;
    species.H4SiO4 = 0;
    species.H3SiO4 = 0;
end
if ismember({'TSO4','HSO4','SO4'},species_names) == 0
    species.TSO4 = sulfate;
end
if ismember({'TF','HF','F'},species_names) == 0
    species.TF = fluoride;
end
species_names = fieldnames(species);
species_conc = cell2mat(struct2cell(species));

%------------------------------------------------------------------------
% Define fit variables and set initial guesses
%------------------------------------------------------------------------
eqsys = cell2struct(num2cell(zeros(size(eqsys_names))),eqsys_names,2);
conc_init = rmfield(eqsys,'pH');
conc_init = rmfield(conc_init,species_names);
conc_names = fieldnames(conc_init);
conc0 = cell2mat(struct2cell(conc_init));

%------------------------------------------------------------------------
% Calculate equilibrium constants
%------------------------------------------------------------------------
Keq_names = {'h2o','co2','hco3','boh4','h2s','nh4','h3po4','h2po4','hpo4','sio2','so4','hf'};
for i=1:length(Keq_names)
    Keq_values(i) = kequilib_sw(press,temp,sal,char(Keq_names(i)));
end
% For CO2 concentration >0.5 mol/kg use KCO2_SW to calculate Keq's
co2toggle = 'off';
if ismember({'CO2'},species_names) == 1
    co2toggle = 'on';
    if species.CO2 > 0.01
        Keq_values(1) = kco2_sw(press,temp,sal,'h2o',species.CO2);
        Keq_values(2) = kco2_sw(press,temp,sal,'co2',species.CO2);
        Keq_values(3) = kco2_sw(press,temp,sal,'hco3',species.CO2);
        Keq_values(11) = kco2_sw(press,temp,sal,'so4',species.CO2);
        Keq_values(12) = kco2_sw(press,temp,sal,'hf',species.CO2);
    end
end

Keq = cell2struct(num2cell(Keq_values),Keq_names,2);

%------------------------------------------------------------------------
% Solve pH system
%------------------------------------------------------------------------
options = optimset('TolFun',1e-16,'TolX',1e-18,'MaxFunEvals',5000,'MaxIter',2000);
[conc,fval,exitflag,output] = ...
    fsolve(@phequations,conc0,options,species_conc,Keq,conc_names,species_names);

%------------------------------------------------------------------------
% Recalculate pH system for high CO2 concentrations
%------------------------------------------------------------------------
dum = find(ismember(conc_names,{'CO2'})==1);
if strcmp(co2toggle,'off') == 1 && conc(dum) > 0.01
    Keq = setfield(Keq,'h2o',kco2_sw(press,temp,sal,'h2o',conc(dum)));
    Keq = setfield(Keq,'co2',kco2_sw(press,temp,sal,'co2',conc(dum)));
    Keq = setfield(Keq,'hco3',kco2_sw(press,temp,sal,'hco3',conc(dum)));
    Keq = setfield(Keq,'so4',kco2_sw(press,temp,sal,'so4',conc(dum)));
    Keq = setfield(Keq,'hf',kco2_sw(press,temp,sal,'hf',conc(dum)));
    conc0 = conc;

    [conc,fval,exitflag,output] = ...
        fsolve(@phequations,conc0,options,species_conc,Keq,conc_names,species_names);
end

%------------------------------------------------------------------------
% Final output
%------------------------------------------------------------------------
% set input species concentrations
for i=1:length(species_names)
    sdum = find(ismember(eqsys_names,species_names(i))==1);
    eqsys = setfield(eqsys,char(eqsys_names(sdum)),species_conc(i));
end
% set solver output concentrations
for i=1:length(conc_names)
    cdum = find(ismember(eqsys_names,conc_names(i))==1);
    eqsys = setfield(eqsys,char(eqsys_names(cdum)),conc(i));
end
% calculate pH from H+ concentration
eqsys.pH = -log10(eqsys.H);

%------------------------------------------------------------------------

function EQFUN = phequations(conc,species_conc,Keq,conc_names,species_names)

%------------------------------------------------------------------------
% Define fixed input species
%------------------------------------------------------------------------
for i=1:length(species_names)
    sdum = genvarname(char(species_names(i)));
    eval([sdum ' = species_conc(i);']);
end

%------------------------------------------------------------------------
% Define fitted variables
%------------------------------------------------------------------------
for i=1:length(conc_names)
    cdum = genvarname(char(conc_names(i)));
    eval([cdum ' = conc(i);']);
end

%------------------------------------------------------------------------
% Equations of pH system
%------------------------------------------------------------------------
EQFUN = [ Keq.h2o - H * OH ;
      Keq.co2 * CO2 - H * HCO3 ;
      Keq.hco3 * HCO3 - H * CO3 ;
      Keq.boh4 * BOH3 - H * BOH4 ;
      Keq.h2s * H2S - H * HS ;
      Keq.nh4 * NH4 - H * NH3 ;
      Keq.h3po4 * H3PO4 - H * H2PO4 ;
      Keq.h2po4 * H2PO4 - H * HPO4 ;
      Keq.hpo4 * HPO4 - H * PO4 ;
      Keq.sio2 * H4SiO4 - H * H3SiO4 ;
      Keq.so4 * HSO4 - H * SO4 ;
      Keq.hf * HF - H * F ;
      TAlk - HCO3 - 2*CO3 - BOH4 - HS - NH3 - HPO4 - 2*PO4 - H3SiO4 - OH + H + H3PO4 + HSO4 + HF ;
      DIC - CO2 - HCO3 - CO3 ;
      TBOH4 - BOH4 - BOH3 ;
      TH2S - H2S - HS ;
      TNH4 - NH4 - NH3 ;
      TPO4 - H3PO4 - H2PO4 - HPO4 - PO4 ;
      TSi - H4SiO4 - H3SiO4 ;
      TSO4 - HSO4 - SO4 ;
      TF - HF - F ];
