function pvh2s = pp2pv_h2s(press,temp,ph2s)

% PP2PV_H2S - converts the partial pressure of H2S in a gas mixture into
%     partial volume
%
%     Usage: pv = pp2pv_h2s(p,T,pp)
%
%     Input parameter (p,T: scalars or vectors; pp: scalar or vector of same size as p): 
%           p      - overall pressure [MPa]        (0.1 < p < 35)
%           T      - temperature [degC]            (0 < T < 137.7)
%           pp     - partial pressure of H2S [MPa] (0 < pp < p)
%           
%     Output parameter: 
%           pv     - partial volume of H2S [ppmV]
%
%     Comments: 
%     Equations are only valid for pressures below the vapor pressure of H2S.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (June 2013)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('PP2PV_H2S:Input','PP2PV_H2S: Missing input parameter! Please provide p,T,S and pp(optional)!')
else
 
%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>35) && ~any(temp<0 | temp>137.7)

else
    bad = find(press<0.1 | press > 35);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>137.7);
    temp(bad)=NaN*ones(size(bad));
    warning('PP2PV_H2S:OutOfParameterRange','PP2PV_H2S: Input values out of range!')
    warning('PP2PV_H2S:OutOfParameterRange','Equations valid for p = 0.1-35 MPa, T= 0-137.7 degC.')
end


%------------------------------------------------------------------------
% Definition of constants and thermodynamic parameters
%------------------------------------------------------------------------
[P] = ndgrid(press,temp+273.15); % temperature in [K]

rho = eos_h2s(press,temp); % density of H2S at total pressure

if size(press) == size(ph2s)
    rhop = eos_h2s(ph2s,temp); % density of H2S at partial pressure
else
    rhop2 = eos_h2s(ph2s,temp); % density of H2S at partial pressure
    rhop = zeros(length(press),length(temp));
    for i=1:length(press)
        rhop(i,:) = rhop2(:,:);
    end
end

pvh2s = rhop./rho; % volume fraction of H2S in gas phase
pvh2s = pvh2s.*1d6; % partial volume in ppmV


% check, if H2S is liquid at given pT conditions 
  pv = pvaph2s(temp + 273.15);
    for i = 1:length(temp)
            for k = 1:length(press)
                if P(k,i) > pv(i) || isnan(pv(i))
                    pvh2s(k,i) = NaN;
                    warning('PP2PV_H2S:OutOfParameterRange','Equations valid for p < p_vap!')
                end
            end
    end


end

% ------------------------------------------------------------
function [Pvh2s] = pvaph2s(T)

%------------------------------------------------------------------------
% Definition of constants and thermodynamic parameters
%------------------------------------------------------------------------

Tc = 373.1; % critical temperature of H2S [K]
Pc = 8.9987; % critical pressure of H2S [MPa]
Tt = 187.6991; % triple point temperature of H2S [K]
Pt = 23.2d-3; % triple point pressure of H2S [MPa]
Ptr = Pt/Pc; % reduced pressure at triple point
Ttr = Tt/Tc; % reduced temperature at triple point
Tbr = 0.57049;
Tr = T./Tc; % reduced temperature
t = (T - Tt)./(Tc -Tt);
tb = (Tbr - Ttr)./(1 - Ttr);

a1 = 0.090204;
b0 = -0.154;

phi = (1 - t)./(1 + a1.*t)./(1 + b0.*t.*(t - tb));

lnpr = phi.*Ttr.*log(Ptr)./Tr;
Pvh2s = exp(lnpr)*Pc;
