function pvn2 = pp2pv_n2(press,temp,pn2)

% PP2PV_N2 - converts the partial pressure of N2 in a gas mixture into
%     partial volume
%
%     Usage: pv = pp2pv_n2(p,T,pp)
%
%     Input parameter (p,T: scalars or vectors; pp: scalar or vector of same size as p): 
%           p      - overall pressure [MPa]        (0 < p < 1000)
%           T      - temperature [degC]            (-73 < T < 727)
%           pp     - partial pressure of N2 [MPa]  (0 < pp < p)
%           
%     Output parameter: 
%           pv     - partial volume of N2 [ppmV]
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (June 2013)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('PP2PV_N2:Input','PP2PV_N2: Missing input parameter! Please provide p, T and pp!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0 | press>1000) && ~any(temp< -73 | temp>727)
else
    bad = find(press<0 | press > 1000);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<-73 | temp>727);
    temp(bad)=NaN*ones(size(bad));
    warning('PP2PV_N2:OutOfParameterRange','PP2PV_N2: Input values out of range!')
    warning('PP2PV_N2:OutOfParameterRange','Equations valid for p = 0-1000 MPa, T=(-73) - 727 degC.')
end

rho = eos_n2(press,temp); % density of N2 at total pressure

if size(press) == size(pn2)
    rhop = eos_n2(pn2,temp); % density of N2 at partial pressure
else
    rhop2 = eos_n2(pn2,temp); % density of N2 at partial pressure
    rhop = zeros(length(press),length(temp));
    for i=1:length(press)
        rhop(i,:) = rhop2(:,:);
    end
end

pvn2 = rhop./rho; % volume fraction of N2 in gas phase
pvn2 = pvn2.*1d6; % partial volume in ppmV

end