function [cNa,cMg,cCa,cK,cSr,cLi,cCl,cBr,cF,cSO4,cTBOH4,cDIC,cOH] = seasalt_molarity(press,temp,sal)

% SEASALT_MOLARITY -  calculates the major salt concentrations in seawater
%      based on its salinity and density (which is also a function of
%      pressure and temperature)
%
%      Usage:  [cNa,cMg,cCa,cK,cSr,cLi,cCl,cBr,cF,cSO4,cTBOH4,cDIC,cOH] =
%               seasalt_molarity(p,T,S) 
%
%      Input parameters (scalar, vector or matrix):
%          p       - pressure [MPa]
%          T       - temeperature [degC]
%          S       - practical salinity of seawater 
%                           (absolute salinity = S*1.005 [g/kg])
%
%      Output parameters: 
%          cI      - molarity of salt component I [mol/l]
%
%      References: 
%      Millero (2008) Deep-Sea Research I 55, 5072.
%      Stoffyn-Egly & MacKenzie (1984) GCA 48, 859-872.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SEASALT_MOLARITY:Input','SEASALT_MOLARITY: Missing input parameter! Please provide p,T,S!')
else

    
%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------
[~,~,S] = ndgrid(press,temp,sal);      

% call external function to calculate seawater density
rho = density_sw(press,temp,sal); % [kg/m^3]
rho = rho./1000;  % [kg/l]


%------------------------------------------------------------------------
% Calculate molarities for salt ions
%------------------------------------------------------------------------

REFS = 35; % reference salinity


% molalities [mol/kg]
cNa = S./REFS*0.4689674.*rho;
cMg = S./REFS*0.0528171.*rho;
cCa = S./REFS*0.0102821.*rho;
cK = S./REFS*0.0102077.*rho;
cSr = S./REFS*0.0000907.*rho;
cBr = S./REFS*0.0008421.*rho;
cF = S./REFS*0.0000683.*rho;
cSO4 = S./REFS*0.0282352.*rho;
cCl = S./REFS*0.5458696.*rho;
cBOH4 = S./REFS*0.0001008.*rho;
cHCO3 = S./REFS*0.0017177.*rho;
cOH = S./REFS*0.0000080.*rho;
cBOH3 = S./REFS*0.0003143.*rho;
cCO3 = S./REFS*0.0002390.*rho;
cCO2 = S./REFS*0.0000096.*rho;
cLi = 181d-6*S./35./6.94;
cTBOH4 = cBOH4 + cBOH3;
cDIC = cHCO3 + cCO3 + cCO2;

 % mLi = Cl*9.36d-6 g/kg_seawater @ S = 35 -> converted to mol/kg_water @
 % Reference Salinity REFS

end
