function [ka, kg, kh] = solu_caso4(press,temp,sal)

% SOLU_CASO4 -  calculates the solubility product  Ksp = m_Ca * m_SO4 for  
%     gypsum (CaSO4*2H2O), anhydrite (CaSO4), ad hemihydrate (CaSO4*0.5H2O)
%     in seawater
%
%     Usage: [ka kg kh] = solu_caso4(p,T,sal)
%
%     Input parameters (scalars or vectors): 
%         p      - pressure [MPa]       (0.1 < p < 100)
%         T      - temperature [degC]   (30 < T < 300)
%         S      - practical salinity   (0 < S < 250) (see comments!)
%
%     Output parameter:
%         ka     - solubility product of anhydrite 
%         kg     - solubility product of gypsum 
%         kh     - solubility product of hemihydrate 
%
%     References:
%     Marshall & Slusher (1968) J.Chem.Eng.Data 13, 83-93. (solubility at constant pressure) 
%     Blount & Dickson (1973) Amer.Mineral. 58, 323-331. (pressure correction)
%
%     Comments: 
%     Ionic strength refers to NaCl in NaCl solutions. 
%     For seawater, the solubility product is assumed to equal the 
%     solubility product in NaCl solutions of the same ionic strength.
%     However, this is true only for moderate temperatures (T < 100 degC)
%     and low or moderate ionic strengths.
%     Equations are valid for:
%     p = 0.1 - 100 MPa
%     T = 60 - 95 degC (Gypsum)
%     T = 100 - 200 degC (Anhydrite)
%     S = 0 - 250 (==> I = 0 - 5 mol/kg)
%       
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLU_CASO4:Input','SOLU_CASO4: Missing input parameter! Please provide p, T and I!')
else


%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>100) && ~any(temp<(30) | temp>300) && ~any(sal<(0) | sal>250) 
        
else
    bad = find(press<0.1 | press > 100);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<(30) | temp>300);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<(0) | sal>250);
    sal(bad)=NaN*ones(size(bad));

 warning('SOLU_CaSO4:OutOfParameterRange','SOLU_CaSO4: Input values out of range!')
 warning('SOLU_CaSO4:OutOfParameterRange','Equations valid for p = 0.1-100 MPa, T=30-300 degC, S = 0-250.')
end


%------------------------------------------------------------------------
% Definition of constants, thermodynamic variables and parameters
%------------------------------------------------------------------------
isw = i_sw(sal); % calculate ionic strength from salinity

[p, T, I] = ndgrid(press.*10, temp+273.15,isw);% pressure in bar, temperature in K


% define arrays for gypsum parameters
A = zeros(size(T));
Bg = zeros(size(T));
Cg = zeros(size(T));
S = zeros(size(T));

% Parameter values for defined temperatures, as given in the papers

temp1 = [(0:25:200) 250 300];

temp3 = [0 5 10 15 20 23 25 30 35 40 45 50 60 65 70 80 82 85 90 95 100 110 125 150 175 200 250 300];

A_i = [1.45 1.458 1.468 1.479 1.49 1.495 1.5 1.51 1.52 1.53 1.537 1.544 1.558 1.564 1.57 1.58 1.581 1.584 1.588 1.591 1.594 1.595 1.6 1.6 1.6 1.6 1.6 1.6]; % temp3

B_i = [0.088 0.062 0.0474 0.036 0.0272 0.0224 0.0194 0.0124 0.006 0.0022 0.0011 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0] ; % temp3

C_i = [0.0234 0.0202 0.0182 0.0164 0.0148 0.0142 0.0134 0.013 0.0124 0.012 0.0114 0.0108 0.0096 0.009 0.0082 0.0064 0.006 0.0054 0.0042 0.003 0.002 0.0008 0 0 0 0 0 0]; % temp3

S_i = [0.4875 0.5080 0.5337 0.5645 0.6006 0.6422 0.6900 0.7451 0.8097 0.9848 1.2870]; % temp1

%------------------------------------------------------------------------
% Interpolation of parameter values for input temperatures
%------------------------------------------------------------------------

for i = 1:length(temp)
if temp(i) >= 300
    % if temperature is equal or larger than 300 C, take values for 300C
    A(:,i,:) = 1.6;
    Bg(:,i,:) = 0;
    Cg(:,i,:) = 0;
    S(:,i,:) = 1.2870;
elseif ~isnan(temp(i))
    % find the first temperature values from temperature lists that is larger
    % than the input temperature 
    ind1 = find(temp1 >= temp(i));
    ind1_1 = ind1(1);
    
    ind3 = find(temp3 >= temp(i));
    ind3_1 = ind3(1);
    
    % linear extrapolation of parameter values
    
    faktor1 = (temp(i) - temp1(ind1_1 -1))/(temp1(ind1_1) - temp1(ind1_1 -1));
    faktor3 = (temp(i) - temp3(ind3_1 -1))/(temp3(ind3_1) - temp3(ind3_1 -1));
    
    A(:,i,:) = A_i(ind3_1 -1) + faktor3 * (A_i(ind3_1) - A_i(ind3_1 -1));
    Bg(:,i,:) = B_i(ind3_1 -1) + faktor3 * (B_i(ind3_1) - B_i(ind3_1 -1));
    Cg(:,i,:) = C_i(ind3_1 -1) + faktor3 * (C_i(ind3_1) - C_i(ind3_1 -1));
    S(:,i,:) = S_i(ind1_1 -1) + faktor1 * (S_i(ind1_1) - S_i(ind1_1 -1));
   
end   
end

% calculation of parameters for anhydrite
Ba = Bg - 0.02;
Ca = Cg + 0.003;

% calculation of parameters for hemihydrate
Bh = Ba + 0.25.*(Bg - Ba);
Ch = Ca + 0.25.*(Cg - Ca);


%------------------------------------------------------------------------
% Calculation of solubility product for anhydrite
%------------------------------------------------------------------------
m = zeros(size(T));
logka0 = -215.509 + 85.685.*log10(T) + 6075.2./T - 0.070707.*T ;

for i = 1:100
    I_1 = I + 4.*m;
logka = logka0 + 8.*S.*sqrt(I_1)./ ...
    (1 + A.*sqrt(I_1)) + Ba.*(I_1)...
    - Ca.*(I_1).^2;
m = sqrt(10.^(logka));
end

ka = 10.^logka;

lnma = log(sqrt(ka));

lnma = lnma + 1.22e-3.*p - 0.0028e-3.*p.*(T -273.15) ...
     + 0.14e-7.*p.*(T - 273.15).^2;
 ka = exp(lnma).^2;
%------------------------------------------------------------------------
% Calculation of solubility product for gypsum
%------------------------------------------------------------------------
m = zeros(size(T));

logkg0 = 390.9619 - 152.6246.*log10(T) - 12545.62./T + 0.0818493.*T;

for i = 1:10
    I_1 = I + 4.*m;
logkg = logkg0 + 8.*S.*sqrt(I_1)./ ...
    (1 + A.*sqrt(I_1)) + Bg.*(I_1)...
    - Cg.*(I_1).^2;
m = sqrt(10.^(logkg));
end

kg = 10.^(logkg);

lnmg = log(sqrt(kg));

lnmg = lnmg + 0.84e-3.*p;
 kg = exp(lnmg).^2;

%------------------------------------------------------------------------
% Calculation of solubility product for hemihydrate (bassanite)
%------------------------------------------------------------------------
m = zeros(size(T));

logkh0 = 154.527 - 54.958.*log10(T) - 6640./T ;

for i = 1:100
    I_1 = I + 4.*m;
logkh = logkh0 + 8.*S.*sqrt(I_1)./(1 + A.*sqrt(I_1)) + Bh.*I_1 - Ch.*I_1.^2;
m = sqrt(10.^(logkh));
end

kh = 10.^(logkh);

% no pressure correction for Hemihydrate available
end
