function mCO2sw = solu_co2(press,temp,sal) 

% SOLU_CO2 - calculates the CO2 equilibrium concentration in seawater with 
%     respect to gas phase, liquid phase or hydrate phase.
%     The phase at given parameters p,T,S is determined and the appropriate
%     function for the calculation of the solubility is called.
%
%     Usage: m = solu_co2(p,T,S)
%
%     Input parameters (scalars or vectors): 
%           p        -  pressure [MPa]
%           T        -  temperature [degC] 
%           S        -  practical salinity
%     
%     Output parameter: 
%           m        - molality of CO2 in seawater [mol/kg]
% 
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
%   Modification history: 
%       December 2009: filename in header corrected (line 5)
%       September 2011: name changed in function call: 
%                       phase_co2h2o -> phase_co2sw
%       November 2012: syntax changed to Matlab2012 style
%------------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLU_CO2:Input','SOLU_CO2: Missing input parameter! Please provide p,T and S!')
else


%------------------------------------------------------------------------
% Determine The phase of CO2 and call the corresponding external programs 
% for calculations in gas phase or hydrate phase.
%------------------------------------------------------------------------

% call external function to calculate the phase boundaries
[~,~,~,pgh] = phase_co2sw(temp,sal); 

% resize matrices to account for pressure vector
Pgh= zeros(length(press),length(temp),length(sal));

for i=1:size(press,2) % create matrices for phase boundaries
Pgh(i,:,:) = pgh(:,:); 
end

[P] = ndgrid(press,temp,sal); % pressure in MPa

% call external function to calculate molality of dissolved CO2 in hydrate phase
[mCO2sw] = solu_co2gh(press,temp,sal); % dissolved CO2 in hydrate phase
% call external function to calculate molality of dissolved CH4 in gas phase
[mCO2sw_gl] =  solu_co2gl(press,temp,sal); % dissolved CO2 in gas and liquid phase 

% Determine, if PTS-values are within the hydrate stability field. Decide,
% which equations to use for calculation of solubility.

   decide = P < Pgh;
   mCO2sw(decide) = mCO2sw_gl(decide);
    
end
