function m = solu_h2ssw(press,temp,sal)

% SOLU_H2SSW -  calculates the equilibrium concentration of H2S in seawater
%   
%     Usage:   m = solu_h2ssw(p,T,S)
%
%     Input parameters (scalars or vectors): 
%         p        - pressure [MPa]     (0.1 < P < 20)
%         T        - temperature [degC] (0 < T < 127)
%         S        - practical salinity (S < 350)
%     
%     Output parameter: 
%         m        - molality of H2S in seawater [mol/kg]
%
%     References: 
%     Duan et al. (2007) Energy & Fuels 21, 2056-2065.
%     Duan et al. (1996) Chemical Geology 130, 15-20. (H2S-EOS)
%     Shibue (2003) Fluid Phase Equilibria 213, 39-51. (water saturation pressure)
%     Velasco et al. (2008) J.Chem.Thermo. 40, 789-797. (H2S vapor pressure)
%        
% -----------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (January 2013)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% -----------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLU_H2SSW:Input','SOLU_H2SSW: Missing input parameter! Please provide p,T and S!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>20) && ~any(temp<0 | temp>127) ...
        && ~any( sal<0 | sal>350)
else
    bad = find(press<0.1 | press > 20);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>127);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>350);
    sal(bad)=NaN*ones(size(bad));
    warning('SOLU_H2SSW:OutOfParameterRange','SOLU_H2SSW: Input values out of range!')
    warning('SOLU_H2SSW:OutOfParameterRange','Equations valid for p = 0.1-20 MPa, T= 0-127 degC, S = 0-350.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic parameters
%------------------------------------------------------------------------

[P, T, S] = ndgrid(press*10,temp+273.15,sal); % pressure in [bar] temperature in [K]

mmh2o    = 18.0152; % molar mass of H2O [g/mol]
mh2o = 1/mmh2o * 1000; % [mol/g * g/m^3] = [mol/kg]

C1m = 42.564957; 
C2m = -8.6260377d-2; 
C3m =-6084.3775; 
C4m =  6.8714437d-5;
C5m =-102.76849;
C6m =  8.4482895d-4; 
C7m = -1.0590768;
C8m =  3.5665902d-3;


C1l = 8.5004999d-2; 
C2l = 3.5330378d-5;
C3l = -1.5882605;
C6l = 1.1894926d-5;


C1x = -1.0832589d-2;

% call external function to calculate the composition of seawater
[mNa,mMg,mCa,mK,~,~,mCl,~,~,mSO4] = seasalt(S); % m [mol/kg]

mNaCl = i_sw(S); % use ionic strength as a surrogate for NaCl concentration
mfnacl = (mNaCl )./(mh2o +  mNaCl);

%------------------------------------------------------------------------
%Calculation of water saturation pressure of from a multiparametric equation.
%------------------------------------------------------------------------
[psh2o] = satpress(mfnacl,T);

%------------------------------------------------------------------------
% Calculation of the mole fraction of H2S in the gas phase
%------------------------------------------------------------------------
yH2S = (P - psh2o)./P;

%------------------------------------------------------------------------
% Calculation of mu0(H2S)/R/T and Pitzer interaction coefficients lambda and
% xi
%------------------------------------------------------------------------

% calculation of mu0(H2S)/R/T
muRT = C1m + C2m.*T + C3m./T + C4m.*T.^2 + C5m./(680-T) + C6m.*P + ...
       C7m.*P./(680-T)  + C8m.*P.^2./T;

% calculation of lambda
lambda = C1l + C2l.*T + C3l./T + C6l.*P;

% calculation of xi
xi = C1x;

%------------------------------------------------------------------------
% solve equation of state for volume of H2S
%------------------------------------------------------------------------

V = zeros(size(P)); % empty matrix for Vm
options=optimset('Display','off','TolFun',1e-16); % set options for fitfunction

for i=1:size(P,1)
    for j=1:size(P,2)
        if isnan(press(i)) || isnan(temp(j))
            V(i,j,:) = NaN;
        else  
            V_init = guessfunc(press(i),temp(j)); % starting value for solver
            % calculate volume by numerical iteration
            V(i,j,:)= fsolve(@(V) minfunc(V,P(i,j,1),T(i,j,1)), V_init,options);
        end
    end
end

 [~, phih2s] = eqofstate_h2s(T,V); % fugacity coefficient of gaseous H2S
 
%------------------------------------------------------------------------
% calculate solubility m
%------------------------------------------------------------------------

m = exp(log(yH2S.*phih2s.*P) - muRT - 2.*lambda.*(mNa + mK + 2*mCa + 2*mMg)...
    - xi.*(mNa + mK + 2*mCa + 2*mMg).*(mCl + 2*mSO4) - 0.18*mSO4);

end
%------------------------------------------------------------------------
%------------------------------------------------------------------------

%------------------------------------------------------------------------
function [Z, phi] = eqofstate_h2s(T,V)
%calculates the gas phase fugacity of H2S

%------------------------------------------------------------------------
% Definition of constants and thermodynamic parameters
%------------------------------------------------------------------------

R = 8.314467; % gas constant [Pa m^3/K/mol]
Tc = 373.6; % critical temperature [K]
Pc = 90.08; % critical pressure [bar]
Vc = R*Tc./Pc;  % pseudo-critical volume

Tr = T./Tc;     % reduced temperature
Vr = V./Vc;  % reduced volume

a1 =5.2386075d-2;
a2 = -2.7463906d-1;
a3 = -9.6760173d-2;
a4 = 1.3618104d-2;
a5 = -8.8681753d-2;
a6 = 4.1176908d-2;
a7 =3.6354018d-4;
a8 = 2.2719194d-3;
a9 = -7.6962514d-4;
a10 = -2.1948579d-5;
a11 = -1.1707631d-4;
a12 = 4.0756926d-5;
a13 = 5.7582260d-2;
a14 = 1.00;
a15 =0.06;

T1 = (a1 + a2./Tr.^2 + a3./Tr.^3)./Vr ;
T2 = (a4 + a5./Tr.^2 + a6./Tr.^3)./Vr.^2;
T3 = (a7 + a8./Tr.^2 + a9./Tr.^3)./Vr.^4;
T4 = (a10 + a11./Tr.^2 + a12./Tr.^3)./Vr.^5;

%------------------------------------------------------------------------
% Calculation of Z and fugacity coefficient phi
%------------------------------------------------------------------------

Z = 1 + T1 + T2 + T3 + T4 + ...
    a13./Tr.^3./Vr.^2.*(a14 + a15./Vr.^2).*exp(-a15./Vr.^2);

phi = exp(Z - 1 - log(Z) + T1 + T2./2 + T3./4 + T4./5 +...
     a13./2./Tr.^3./a15.*(a14 + 1 - (a14 + 1 + a15./Vr.^2).*exp(-a15./Vr.^2)));

 %------------------------------------------------------------------------
function f = minfunc(V,P,T)
%minimization function for calculation of Vm

R = 8.314467; % gas constant [Pa m^3/K/mol]
Tc = 373.6; % critical temperature [K]
Pc = 90.08; % critical pressure [bar]
Vc = R*Tc./Pc;  % pseudo-critical volume

Tr = T./Tc;  % reduced temperature
Vr = V./Vc;  % reduced volume
Pr = P./Pc;  % reduced pressure

% call external function to calculate Z
Z = eqofstate_h2s(T,V);
f = Z - Pr.*Vr./Tr;

% ------------------------------------------------------------

function [psh2o] = satpress(mfnacl,T)

% calculation of saturation pressure of water [bar]

Pc = 22.064; % in MPa
Tc = 647.096;
Tr = T./Tc;
a1 = 0.128746;
a2 = -0.731097;
a3 = -315.058;
b1 = 392.767;
b2 = -2464.4;
u = 0.024;
c1 = -7.85951783;
c2 = 1.84408259;
c3 = -11.7866497;
c4 = 22.6807411;
c5 = -15.9618719;
c6 = 1.80122502;
x = mfnacl;


lnppc = 1./Tr.*( c1.*(1-Tr) + c2.*(1-Tr).^(1.5) + c3.*(1-Tr).^3) ...
        + 1./Tr.*(c4.*(1-Tr).^(3.5) + c5.*(1-Tr).^4 + c6.*(1-Tr).^(7.5));
    
  if (x < u)
      h = a2.*x./(x+a1.^2)+a3.*x.^2;
  else
      h = ((a1.^2.*a2)./(u + a1.^2).^2 + 2.*a3.*u).*(x - u) + b1.*(x - u).^2 ...
        + b2 .* (x-u).*(x.^2-u.^2) + a2.*u./(u+a1.^2) + a3.*u.^2;
  end
    
  lnppc = lnppc + h;
    
psh2o = Pc.* exp(lnppc)*10;   % pressure in bar 

% ------------------------------------------------------------
function [Pvh2s] = pvaph2s(T)

%------------------------------------------------------------------------
% Definition of constants and thermodynamic parameters
%------------------------------------------------------------------------

Tc = 373.1; % critical temperature of H2S [K]
Pc = 8.9987; % critical pressure of H2S [MPa]
Tt = 187.6991; % triple point temperature of H2S [K]
Pt = 23.2d-3; % triple point pressure of H2S [MPa]
Ptr = Pt/Pc; % reduced pressure at triple point
Ttr = Tt/Tc; % reduced temperature at triple point
Tbr = 0.57049;
Tr = T./Tc; % reduced temperature
t = (T - Tt)./(Tc -Tt);
tb = (Tbr - Ttr)./(1 - Ttr);

a1 = 0.090204;
b0 = -0.154;

phi = (1 - t)./(1 + a1.*t)./(1 + b0.*t.*(t - tb));

lnpr = phi.*Ttr.*log(Ptr)./Tr;
Pvh2s = exp(lnpr)*Pc;

% ------------------------------------------------------------
function [Vinit] = guessfunc(press,temp)

% this function determines a starting value for the molar volume

R = 8.314472; % gas constant in [Pa*m^3/mol/K] 
Pv = pvaph2s(temp + 273.15); % vapor pressure of H2S at T = temp

tguess = [40 100 160 220 280]; % temperature values for lookup table [F]
tguess = (tguess-32).*5./9; % temperature values for lookup table [C]

Vguess = [0.65 0.71 0.8 0.95 1.5];

if (press > Pv)
    if temp <= tguess(1)
        Vinit = Vguess(1);
        Vinit = Vinit.*28.316846592./453.59237*1d2; % molar volume in [m^3/mol/10] (Duan's strange unit)
    else
        f = find(tguess < temp);
        f = f(end);
        fraca = (temp - tguess(f))/(tguess(f+1)-tguess(f));
        fracb = (tguess(f+1) - temp)/(tguess(f+1)-tguess(f));
        Vinit = fraca*Vguess(f) + fracb*Vguess(f+1); % linear interpolation
        Vinit = Vinit.*28.316846592./453.59237*1d2; % molar volume in [m^3/mol/10] (Duan's strange unit)
    end
else
    Vinit = R*(temp + 273.15)/(press*10); % ideal gas law , T in [K], P in [bar]
end



