function m = solu_n2sw(press,temp,sal)

% SOLU_N2SW -  calculates the equilibrium concentration of N2 in seawater
%   
%     Usage:   m = solu_n2sw(p,T,S)
%
%     Input parameters (scalars or vectors): 
%         p        -  pressure [MPa]     (0.1 < P < 60)
%         T        -  temperature [degC] (0 < T < 317)
%         S        -  practical salinity (S < 350)
%     
%     Output parameter: 
%         m        - molality of N2 in seawater [mol/kg]
%
%     References: 
%     Mao & Duan (2006) Fluid Phase Equilibria 248, 103-114.
%     Shibue (2003) Fluid Phase Equilibria 213, 39-51.
%
%     Comments: 
%     The mole fraction of N2 in gas phase is set to 1. 
%     At high temperatures and low pressures, the equation to calculate
%     the H2O mole fraction in the vapor phase as given by Duan results
%     in fractions >1. The formalism to calculate yH2O is still implemented,
%     but has been marked as comment.
%        
% -----------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (January 2013)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% -----------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLU_N2SW:Input','SOLU_N2SW: Missing input parameter! Please provide p,T and S!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>60) && ~any(temp<0 | temp>317) ...
        && ~any( sal<0 | sal>350)
else
    bad = find(press<0.1 | press > 60);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>317);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>350);
    sal(bad)=NaN*ones(size(bad));
    warning('SOLU_N2SW:OutOfParameterRange','SOLU_N2SW: Input values out of range!')
    warning('SOLU_N2SW:OutOfParameterRange','Equations valid for p = 0.1-60 MPa, T= 0-317 degC, S = 0-350.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic parameters
%------------------------------------------------------------------------

[P, T, S] = ndgrid(press*10,temp+273.15,sal); % pressure in [bar] temperature in [K]

R = 83.14472; % gas constant in bar*cm^3/mol/K 
%mmh2o    = 18.0152; % molar mass of H2O [g/mol]

C1m = -0.23093813d2; 
C2m =0.5604852d-1; 
C3m =0.98808898d4; 
C4m =-0.51091621d-4;
C5m =-0.13220298d7;
C6m =-0.49542866d-3; 
C7m =0.12698747d-5;
C8m  =0.51411144;
C9m  =-0.64733978d-4; 

C1l = -0.24434074d1; 
C2l = 0.36351795d-2;
C3l = 0.44747364d3;
C6l = -0.13711527d-4;
C9l = 0.71037217d-5;

C1x = -0.58071053d-2;

sigma = 3.63; % [A]

% vh2o = 1./(rsw.*1d-3./mmh2o); % molar volume of H2O [cm^3/mol]
%mh2o = 1/mmh2o * 1000; % [mol/g * g/m^3] = [mol/kg]

% call external function to calculate the composition of seawater
[mNa,mMg,mCa,mK,~,~,mCl,~,~,mSO4] = seasalt(S); % m [mol/kg]

% mNaCl = i_sw(S); % use ionic strength as a surrogate for NaCl concentration
% mfnacl = (mNaCl )./(mh2o +  mNaCl);

%------------------------------------------------------------------------
% Calculation of water saturation pressure of from a multiparametric equation.
%------------------------------------------------------------------------

%[psh2o] = satpress(mfnacl,T);

%------------------------------------------------------------------------
% Calculation of H2O fugacity coefficient phi
%------------------------------------------------------------------------
%phih2o = fugcoeff(P,T);

%------------------------------------------------------------------------
% Calculation of the mole fraction of N2 in the gas phase
%------------------------------------------------------------------------
%mole fraction of H2O in the gas phase
%yh2o = mfh2o.*psh2o./phih2o./P.*exp(vh2o.*(P - psh2o)./R./T);
yn2 = 1;%-yh2o;

%------------------------------------------------------------------------
% Calculation of mu0(N2)/R/T and Pitzer interaction coefficients lambda and
% xi
%------------------------------------------------------------------------

% calculation of mu0(N2)/R/T
muRT = C1m + C2m.*T + C3m./T + C4m.*T.^2 + C5m./T.^2 + C6m.*P + C7m.*P.*T ...
    + C8m.*P./T + C9m.*P.^2./T;

% calculation of lambda
lambda = C1l + C2l.*T + C3l./T + C6l.*P + C9l.*P.^2./T;
lambdaso4 = 0.0371;

% calculation of xi
xi = C1x;

%------------------------------------------------------------------------
% solve equation of state for "reduced volume" vm of N2
%------------------------------------------------------------------------

Vm = zeros(size(P)); % empty matrix for Vm
options=optimset('Display','off','TolFun',1e-16); % set options for fitfunction

for i=1:size(P,1)
    for j=1:size(P,2)
        if isnan(press(i)) || isnan(temp(j))
            Vm(i,j,:) = NaN;
        else  
            VM_init = R*T(i,j,1)/P(i,j,1); % ideal gas law for initial guess of volume
            VM_init = VM_init./1000./(sigma/3.691).^3; % initial guess for "reduced volume"
            % calculate reduced volume by numerical iteration
            Vm(i,j,:)= fsolve(@(VM) minfunc(VM,P(i,j,1),T(i,j,1)), VM_init,options);
        end
    end
end

 [~, phin2] = eqofstate_n2(T,Vm); % fugacity coefficient of gaseous N2

%------------------------------------------------------------------------
% calculate solubility m
%------------------------------------------------------------------------

m = exp(log(yn2.*phin2.*P) - muRT - 2.*lambda.*(mNa + mK + 2*mCa + 2*mMg)...
    - xi.*(mNa + mK + 2*mCa + 2*mMg).*(mCl + 2*mSO4) - 4*lambdaso4.*mSO4);

end
%------------------------------------------------------------------------
%------------------------------------------------------------------------

%------------------------------------------------------------------------
function [Z, phi] = eqofstate_n2(T,Vm)
%calculates the gas phase fugacity of N2

%------------------------------------------------------------------------
% Definition of constants and thermodynamic parameters
%------------------------------------------------------------------------

epsilon = 101; % [K]
Tm = 154.*T./epsilon;

a1 =  3.75504388d-02;
a2 = -1.08730273d4;
a3 =  1.10964861d6;
a4 =  5.41589372d-4;
a5 =  1.12094559d2;
a6 = -5.92191393d3;
a7 =  4.37200027d-6;
a8 =  4.95790731d-1;
a9 = -1.64902948d2;
a10 =-7.07442825d-8;
a11 = 9.65727297d-3;
a12 = 4.87945175d-1;
a13 = 1.62257402d4;
a14 = 8.99d-3;

T1 = (a1 + a2./Tm.^2 + a3./Tm.^3)./Vm ;
T2 = (a4 + a5./Tm.^2 + a6./Tm.^3)./Vm.^2;
T3 = (a7 + a8./Tm.^2 + a9./Tm.^3)./Vm.^4;
T4 = (a10 + a11./Tm.^2 + a12./Tm.^3)./Vm.^5;

%------------------------------------------------------------------------
% Calculation of Z and fugacity coefficient phi
%------------------------------------------------------------------------

Z = 1 + T1 + T2 + T3 + T4 + ...
    a13./Tm.^3./Vm.^2.*(1 + a14./Vm.^2).*exp(-a14./Vm.^2);

phi = exp(Z - 1 - log(Z) + T1 + T2./2 + T3./4 + T4./5 +...
     a13./2./Tm.^3./a14.*(2 - (2 + a14./Vm.^2).*exp(-a14./Vm.^2)));
 
 %------------------------------------------------------------------------
function f = minfunc(Vm,P,T)
%minimization function for calculation of Vm

R = 0.08314467; % [bar dm^3/mol/K]
sigma = 3.63; % [A]
epsilon = 101; % [K]
Pm = 3.0626.*sigma.^3.*P./epsilon;
Tm = 154.*T./epsilon;

% call external function to calculate Z
Z = eqofstate_n2(T,Vm);
f = Z - Pm.*Vm./R./Tm;

% ------------------------------------------------------------

% function [psh2o] = satpress(mfnacl,T)
% 
% % calculation of saturation pressure of water [bar]
% 
% Pc = 22.064; % in MPa
% Tc = 647.096;
% Tr = T./Tc;
% a1 = 0.128746;
% a2 = -0.731097;
% a3 = -315.058;
% b1 = 392.767;
% b2 = -2464.4;
% u = 0.024;
% c1 = -7.85951783;
% c2 = 1.84408259;
% c3 = -11.7866497;
% c4 = 22.6807411;
% c5 = -15.9618719;
% c6 = 1.80122502;
% x = mfnacl;
% 
% 
% lnppc = 1./Tr.*( c1.*(1-Tr) + c2.*(1-Tr).^(1.5) + c3.*(1-Tr).^3) ...
%         + 1./Tr.*(c4.*(1-Tr).^(3.5) + c5.*(1-Tr).^4 + c6.*(1-Tr).^(7.5));
%     
%   if (x < u)
%       h = a2.*x./(x+a1.^2)+a3.*x.^2;
%   else
%       h = ((a1.^2.*a2)./(u + a1.^2).^2 + 2.*a3.*u).*(x - u) + b1.*(x - u).^2 ...
%         + b2 .* (x-u).*(x.^2-u.^2) + a2.*u./(u+a1.^2) + a3.*u.^2;
%   end
%     
%   lnppc = lnppc + h;
%     
% psh2o = Pc.* exp(lnppc)*10;   % pressure in bar 

% ------------------------------------------------------------
% 
% function phih2o = fugcoeff(P,T)
% 
% % calculation of the fugacity coefficient of water
% 
% 
% % a1 = 1.86357885d-3;
% % a2 = 1.17332094d-2;
% % a3 = 7.82682497d-7;
% % a4 =-1.15662779d-5;
% % a5 =-3.13619739;
% % a6 =-1.29464029d-3;
% a1 = -0.0142006707;
% a2 =  0.0108369910;
% a3 = -0.00000159213160;
% a4 = -0.0000110804676;
% a5 = -3.14287155;
% a6 = 0.00106338095;
% 
% phih2o = exp(a1 + a2*P + a3*P.^2 + a4.*P.*T + a5.*P./T + a6.*P.^2./T);


%------------------------------------------------------------------------
