function m = solu_o2sw(press,temp,sal)

% SOLU_O2SW -  calculates the equilibrium concentration of O2 in seawater
%   
%     Usage:   m = solu_o2sw(p,T,S)
%
%     Input parameters (scalars or vectors): 
%           p        -  pressure [MPa]     (0.1 < P < 20)
%           T        -  temperature [degC] (0 < T < 127)
%           S        -  practical salinity (S < 350)
%     
%     Output parameter: 
%           m        - molality of O2 in seawater [mol/kg]
%
%     References: 
%     Geng & Duan (2010) GCA 74, 5631-5640.
%     Shibue (2003) Fluid Phase Equilibria 213, 39-51.
%        
% -----------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (January 2013)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% -----------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLU_O2SW:Input','SOLU_O2SW: Missing input parameter! Please provide p,T and S!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>20) && ~any(temp<0 | temp>327) ...
        && ~any( sal<0 | sal>350)
else
    bad = find(press<0.1 | press > 20);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<0 | temp>327);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<0 | sal>350);
    sal(bad)=NaN*ones(size(bad));
    warning('SOLU_O2SW:OutOfParameterRange','SOLU_O2SW: Input values out of range!')
    warning('SOLU_O2SW:OutOfParameterRange','Equations valid for p = 0.1-20 MPa, T= 0-327 degC, S = 0-350.')
end

%------------------------------------------------------------------------
% Definition of constants and thermodynamic parameters
%------------------------------------------------------------------------

[P, T, S] = ndgrid(press*10,temp+273.15,sal); % pressure in [bar] temperature in [K]

%R = 8.314472; % gas constant in Pa*m^3/mol/K 
mmh2o    = 18.0152; % molar mass of H2O [g/mol]
mh2o = 1/mmh2o * 1000; % [mol/g * g/m^3] = [mol/kg]

C1m = 3.2832d1; 
C2m =-4.2731d-2; 
C3m =-4.4074d3; 
C4m = 1.5253d-5;
C5m = 1.0279d-3;
C6m =-2.9945d-3; 
C7m = 2.6179d-3;
C8m =-5.6660d-5;

lambdaNa = 1.9997d-1;
lambdaK  = 1.5022d-1;
xiNaCl = -1.2793d-2;
lambdaMg = 3.1715d-1;
lambdaCa = 3.5135d-1;
lambdaSO4 = 1.4383d-1;

% call external function to calculate the composition of seawater
[mNa,mMg,mCa,mK,~,~,mCl,~,~,mSO4] = seasalt(S); % m [mol/kg]

mNaCl = i_sw(S); % use ionic strength as a surrogate for NaCl concentration
mfnacl = (mNaCl )./(mh2o +  mNaCl);

%------------------------------------------------------------------------
%Calculation of water saturation pressure of from a multiparametric equation.
%------------------------------------------------------------------------
[psh2o] = satpress(mfnacl,T);

%------------------------------------------------------------------------
% Calculation of the mole fraction of O2 in the gas phase
%------------------------------------------------------------------------
yO2 = (P - psh2o)./P;

%------------------------------------------------------------------------
% Calculation of mu0(O2)/R/T 
%------------------------------------------------------------------------

% calculation of mu0(O2)/R/T
muRT = C1m + C2m.*T + C3m./T + C4m.*T.^2  + C5m.*P + C6m.*log(T) +...
       C7m.*P./(630-T)  + C8m.*T.*log(P);


%------------------------------------------------------------------------
% call external function to calculate fugacity coefficient of O2
%------------------------------------------------------------------------
% create empts matrix
phiO2 = zeros(size(P));

 [~, ~, ~, ~, ~ , ~, phi0] = eos_o2(press,temp); % fugacity coefficient of gaseous O2

% enlarge array
 for i = 1:length(sal)
     phiO2(:,:,i)=phi0;
 end
 
%------------------------------------------------------------------------
% calculate solubility m
%------------------------------------------------------------------------

m = exp(log(yO2.*phiO2.*P) - muRT - 2.*lambdaNa.*mNa - 2.*lambdaK.*mK ...
    - 4.*lambdaCa.*mCa - 4.*lambdaMg.*mMg...
    - xiNaCl.*(mNa + mK + 2*mCa + 2*mMg).*(mCl + 2*mSO4) ...
    - 4.*lambdaSO4.*mSO4);

end
%------------------------------------------------------------------------
%------------------------------------------------------------------------

function [psh2o] = satpress(mfnacl,T)

% calculation of saturation pressure of water [bar]

Pc = 22.064; % in MPa
Tc = 647.096;
Tr = T./Tc;
a1 = 0.128746;
a2 = -0.731097;
a3 = -315.058;
b1 = 392.767;
b2 = -2464.4;
u = 0.024;
c1 = -7.85951783;
c2 = 1.84408259;
c3 = -11.7866497;
c4 = 22.6807411;
c5 = -15.9618719;
c6 = 1.80122502;
x = mfnacl;


lnppc = 1./Tr.*( c1.*(1-Tr) + c2.*(1-Tr).^(1.5) + c3.*(1-Tr).^3) ...
        + 1./Tr.*(c4.*(1-Tr).^(3.5) + c5.*(1-Tr).^4 + c6.*(1-Tr).^(7.5));
    
  if (x < u)
      h = a2.*x./(x+a1.^2)+a3.*x.^2;
  else
      h = ((a1.^2.*a2)./(u + a1.^2).^2 + 2.*a3.*u).*(x - u) + b1.*(x - u).^2 ...
        + b2 .* (x-u).*(x.^2-u.^2) + a2.*u./(u+a1.^2) + a3.*u.^2;
  end
    
  lnppc = lnppc + h;
    
psh2o = Pc.* exp(lnppc)*10;   % pressure in bar 

% ------------------------------------------------------------

