function m = solu_opalFournier(press,temp,sal)

% SOLU_OPALFOURNIER -  calculates the solubility of amorphous silica in seawater
%
%      Usage: m = solu_opalFournier(p,T,S)
%
%      Input parameters (scalars or vectors): 
%          p      - pressure [MPa]       (10 < p < 120)
%          T      - temperature [degC]   (150 < T < 350)
%          S      - practical salinity   (0 < S < 80) 
%   
%      Output parameter:
%          m      - solubility of silica [mol/kg]
%
%      References:
%      Fournier & Marshall (1983) GCA 47, 587-596. (solubility at p=1000 bar)
%      Willey (1974) Mar.Chem. 2, 239-250. (pressure correction) 
%      Fournier (1983) GCA 47, 579-586. (salinity correction of m)
%      Sun et al. (2008) DSR I 55, 1304-1310. (salinity correction density)
%
%      Comments: 
%      The temperature dependency has been evaluated by Fournier & Marshall
%      for data in the range of 150-350 degC at 1000 bar.
%      The pressure dependency has been evaluated by Willey for data in 
%      the range of 100-200 atm at 0 degC.
%      A salinity correction for the solubility as proposed for the 
%      solubility of quarz by Fournier (1983) is implemented, assuming
%      that hydration numbers for quartz and amorphous silica are identical.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       January 2013: name in internal function call changed: eos_H2O -> eos_h2o 
% ------------------------------------------------------------------


%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLU_OPALFOURNIER:Input','SOLU_OPALFOURNIER: Missing input parameter! Please provide p,T and S!')
else


%------------------------------------------------------------------------
% Check data range validity!
%------------------------------------------------------------------------

if ~any(press<10 | press>120) && ~any(temp<(150) | temp>350) && ~any(sal<(0) | sal>80) 
        
else
    bad = find(press<10 | press > 120);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<(150) | temp>350);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<(0) | sal>80);
    sal(bad)=NaN*ones(size(bad));

 warning('SOLU_OPALFOURNIER:OutOfParameterRange','SOLU_OPALFOURNIER: Input values out of range!')
 warning('SOLU_OPALFOURNIER:OutOfParameterRange','Equations valid for p = 10-120 MPa, T=150-350 degC, S = 0-80.')
end

%------------------------------------------------------------------------
% Definition of constants, thermodynamic variables and parameters
%------------------------------------------------------------------------

[P, T, S] = ndgrid(press*10,temp + 273.15, sal*1.005); % pressure in bar, absolute salinity
[PM, TC] = ndgrid(press, temp, sal); % p in MPa, temp in C
[PA] = ndgrid(press.*0.98692*10, temp, sal); % p in atm
R = 82.05756; % Gas constant in atm*cm^3/mol/K
rho = zeros(size(P));

b(1) = -7.99992230e-1;
b(2) =  2.40936500e-3;
b(3) = -2.58052775e-5;
b(4) =  6.85608405e-8;
b(5) =  6.29761106e-4;
b(6) = -9.36263713e-7;

% call external function to calculate density of water
rho_i = eos_h2o(press,temp); 

% resize density matrix
for i = 1: size(rho,3)
    rho(:,:,i) = rho_i;
end

% calculate correction for salinity
B = S.*b(1) + S.*TC.*b(2) + S.*TC.^2.*b(3) + S.*TC.^3.*b(4) + PM.*S.*b(5) ...
    + PM.^2.*S.*b(6);

rho = rho -B;

rho_e = rho./1000 .*(1-S./1000);
% Hydration number
n = 0.001418.*T + 806.97./T + 0.00039465.*T.*log10(rho_e);

% Fournier: reference solubility for p = 1000 bars in [mg/kg]

logmR =  -7.010 + 0.02285.*T - 3.262e-5.*T.^2 + 1.73e-8.*T.^3;
mR = 10.^logmR; 

% pressure correction as proposed by Willey, T [K], P [atm]
PAR = 1000.*0.98692; % reference pressure in atm
lnmmR = 1./T./R.* ...
    (16.5.*(PA-PAR) -2.2e-2.*(PA.^2-PAR.^2) + 1.2e-5.*(PA.^3 - PAR.^3));

mT = exp(lnmmR).*mR;

% salinity correction as proposed by Fournier
logm = log10(mT);

logm = logm + n.*log10(rho_e./rho.*1000);
m = 10.^logm; 
end


