function m = solu_opalWalther(press,temp,sal)

% SOLU_OPALWALTHER -  calculates the solubility of amorphous slica in seawater
%
%      Usage: m = solu_opalWalther(p,T,S)
%
%      Input parameters (scalars or vectors): 
%           p     - pressure [MPa]       (10 < p < 120)
%           T     - temperature [degC]   (0 < T < 350)
%           S     - practical salinity   (0 < S < 80) 
%   
%      Output parameter:
%          m      - solubility of silica [mol/kg]
%
%      References:
%      Walther & Helgeson (1977) AJS 277, 1315-1351.
%      Helgeson & Kirkham (1974) AJS 274, 1089-1198.
%      Fournier & Marshall (1983) GCA 47, 579-586. (salinity correction m)
%      Sun et al. (2008) DSR I 55, 1304-1310. (salinity correction density)
%
%      Comments: 
%      The regression in the paper was done on data from T=0-350 degC and
%      p=100-1200 bar. A salinity correction for the solubility as 
%      proposed for the solubility of quarz by Fournier (1983) is 
%      implemented, assuming that hydration numbers for quartz and 
%      amorphous silica are identical.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       November 2012: adapted to Matlab2012 syntax
%       January 2013: name in internal function call changed: eos_H2O -> eos_h2o 
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLU_OPALWALTHER:Input','SOLU_OPALWALTHER: Missing input parameter! Please provide p,T and S!')
else


%------------------------------------------------------------------------
% Check data range validity!
%------------------------------------------------------------------------

if ~any(press<10 | press>120) && ~any(temp<(0) | temp>350) && ~any(sal<(0) | sal>80) 
        
else
    bad = find(press<10 | press > 120);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<(0) | temp>350);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<(0) | sal>80);
    sal(bad)=NaN*ones(size(bad));

 warning('SOLU_OPALWALTHER:OutOfParameterRange','SOLU_OPALWALTHER: Input values out of range!')
 warning('SOLU_OPALWALTHER:OutOfParameterRange','Equations valid for p = 10-120 MPa, T=0-350 degC, S = 0-80.')
end

%------------------------------------------------------------------------
% Definition of constants, thermodynamic variables and parameters
%------------------------------------------------------------------------

[P, T, S] = ndgrid(press*10,temp + 273.15, sal*1.005); % pressure in bar, absolute salinity
[PM, TC] = ndgrid(press, temp, sal); % p in MPa, temp in C
Tr = 298.15; % reference temperature [K]
rho = zeros(size(P));
rho_r = 9.9705e+002; % density in reference state
R = 1.9872; % Gas constant in cal/mol/K
delta_G_r_m_s = -202892; %[cal/mol]
delta_G_r_m_aq = -199190; %[cal/mol]

b(1) = -7.99992230e-1;
b(2) =  2.40936500e-3;
b(3) = -2.58052775e-5;
b(4) =  6.85608405e-8;
b(5) =  6.29761106e-4;
b(6) = -9.36263713e-7;

% call external function to calculate density of water
rho_i = eos_h2o(press,temp); 

% resize density matrix
for i = 1: size(rho,3)
    rho(:,:,i) = rho_i;
end

% calculate correction for salinity
B = S.*b(1) + S.*TC.*b(2) + S.*TC.^2.*b(3) + S.*TC.^3.*b(4) + PM.*S.*b(5) ...
    + PM.^2.*S.*b(6);

rho = rho -B;

rho_e = rho./1000 .*(1-S./1000);
% Hydration number
n = 0.001418.*T + 806.97./T + 0.00039465.*T.*log10(rho_e);

[alpha] = calc_ab(TC,rho);
[alpha_r] = calc_ab((Tr - 273.15),rho_r);
[~, Z] = calc_e(T, rho, alpha);
[Yr, Zr] = calc_e(Tr, rho_r, alpha_r); % reference state

delta_G0_aq = aqueous(P,T,Yr,Z,Zr);

delta_G0_s = solid(P,T);

% molal Gibbs energy of formation in aqueous state
delta_G_aq = delta_G_r_m_aq + (delta_G0_aq); 
% molal Gibbs energy of formation in solid
delta_G_s = delta_G_r_m_s + (delta_G0_s);

logK = (-delta_G_aq + delta_G_s)./2.303./R./T;
logm = logK + n.*log10(rho_e./rho.*1000);
m = 10.^logm; % solubility in mol/kg

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [alpha] = calc_ab(T, rho)
% calculates alpha (isobaric thermal expansion) and beta (isothermal
% compressibility) of water

rho = rho./1000; % density in [g/cm^3]
Bd = zeros(9,6);
Bs = zeros(9,6);

Bd(2,1) =  2.001619452;
Bd(3,1) = -8.445248783;
Bd(4,1) =  1.849203613;
Bd(5,1) = -2.344156348;
Bd(6,1) =  1.782166279;
Bd(7,1) = -8.063473576;
Bd(8,1) =  2.064415341;
Bd(9,1) = -2.498805152;
Bd(1,2) =  5.541654961;
Bd(2,2) = -6.480330829;
Bd(3,2) =  2.599081364;
Bd(4,2) = -5.126544377;
Bd(5,2) =  5.544129599;
Bd(6,2) = -3.321663469;
Bd(7,2) =  1.021229414;
Bd(8,2) = -1.237172220;
Bd(1,3) = -5.647694994;
Bd(2,3) =  7.804635059;
Bd(3,3) = -2.997059355;
Bd(4,3) =  5.215914752;
Bd(5,3) = -4.597168845;
Bd(6,3) =  1.982307354;
Bd(7,3) = -3.272068272;
Bd(1,4) =  1.391084523;
Bd(2,4) = -4.149778955;
Bd(3,4) =  1.567146158;
Bd(4,4) = -2.327295430;
Bd(5,4) =  1.520379055;
Bd(6,4) = -3.619181443;
Bd(1,5) =  9.314051406;
Bd(2,5) =  8.284235229;
Bd(3,5) = -3.437390375;
Bd(4,5) =  4.123082012;
Bd(5,5) = -1.562138539;
Bd(1,6) = -4.224085781;
Bd(2,6) = -3.173272705;
Bd(3,6) =  1.864672542;
Bd(4,6) = -1.650956925;

Bs(2,1) = 4;
Bs(3,1) = 1;
Bs(4,1) = -1;
Bs(5,1) = -4;
Bs(6,1) = -7;
Bs(7,1) = -11;
Bs(8,1) = -14;
Bs(9,1) = -18;
Bs(1,2) = 6;
Bs(2,2) = 4;
Bs(3,2) = 2;
Bs(4,2) = -1;
Bs(5,2) = -4;
Bs(6,2) = -7;
Bs(7,2) = -10;
Bs(8,2) = -14;
Bs(1,3) = 6;
Bs(2,3) = 4;
Bs(3,3) = 2;
Bs(4,3) = -1;
Bs(5,3) = -4;
Bs(6,3) = -7;
Bs(7,3) = -11;
Bs(1,4) = 6;
Bs(2,4) = 4;
Bs(3,4) = 2;
Bs(4,4) = -1;
Bs(5,4) = -4;
Bs(6,4) = -8;
Bs(1,5) = 5;
Bs(2,5) = 3;
Bs(3,5) = 1;
Bs(4,5) = -2;
Bs(5,5) = -5;
Bs(1,6) = 5;
Bs(2,6) = 1;
Bs(3,6) = 0;
Bs(4,6) = -3;

B = Bd.*10.^Bs;

b1 = zeros(size(T));
alpha = zeros(size(T));

for j = 0:5
    for i = 0:(8-j)
        b1 = b1 + j.*B(i+1,j+1).*T.^i .*rho.^j;
    end
end

beta = 1./b1;

for j = 0:5
    for i = 0:(8-j)
        alpha = alpha + i.*B(i+1,j+1).*T.^(i-1) .*rho.^j;
    end
end

alpha = alpha.*beta;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [Y, Z] = calc_e(T,rho, alpha)
% calculates Y and Z from the dielectric constant of water

rho = rho./1000; % density in [g/cm^3]

ed = zeros(5,5);
es = zeros(5,5);

ed(1,1) =  4.39109592;
ed(2,1) = -2.18995148;
ed(3,1) =  1.82898246;
ed(4,1) =  1.54886800;
ed(5,1) = -6.13542375;
ed(1,2) = -2.33277456;
ed(2,2) =  1.00498361;
ed(3,2) = -2.08896146;
ed(4,2) = -6.13941874;
ed(1,3) =  4.61662109;
ed(2,3) = -1.35650709;
ed(3,3) =  1.60491325;
ed(1,4) = -4.03643333;
ed(2,4) =  5.94046919;
ed(1,5) =  1.31604037;

es(1,1) = 2;
es(2,1) = 2;
es(3,1) = 1;
es(4,1) = 2;
es(5,1) = 1;
es(1,2) = 0;
es(2,2) = 0;
es(3,2) = -1;
es(4,2) = -2;
es(1,3) = -3;
es(2,3) = -3;
es(3,3) = -4;
es(1,4) = -6;
es(2,4) = -7;
es(1,5) = -9;

e = ed.*10.^es;

epsilon = zeros(size(T)); % dielectric constant of water
dedt = zeros(size(T)); % d(epsilon)/d(T)

for i = 0:4
    for j = 0:(4-i)
        epsilon = epsilon + e(j+1, i+1).*T.^i .* rho.^j;
    end
end

for i = 0:4
    for j = 0:(4-i)
        dedt = dedt + e(j+1, i+1).*(i.*T.^(i-1) - j.*alpha.*T.^i ).* rho.^j;
    end
end

dlnedt = dedt./epsilon; % d(ln(epsilon))/d(T)

Y = dlnedt./epsilon;
Z = - 1./epsilon;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function delta_G0_aq = aqueous(P,T,Yr,Z,Zr)
% calculates difference of Gibbs energy from reference state for aqueous
% state

a1 = 2.1394e-2 ; %[cal/mol/bar]
a2 = 4.9341e-5; %[cal/mol/bar.^2]
a3 = 6.8259e-2; %[cal/mol/bar.^2]
a4 = 2.9786e-6; %[cal/mol/bar.^2]
theta = 233; % [K]
omega = 1.2908e4; % [cal/mol]
c1 = 48.517; % [cal/mol/bar.^2]
c2 = -15.789; % [cal /mol/K]
S0 = 14.8; % [cal mol/K]
Tr = 298.15; % reference temperature [K]
Pr = 1; % reference pressure [bar]

delta_G0_aq = -S0 .*(T - Tr) - c1.*(T.*log(T./Tr) - T + Tr) + ...
    c2.*((T -Tr -(T-theta).*log((T - theta)./(Tr - theta)))) + ...
    (2.*(a1.*(T - theta) + a3.*T).*(P - Pr) + (a2.*(T - theta) + a4.*T).* ...
    (P.^2 - Pr.^2))./(2.*(T - theta)) - ...
    omega.*(Z - Zr - Yr.*(T - Tr));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function delta_G0_s = solid(P,T)
% calculates difference of Gibbs energy from reference state for solid
% state - a division by 41.87 in the last term was inserted to correct for
% inconsistent units 

S0 = 14.34; % [cal/mol/K]
V0 = 29; % [cm^3/mol]
a = 5.93;
b = 47.2e-3;
c = -22.78e5;
Tr = 298.15; % reference temperature [K]
Pr = 1; % reference pressure [bar]

delta_G0_s = -S0.*(T - Tr) + a.*(T - Tr - T.*log(T/Tr)) - ...
    (b.*T.*Tr.^2 + c).*(T - Tr).^2./(2.*T.*Tr.^2) + V0.*(P - Pr)./41.87;

