function m = solu_quartzFournier(press,temp,sal)

% SOLU_QUARTZFOURNIER -  calculates the solubility of quartz in seawater
%
%      Usage: m = solu_quartzFournier(p,T,S)
%
%      Input parameters (scalars or vectors): 
%          p      - pressure [MPa]       (0.1 < p < 100)
%          T      - temperature [degC]   (0 < T < 350)
%          S      - practical salinity   (0 < S < 80) 
%   
%      Output parameter:
%          m      - solubility of quartz [mol/kg]
%
%      References:
%      Fournier (1983) GCA 47, 579-586. (solubility)
%      Sun et al. (2008) DSR I 55, 1304-1310. (salinity correction density)
%
%      Comments: 
%      The solubility is derived from the general formula
%       log(m)= log(K) + n log(rho_e) - log(gamma) + log (Gamma)
%      with the last two terms set to zero. 
%      rho_e is the density of free water, defined as 
%       rho_e = rho F (1 - (h m(NaCl))/55.51).
%      The hydration number h of NaCl in solution is unknown, therefore 
%      Rho_e is approximated by 
%       rho_e = rho F
%      with F being the weight fraction of water. 
%      Log(K) and n are calculated from multi-parameter equations.
%      The equation is valid for T = 350 degC. There is only a weak pressure
%      dependency (in the density) and a weak dependency on salt
%      concentration (in density and F).
%      The salinity correction for the density is valid in the liquid
%      phase only for 0.1 < p < 100, 0 < T < 374, 0 < S < 40 (authors
%      state, that extrapolation up to S=80 is ok). The calculated density
%      values for T < 350 degC compare well with the data from Fontaine et
%      al. (2007) EPSL 275, 132-145. Data deviate for higher temperatures. 
%      Calculation of rho_e requires the hydration number of NaCl ions.
%      Fournier sets the hydration number to zero and states that this is
%      a good approximation for temperatures > 200 degC. Therefore,
%      equations may be less accurate for lower temperatures.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       January 2013: name in internal function call changed: eos_H2O -> eos_h2o 
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('SOLU_QUARTZFOURNIER:Input','SOLU_QUARTZFOURNIER: Missing input parameter! Please provide p,T and S!')
else

      

%------------------------------------------------------------------------
% Check data range validity!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>100) && ~any(temp<(0) | temp>350) && ~any(sal<(0) | sal>80) 
        
else
    bad = find(press<0.1 | press > 100);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<(0) | temp>350);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<(0) | sal>80);
    sal(bad)=NaN*ones(size(bad));

 warning('SOLU_QUARTZFOURNIER:OutOfParameterRange','SOLU_QUARTZFOURNIER: Input values out of range!')
    warning('SOLU_QUARTZFOURNIER:OutOfParameterRange','Equations valid for p = 0.1-100 MPa, T=0-350 degC, S = 0-80.')
end

%------------------------------------------------------------------------
% Definition of constants, thermodynamic variables and parameters
%------------------------------------------------------------------------

[P, T, S] = ndgrid(press*10,temp + 273.15, sal*1.005); % p in bar, T in [K], absolute salinity
[PM, TC] = ndgrid(press, temp, sal); % p in MPa, T in C
rho = zeros(size(P));

b(1) = -7.99992230e-1;
b(2) =  2.40936500e-3;
b(3) = -2.58052775e-5;
b(4) =  6.85608405e-8;
b(5) =  6.29761106e-4;
b(6) = -9.36263713e-7;

% call external function to calculate density of water, approximate density
% of solution by density of water
rho_i = eos_h2o(press,temp); 

% resize density matrix
for i = 1: size(rho,3)
    rho(:,:,i) = rho_i;
end

% calculate correction for salinity
B = S.*b(1) + S.*TC.*b(2) + S.*TC.^2.*b(3) + S.*TC.^3.*b(4) + PM.*S.*b(5) ...
    + PM.^2.*S.*b(6);

rho = rho -B;

rho_e = rho./1000 .*(1-S./1000);
n = 0.001418.*T + 806.97./T + 0.00039465.*T.*log10(rho_e);

logK = -4.66206 + 0.0034063.*T + 2179.7./T - 1.1292e6 ./T.^2 + 1.3543e8 ./T.^3;

%------------------------------------------------------------------------
% Calculation of solubility
%------------------------------------------------------------------------

m = 10.^(logK + n.*log10(rho_e));

end


