function tort2 = tortuos(poros,method,aa)

% TORTUOS -  calculates the tortuosity correction (to)^2 for diffusion 
%       coefficients in sediments
%           Ds = D0 / (to)^2
%       where   D0 = molecular diffusion coefficient
%               to = tortuosity
%
%       Usage: tort2 = tortuos(poros,'method',aa)
%
%       Input parameters:
%           poros   - porosity (scalar or vector)
%           aa      - fitting parameter for equations (scalar)
%                     recommended values are:
%                      'wb': aa = 2.02
%                      'ar': aa = 2.14
%                      'bf': aa = 3.79
%          'method' - determines the empirical relationship
%                     between porosity and tortuosity to be used.
%                     'bb'    Boudreau:           to^2 = 1-2*log(poros)
%                     'wb'    modified Weissberg: to^2 = 1-a*log(poros)
%                     'ar'    Archie:             to^2 = poros^(1-a)
%                     'bf'    Buerger-Friecke:    to^2 = poros+a*(1-poros)
%
%       Output parameter:
%           tort2   - squared tortuosity
%
%       Default values:
%           Porosity has to be provided at function call. 
%           Default for aa is 0 and default for method is 'bb'.
%
%   References: 
%   Boudreau (1997) Diagenetic Models and Their Implementation, Springer Verlag.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               IFM-GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 1
     warning('TORTOUS:Input','TORTUOS: Missing input parameter! Please provide porosity!')
else


if nargin < 3
    aa = 0;
    method = '';
end

switch method
    case 'bb'
        tort2 = 1 - 2.*log(poros);
    case 'wb'
%        aa = 2.02; % best fit to data
        tort2 = 1 - aa.*log(poros);
    case 'ar'
%        aa = 2.14; % best fit to data
        tort2 = poros .^ (1-aa);
    case 'bf'
%        aa = 3.79; % best fit to data
        tort2 = poros + aa.*(1-poros);
    otherwise
        tort2 = 1 - 2.*log(poros); % default is Boudreau        
end
end
