function eta = visco_sw(press,temp,sal,choice)

% VISCO_SW - calculates the dynamic viscosity of seawater
%      The user can choose from 4 formalisms for different pTS ranges.
%      If no choice is given, the formalism is chosen based on pTS input.
%
%      Usage: eta = visco_sw(p,T,S,c) 
%
%      Input parameters (scalars or vectors): 
%          p     -  pressure [MPa]     (0 < p < 100) 
%          T     -  temperature [degC] (0 < T < 30) 
%          S     -  practical salinity (0 < S < 36.1) 
%          c     -  user choice for algorithm
%                   'KUKULKA' (default)
%                   'SPIVEY'
%                   'MAO'
%                   'PALLISER'
%
%      Output parameter: 
%          eta  -   dynamic viscosity [Pa*s]
%
%      References:
%      Kukulka et al. (1987) Adv. Heat Transfer 18, 325-363.
%      Spivey & McCain (2004) J.Can.Petrol.Tech. 43, 52-61.
%      Mao & Duan (2009) Int.J.Thermophys. 30, 1510-1523.
%      Palliser & McKibbin (1998) Transport in Porous Media 33, 155-171.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
%   Modification History: Parameter range changed from 0<S<36 to <S<36.1,
%                         equations for high temperatures added
%          December 2011: internal function call updated
%                         from visco_highpts to visco_swSpivey
%          November 2012: input parameter 'choice' added,
%                         visco_swMao + visco_swPalliser added
%------------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('VISCO_SW:Input','VISCO_SW: Missing input parameter! Please provide p,T and S!')
else

%------------------------------------------------------------------------
% Choose algorithm based on parameter range
%------------------------------------------------------------------------
if nargin == 3
    if ~any(press<0 | press>100) && ~any(temp< 0 | temp>30) ...
        && ~any(sal < 0 | sal>36.1)

        choice = 'KUKULKA'; % p=0-100 MPa, T=0-30 degC, S=0-36.1

    elseif ~any(press<0.1 | press>100) && ~any( temp<0 | temp>356) ...
        && ~any( sal<0 | sal>300)

        choice = 'MAO'; % p=0.1-100 MPa, T=0-356 degC, S=0-300

    elseif ~any(press<0 | press>200) && ~any( temp<0 | temp>275) ...
        && ~any( sal<0 | sal>290)
    
        choice = 'SPIVEY'; % p=0-200 MPa, T=0-275 degC, S=0-290 M
        
     elseif ~any(press<0.1) && ~any( temp<0.1 | temp>727) 
    
        choice = 'Palliser'; % p=0.1-??? MPa, T=0.1-727 degC, S = 0 ~ saturation

    else
        warning('VISCO_SW:OutOfParameterRange','VISCO_SW: Input values out of range!')
        warning('VISCO_SW:OutOfParameterRange','VISCO_KUKULKA valid for p=0-100 MPa, T= 0-30 degC, S=0-36.1.')
        warning('VISCO_SW:OutOfParameterRange','Extended Equations valid for p>0.1 MPa, T=0.1-727 degC, S=0-saturation.')
    end
end

%------------------------------------------------------------------------
% Algorithm choices
%------------------------------------------------------------------------
switch choice
    case 'KUKULKA'
        eta = visco_kukulka(press,temp,sal);
    case 'MAO' 
        eta = visco_swMao(press,temp,sal);
    case 'SPIVEY'
        eta = visco_swSpivey(press,temp,sal);
    case 'PALLISER'
        eta = visco_swPalliser(press,temp,sal);
    otherwise % use Kukulka's formalism as default
        eta = visco_kukulka(press,temp,sal);
end

end


function eta = visco_kukulka(press,temp,sal)
%------------------------------------------------------------------------
% Calculation of viscosity from a multiparametric equation
%       References:
%       Kukulka et al: Adv. Heat Transfer 18 (1987), pp. 325-363.
%       Boudreau: Diagenetic Models and Their Implementation (1997) Springer Verlag.
%------------------------------------------------------------------------

%------------------------------------------------------------------------
% Definition of constants and thermodynamic variables
%------------------------------------------------------------------------

[P,T,S] = ndgrid(press*10,temp,sal); % pressure in bar

%------------------------------------------------------------------------
% Calculation of viscosity from a multiparametric equation
%------------------------------------------------------------------------

eta = (1.7910 - 6.144E-2*T +1.4510E-3*T.^2 - 1.6826E-5*T.^3 ...
    - 1.5290E-4*P + 8.3885E-8*P.^2 + 2.4727E-3*S ...
    + T.*(6.0574E-6*P - 2.6760E-9*P.^2) ...
    + S.*(4.8429E-5*T - 4.7172E-6*T.^2 + 7.5986E-8*T.^3)); %[mPa s]
eta = eta./1000; % [Pa s]


