function eta = visco_swMao(press,temp,sal)

% VISCO_SWMAO -  calculates the dynamic viscosity of seawater at elevated
%       (pressure), temperature, and salinity based on a parametric equation
%
%       Usage: eta = visco_swMao(p,T,S)
%
%       Input parameters (scalars or vectors): 
%           p       - pressure [MPa]     (0.1 < p < 100)
%           T       - temperature [degC] (0 < T < 350)
%           S       - practical salinity (0 < S < 300) 
%
%       Output parameter:
%          eta      - dynamic viscosity [Pa*s]
%
%       References:
%       Mao & Duan (2009) Int.J.Thermophys. 30, 1510-1523.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (March 2011)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       January 2013: name in internal function call changed: eos_H2O -> eos_h2o 
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('VISCO_SWMAO:Input','VISCO_SWMAO: Missing input parameter! Please provide p,T and S!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>100) && ~any(temp<(0) | temp>350) && ~any(sal<(0) | sal>300)

else
    bad = find(press<0.1 | press > 100);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<(0) | temp>350);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<(0) | sal>300);
    sal(bad)=NaN*ones(size(bad));

 warning('VISCO_SWMAO:OutOfParameterRange','VISCO_SWMAO: Input values out of range!')
 warning('VISCO_SWMAO:OutOfParameterRange','Equations valid for p=0.1-100 MPa, T=0-350 degC, S=0-300 .')
end

%------------------------------------------------------------------------
% Definition of constants, thermodynamic variables and parameters
%------------------------------------------------------------------------

 % calculate NaCl concentration with ionic strength
 % equivalent to given salinity
 mNaCl=i_sw(sal); 

[p, T, M] = ndgrid(press,(temp+273.15),mNaCl); % temperature in Kelvin

%------------------------------------------------------------------------
% Calculation of viscosity of pure water: eta_p
%------------------------------------------------------------------------

% parameters for the calculation of pure water viscosity
d = zeros(10,1);
rho = zeros(size(p)); % empty matrix for pure water viscosity


d(1)=  0.28853170d7;
d(2)= -0.11072577d5;
d(3)= -0.90834095d1;
d(4)=  0.30925651d-1;
d(5)= -0.27407100d-4;
d(6)= -0.19283851d7;
d(7)=  0.56216046d4;
d(8)=  0.13827250d2;
d(9)= -0.47609523d-1;
d(10)= 0.35545041d-4;

% call external function to calculate density of pure water
rho_s = eos_h2o(press,temp)/1000;

for i = 1:length(mNaCl)             % enlarge matrix to fit input matrix size
     rho(:,:,i) = rho_s(:,:);  
end

% ln(eta) for pure water
lneta_p = d(1).*T.^(-2) + d(2).*T.^(-1)  +d(3)  +d(4).*T +d(5).*T.^2 ...
           + d(6).*rho.*T.^(-2) + d(7).*rho.*T.^(-1)+ d(8).*rho...
           + d(9).*rho.*T+ d(10).*rho.*T.^(2);

eta_p = exp(lneta_p);

%------------------------------------------------------------------------
% Calculation of relative viscosity: eta_r
%------------------------------------------------------------------------
% parameters for the calculation of  NaCl solution viscosity

a = zeros(3,1);
b = zeros(3,1);
c = zeros(2,1);

a(1) = -0.21319213; 
a(2) =  0.13651589d-2; 
a(3) = -0.12191756d-5; 
b(1) =  0.69161945d-1; 
b(2) = -0.27292263d-3; 
b(3) =  0.20852448d-6; 
c(1) = -0.25988855d-2; 
c(2) =  0.77989227d-5; 

A = a(1) + a(2).*T + a(3).*T.^2;
B = b(1) + b(2).*T + b(3).*T.^2;
C = c(1) + c(2).*T;

% calculation of ln(eta_r)
lneta_r= A.*M + B.*M.^2 + C.*M.^3;

% calculation of relative viscosity 
eta_r = exp(lneta_r);

%------------------------------------------------------------------------
% Calculation of viscosity of NaCl solution: eta
%------------------------------------------------------------------------

eta = eta_r.*eta_p;

end


