function eta = visco_swPalliser(press,temp,sal)

% VISCO_SWPALLISER -  calculates the dynamic viscosity of NaCl as function
%       of pressure temperature and salinity based on a parametric equation.
%       The equations were fit to few available experimental data and 
%       include several assumptions and extrapolations. The accuracy might
%       be limited, but no other paper on viscosity of brine in the 
%       supercritical region could be found.
%
%       Usage: eta = visco_swPalliser(p,T,S)
%
%       Input parameters (scalars or vectors): 
%           p        - pressure [MPa]        (0.1 < p < ?)
%           T        - temperature [degC]    (0.1 < T < 727)
%           S        - practical salinity    (0   < S < 1000(?))
%
%       Output parameter:
%           eta      - dynamic viscosity [Pa*s]
%
%       References:
%       Palliser & McKibbin (1998) Transport in Porous Media 33, 155-171.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (March 2011)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

% ------------------------------------------------------------------
%   Modification history: 
%       January 2013: name in internal function call changed: visco_H2O -> visco_h2o 
%                     calculation of mole fraction corrected  
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('VISCO_SWPALLISER:Input','VISCO_SWPALLISER: Missing input parameter! Please provide p, T and S!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(sal<0 | sal>1000) && ~any(temp<(0.1) | temp>727) 

else
    bad = find(sal<0 | sal > 1000);
    sal(bad)=NaN*ones(size(bad));
    bad = find(temp<(0.1) | temp>727);
    temp(bad)=NaN*ones(size(bad));

 warning('VISCO_SWPALLISER:OutOfParameterRange','VISCO_SWPALLISER: Input values out of range!')
 warning('VISCO_SWPALLISER:OutOfParameterRange','Equations valid for T=0.1-727 degC, S = 0-1000.')
end

%------------------------------------------------------------------------
% Definition of constants, thermodynamic variables and parameters
%------------------------------------------------------------------------

% call external function to calculate ionic strength of seawater with given
% salinity
isw = i_sw(sal);

% moles of NaCl in 1 kg water, that have the same ionic strength as
% seawater with given salinity 
nNaCl = isw;  

xNaCl = nNaCl./(nNaCl + 55.5); % mole fraction of NaCl

[p, T, X] = ndgrid(press,temp, xNaCl); 


eta_w = zeros(size(p)); %empts matrix for pure water viscosity
    
%------------------------------------------------------------------------
% Calculation of the viscosity of pure water eta_w
%------------------------------------------------------------------------

% call external function to calculate pure water viscosity
% The paper cites the book of Schmidt (1979) as reference for the pure
% water viscosity. Here, a function based on the IAPW2008 formulation is called

eta_H2O = visco_h2o(press,temp);

for i=1:length(xNaCl)
    eta_w(:,:,i) = eta_H2O(:,:);
end

%------------------------------------------------------------------------
% Calculation of the viscosity of liquid NaCl eta_s
%------------------------------------------------------------------------

eta_s = 4.71624e-3 - 4.02030e-6.*T;

eta_s800 = 4.71624e-3 - 4.02030e-6.*800; % eta_s at 800 C

%------------------------------------------------------------------------
% Calculation of the viscosity of the NaCl solution
%------------------------------------------------------------------------

% viscosity below 800 C
eta_1 = (eta_w.*(1+3.*X).*((800-T)./800).^9 ...
    + (T./800).^9.*(eta_w.*(1-X) + eta_s800.*X))./ ...
    (((800-T)./800).^9 + (T./800).^9);

% viscosity above 800 C
eta_2 = eta_w.*(1-X) + eta_s.*X;

% determine which equation to use for the given temperatures
decide1 = T<800;
decide2 = T>=800;

eta = eta_1.*decide1 + eta_2.*decide2;
        

end


