function eta = visco_swSpivey(press,temp,sal)

% VISCO_SWSPIVEY -  calculates the dynamic viscosity of seawater 
%       at elevated pressure, temperature, and salinity
%
%       Usage: eta = visco_swSpivey(p,T,S)
%
%       Input parameters (scalars or vectors): 
%           p        - pressure [MPa]       (0 < p < 200)
%           T        - temperature [degC]   (0 < T < 275)
%           S        - practical salinity   (0 < S < 290) 
%
%       Output parameter:
%           eta      - dynamic viscosity [Pa*s]
%
%       References:
%       Spivey & McCain (2004) J.Can.Petrol.Tech. 43, 52-61.
%       Kestin et al. (1978) J.Chem.Eng.Data 23, 328-336.
%
% ------------------------------------------------------------------
%   written by: Elke Kossel, Matthias Haeckel (August 2009)
%               GEOMAR, Kiel, Germany
%   Disclaimer:
%   This software is provided "as is" without warranty of any kind.
% ------------------------------------------------------------------

%------------------------------------------------------------------------
% Check number of input parameters
%------------------------------------------------------------------------

if nargin < 3
     warning('VISCO_SWSpivey:Input','VISCO_SWSpivey: Missing input parameter! Please provide p,T and mNaCl!')
else

%------------------------------------------------------------------------
% Check validity of parameter range!
%------------------------------------------------------------------------

if ~any(press<0.1 | press>200) && ~any(temp<(0) | temp>275) && ~any(sal<(0) | sal>290)

else
    bad = find(press<0.1 | press > 200);
    press(bad)=NaN*ones(size(bad));
    bad = find(temp<(0) | temp>275);
    temp(bad)=NaN*ones(size(bad));
    bad = find(sal<(0) | sal>290);
    sal(bad)=NaN*ones(size(bad));

 warning('VISCO_SWSPIVEY:OutOfParameterRange','VISCO_SWSPIVEY: Input values out of range!')
 warning('VISCO_SWSPIVEY:OutOfParameterRange','Equations valid for p=0.1-200 MPa, T=0-275 degC, S=0-290.')
end

%------------------------------------------------------------------------
% Definition of constants, thermodynamic variables and parameters
%------------------------------------------------------------------------

 % calculate NaCl concentration with ionic strength
 % equivalent to given salinity
 mNaCl=i_sw(sal); 

[p, T, c] = ndgrid(press,temp,mNaCl);

A1 = 0.0173;
A2 = 0.068;
B1 = -1.0531;
B2 = 0.0273;

%------------------------------------------------------------------------
% Calculation of viscosity as formulated by Kestin et al
%------------------------------------------------------------------------

eta_K = visco_Kestin(p.*1e-3,T,c);

eta_K125 = visco_Kestin(p.*1e-3,125,c); % viscosity at 125 C

%------------------------------------------------------------------------
% Calculation of viscosity as formulated by Spivey et al
%------------------------------------------------------------------------

eta = eta_K125 .* exp((A2 .* p./100 + A1).* (log(T./125)).^2 + ...
    (B2 .* p ./ 100 + B1) .* log(T ./125));

decide = T < 125;

eta(decide) = eta_K(decide); % take values from Kestin's equation for T<125


 eta = eta.*1e-6; % [Pa s] -> [Pa s]
 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function eta_K = visco_Kestin(p,T,c)
% Calculation of viscosity as formulated by Kestin et al

%------------------------------------------------------------------------
% Definition of constants, thermodynamic variables and parameters
%------------------------------------------------------------------------

eta_20 = 1002; % viscosity at 20C [Pa s]

c_s = 6.044 + 0.28e-2 .* T + 0.36e-4 .* T.^2;

c_star = c./c_s;

A = 0.3324e-1 .*c + 0.3624e-2 .* c.^2 - 0.1879e-3 .* c.^3;

B = -0.396e-1 .* c + 0.102e-1 .* c.^2 - 0.702e-3 .* c.^3;

%------------------------------------------------------------------------
% Calculation of pressure correction factor for Kestin's equation
%------------------------------------------------------------------------

beta_w = -1.297 + 0.574e-1 .* T - 0.697e-3 .* T.^2 + 0.447e-5 .* T.^3 ...
    -0.105e-7 .* T.^4;

beta_e = 0.545 + 0.28e-2 .* T - beta_w;

beta_star = 2.5 .* c_star - 2 .* c_star.^2 + 0.5 .* c_star.^3;

beta = beta_e .* beta_star + beta_w;

%------------------------------------------------------------------------
% Calculation of viscosity 
%------------------------------------------------------------------------

log_etaw0 = log10(eta_20) + (20 - T).*(1.2378 - 1.303e-3 .*(20 - T) + ...
    3.06e-6 .* (20 - T).^2 + 2.55e-8 .* (20 - T).^3)./(96 + T);

log_eta0 = log_etaw0 + A + B .* (log_etaw0 - log10(eta_20));

eta_K = 10.^(log_eta0) .* (1 + beta .* p); 

